/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) Nigel Topham, and University of Edinburgh, 1996.     |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

%{
// $Id: FtnPrelude.in,v 1.2 1998/09/15 14:28:51 npt Exp $


/*----------------------------------------------------------+
 |                                                          |
 |                    Include files.                        |
 |                                                          |
 +----------------------------------------------------------*/

#if defined (sun) || defined (__DECCXX)
#  include <alloca.h>
#endif

#ifdef W32
#  define strcasecmp stricmp
#endif

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "Octave/Fortran.h"

// define DEBUG_NAMES to get a run-time analysis of name lookups

// #define DEBUG_NAMES	1

/*----------------------------------------------------------+
 |                                                          |
 | We use the C++ pre-processor to rename the standard      |
 | Bison/Yacc routines yylex(), yyerror() and yyparse()     |
 | so that they are members of the FortranParser class.     |
 |                                                          |
 +----------------------------------------------------------*/

#define YYDEBUG_VERBOSE 1
#define YYERROR_VERBOSE 1
#define YYINITDEPTH 32
#define YYMAXDEPTH 10000

#define yylex this->lexer
#define yyerror this->error
#define yyparse FortranParser::parser
#define yyoverflow FortranParser::stack_overflow

%}

/*----------------------------------------------------------+
 |                                                          |
 |         Token declarations for Bison Parser              |
 |                                                          |
 +----------------------------------------------------------*/

%token PERCENT 1001
%token AMPERSAND 1002
%token ASTER 1003
%token CLUSTER 1004
%token COLON 1005
%token COMMA 1006
%token DASTER 1007
%token DEFINED_OPERATOR 1008
%token DOT 1009
%token DQUOTE 1010
%token GLOBAL_A 1011
%token LEFTAB 1012
%token LEFTPAR 1013
%token MINUS 1014
%token PLUS 1015
%token POINT_TO 1016
%token QUOTE 1017
%token RIGHTAB 1018
%token RIGHTPAR 1019
%token SHARP 1020
%token AND 1021
%token DSLASH 1022
%token EQV 1023
%token EQ 1024
%token EQUAL 1025
%token FALSE_VAL 1026
%token GE 1027
%token GT 1028
%token LE 1029
%token LT 1030
%token NE 1031
%token NEQV 1032
%token NOT 1033
%token OR 1034
%token TRUE_VAL 1035
%token SLASH 1036
%token XOR 1037
%token REFERENCE 1038
%token ACCEPT 1039
%token ALIGN 1040
%token ALLOCATABLE 1041
%token ALLOCATE 1042
%token ARITHIF 1043
%token ASSIGN 1044
%token ASSIGNGOTO 1045
%token ASSIGNMENT 1046
%token ASSIGN_LOCK 1047
%token ASSOCIATE_VARIABLE 1048
%token BACKSPACE 1049
%token BAD_CCONST 1050
%token BAD_SYMBOL 1051
%token BEGIN_ORDER 1052
%token BEGIN_SECTION 1053
%token BEGIN_TASKS 1054
%token BLANK_STATEMENT 1055
%token BLOCK 1056
%token BLOCKDATA 1057
%token BLOCK_CYCLIC 1058
%token BYTE 1059
%token CALL 1060
%token CASE 1061
%token CDOACROSS 1062
%token CDOALL 1063
%token CHARACTER 1064
%token CHAR_CONSTANT 1065
%token CLEAR 1066
%token CLOOP 1067
%token CLOSE 1068
%token COMMENT 1069
%token COMMON 1070
%token COMPGOTO 1071
%token COMPLEX 1072
%token CONCURRENT 1073
%token CONSTRUCT_ID 1074
%token CONTAINS 1075
%token CONTINUE 1076
%token CRITICALSECTION 1077
%token CYCLE 1078
%token CYCLIC 1079
%token DATA 1080
%token DEALLOCATE 1081
%token DECODE 1082
%token DECOMPOSITION 1083
%token DEFAULT_CASE 1084
%token DEFINE 1085
%token DIMENSION 1086
%token DIRECTIVE 1087
%token DISTRIBUTE 1088
%token DOACROSS 1089
%token DOUBLECOMPLEX 1090
%token DOUBLEPRECISION 1091
%token DOWHILE 1092
%token DP_CONSTANT 1093
%token ELSE 1094
%token ELSEIF 1095
%token ELSEWHERE 1096
%token ENCODE 1097
%token ENDCDOACROSS 1098
%token ENDCDOALL 1099
%token ENDCLOOP 1100
%token ENDCONCURRENT 1101
%token ENDCRITICALSECTION 1102
%token ENDDO 1103
%token ENDDOACROSS 1104
%token ENDEXTEND 1105
%token ENDEXTENDED 1106
%token ENDFILE 1107
%token ENDIF 1108
%token ENDINTERFACE 1109
%token ENDMODULE 1110
%token ENDPARALLEL 1111
%token ENDPARALLELDO 1112
%token ENDPARALLELSECTIONS 1113
%token ENDPDO 1114
%token ENDPSECTIONS 1115
%token ENDSDOALL 1116
%token ENDSECTIONS 1117
%token ENDSELECT 1118
%token ENDSINGLEPROCESS 1119
%token ENDTYPE 1120
%token ENDUNIT 1121
%token ENDWHERE 1122
%token END_ORDER 1123
%token END_SECTION 1124
%token END_TASKS 1125
%token ENTRY 1126
%token EOLN 1127
%token EQUIVALENCE 1128
%token ERROR 1129
%token EVENT 1130
%token EXIST 1131
%token EXIT 1132
%token EXTEND 1133
%token EXTENDED 1134
%token EXTERNAL 1135
%token FIND 1136
%token FORALL 1137
%token FORALLDO 1138
%token FORCE_PARALLEL 1139
%token FORCE_PARALLEL_EXT 1140
%token FORCE_VECTOR 1141
%token FORMAT_TOKEN 1142
%token FREE_LOCK 1143
%token FUNCTION 1144
%token GATE 1145
%token GUARDS 1146
%token HEX_CONSTANT 1147
%token HOLLERITH 1148
%token IDENTIFIER 1149
%token IMPLICIT 1150
%token IMPLICITNONE 1151
%token IN 1152
%token INCLUDE 1153
%token INOUT 1154
%token INQUIRE 1155
%token INTEGER 1156
%token INTENT 1157
%token INTERFACE 1158
%token INTERFACEASSIGNMENT 1159
%token INTERFACEOPERATOR 1160
%token INTRINSIC 1161
%token INT_CONSTANT 1162
%token LABEL 1163
%token LABEL_DECLARE 1164
%token LET 1165
%token LOCK 1166
%token LOGICAL 1167
%token LOGICALIF 1168
%token LOOP 1169
%token MAXPARALLEL 1170
%token MAX_TRIPS 1171
%token MODULE 1172
%token NAMEEQ 1173
%token NAMELIST 1174
%token NEW_STATEMENT 1175
%token NEXT_TASK 1176
%token NO_PARALLEL 1177
%token NO_PEEL 1178
%token NO_PROMOTE_ALL 1179
%token NO_PROMOTE_TEST 1180
%token NO_RECURRENCE 1181
%token NO_SIDE_EFFECTS 1182
%token NO_VECTOR 1183
%token NULLIFY 1184
%token OCTAL_CONSTANT 1185
%token ON 1186
%token ONLY 1187
%token OPEN 1188
%token OPERATOR 1189
%token OPTIONAL 1190
%token OPTIONS 1191
%token ORDERED 1192
%token OTHERWISE 1193
%token OUT 1194
%token PARALLEL 1195
%token PARALLELDO 1196
%token PARALLELSECTIONS 1197
%token PARAMETER 1198
%token PAUSE 1199
%token PDO 1200
%token PEEL 1201
%token PEEL_ALL 1202
%token PLAINDO 1203
%token PLAINGOTO 1204
%token POINTER 1205
%token POINTERLET 1206
%token POST 1207
%token PREFER_PARALLEL 1208
%token PREFER_PARALLEL_EXT 1209
%token PREFER_VECTOR 1210
%token PRINT 1211
%token PRIVATE 1212
%token PROCEDURE 1213
%token PROCESS_CLUSTER 1214
%token PROCESS_GLOBAL 1215
%token PROGRAM 1216
%token PROMOTE_TEST 1217
%token PROMOTE_TEST_ALL 1218
%token PSECTIONS 1219
%token PSTRIP 1220
%token PUBLIC 1221
%token RANGE 1222
%token READ 1223
%token REAL 1224
%token REAL_CONSTANT 1225
%token RECORD 1226
%token RECURSIVE 1227
%token REDUCE 1228
%token RELEASE 1229
%token RESULT 1230
%token RETURN 1231
%token REWIND 1232
%token ROW_WISE 1233
%token SAVE 1234
%token SCALAR 1235
%token SCOMMON 1236
%token SDOALL 1237
%token SECTION 1238
%token SECTIONWAIT 1239
%token SELECT 1240
%token SEQUENCE 1241
%token SET 1242
%token SHORT_COMMENT 1243
%token SINGLEPROCESS 1244
%token SKIPPASTEOF 1245
%token STAT 1246
%token STOP 1247
%token SUBROUTINE 1248
%token SYNC 1249
%token SYNCH_PARALLEL 1250
%token TARGET 1251
%token TASK_CLUSTER 1252
%token TASK_COMMON 1253
%token TASK_GLOBAL 1254
%token THEN 1255
%token TO 1256
%token TRUNC 1257
%token TYPE 1258
%token TYPE_DECL 1259
%token UNKNOWN_TOK 1260
%token UNLOCK 1261
%token UNROLL 1262
%token USE 1263
%token VARIABLE 1264
%token VIRTUAL 1265
%token VSTRIP 1266
%token WAIT 1267
%token WHERE 1268
%token WHERE_ASSIGN 1269
%token WHILE 1270
%token WITH 1271
%token WRAP 1272
%token WRITE 1273


/*----------------------------------------------------------+
 |                                                          |
 |                Stack type declaration.                   |
 |                                                          |
 | %union {                                                 |
 |   int token ;                                            |
 |   statement_tag stmt_tag ;                               |
 |   expression_tag expr_tag ;                              |
 |   char charv ;                                           |
 |   int ival ;                                             |
 |   string *stringp ;                                      |
 |   PTR_STMT bf_node ;                                     |
 |   PTR_EXPR ll_node ;                                     |
 |   PTR_SYMB symbol ;                                      |
 |   PTR_TYPE data_type ;                                   |
 |   PTR_HASH hash_entry ;                                  |
 |   PTR_FTN_LABEL label ;                                  |
 | }                                                        |
 |                                                          |
 +----------------------------------------------------------*/


/*----------------------------------------------------------+
 |                                                          |
 |            Associativities and Precedences.              |
 |                                                          |
 +----------------------------------------------------------*/

%left EPSILON
%left IDENT
%left COMMA
%nonassoc COLON
%right EQUAL
%left DEFINED_OPERATOR
%left BINARY_OP
%left EQV NEQV
%left OR XOR
%left AND
%left NOT
%nonassoc LT GT LE GE EQ NE
%nonassoc UNARY_OP
%left DSLASH
%left PLUS MINUS
%left ASTER SLASH
%right DASTER
%left LEFTPAR
%right RIGHTPAR
%left HYPER

/*----------------------------------------------------------+
 |                                                          |
 |            Basic parser characteristics                  |
 |            ============================                  |
 |                                                          |
 |      (a). uses pure function definition for yylex()      | 
 |                                                          |
 |      (b). start symbol is "program"                      |
 |                                                          |
 +----------------------------------------------------------*/

%pure_parser
%expect 11
%start program


/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |                    (BASIC FORTRAN 77)                    |
 |                                                          |
 +----------------------------------------------------------*/

%type <stringp> CHAR_CONSTANT HOLLERITH REAL_CONSTANT DP_CONSTANT
%type <stringp> IDENTIFIER COMMENT SHORT_COMMENT NAMEEQ HEX_CONSTANT OCTAL_CONSTANT
%type <stringp> DEFINED_OPERATOR CONSTRUCT_ID
%type <ival> INT_CONSTANT LABEL
%type <token> att_type
%type <stmt_tag> stop
%type <expr_tag> addop
%type <charv> letter
%type <hash_entry> name
%type <symbol>  progname blokname args arg arglist call comblock
%type <bf_node> stat spec entry exec iffable goto logif
%type <bf_node> dcl implicit data common dimension external intrinsic attrib
%type <bf_node> equivalence namelist type_dcl end_type
%type <bf_node> intent optional public private allocatable pointer target 
%type <ll_node> implist data1
%type <ll_node> paramlist paramitem alloc_list
%type <ll_node> dim dims dimlist ubound funarglist funarg funargs opt_expr var
%type <ll_node> labellist expr uexpr lhs simple_const lengspec substring
%type <ll_node> complex_const vec triplet 
%type <ll_node> dospec use_name_list 
%type <symbol>  do_var
%type <label>   dotarget
%type <symbol>  funcname typedfunc procname
%type <ll_node> equivset equivlist
%type <ll_node> savelist saveitem

/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |               (FORTRAN I/O STATEMENTS)                   |
 |                                                          |
 +----------------------------------------------------------*/

%type <bf_node> io read write print iofmove iofctl fmkwd ctlkwd 
%type <ll_node> fexpr unpar_fexpr 
%type <ll_node> ioctl ctllist ioclause nameeq inlist inelt
%type <ll_node> outlist out2 other infmt

%type <ll_node> label letgroups letgroup let
%type <ll_node> callarglist callarg
%type <data_type> typename typespec type imptype
%type <label> thislabel


/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |                 (FORTRAN 90 EXTENSIONS)                  |
 |                                                          |
 +----------------------------------------------------------*/

%type <hash_entry> defined_op, intrinsic_op, operator
%type <bf_node> case, whereable, interface, use_stat
%type <bf_node> module_proc_stmt
%type <ll_node> opt_result_clause case_selector case_value_range
%type <ll_node> case_value_range_list, proper_lengspec
%type <symbol>  opt_construct_name 
%type <symbol>  module_name
%type <ll_node> selector, kind_selector initial_value
%type <ll_node> options, attr_spec_list, attr_spec, intent_spec, access_spec
%type <ll_node> string_constant, opt_substring, structure_component
%type <ll_node> array_ele_substring_func_ref, ident
%type <ll_node> asubstring, equi_object
%type <ll_node> allocation_list, allocate_object_list, opt_stat_spec
%type <ll_node> allocate_object, pointer_name_list, rename_list, rename_name
%type <ll_node> only_list, only_name, proc_name_list, opt_construct_name_colon

/* Obsolete non-terminals - kept only for the record - delete at next release */

/* %type <ll_node> subscript_list, array_element */


/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |                  (FORTRAN-D EXTENSIONS)                  |
 |                                                          |
 +----------------------------------------------------------*/

/*-------------------- Currently Unused ---------------------

%type <bf_node> decomposition, forall
%type <ll_node> on_clause,

------------------------------------------------------------*/


/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |                 (PCF FORTRAN EXTENSION)                  |
 |                                                          |
 +----------------------------------------------------------*/

/*-------------------- Currently Unused ---------------------

%type <bf_node> pcf parallel_do pregion_do unstruc_lock_sync
%type <bf_node> critical_section event_sync sequence_sync sync_performance
%type <bf_node> data_sharing  single_process_section
%type <bf_node> pcf_dcl sync_dcl parallel_sections parallel_region pcf_common
%type <ll_node> opt_comma_expr qual qual_list opt_qualities name_list
%type <ll_node> object object_name_list
%type <ll_node> section_name section_name_list opt_r_list gate opt_gate
%type <ll_node> opt_gaurd_list event seq opt_seq_exprs34
%type <ll_node> sync_list r_option r_list opt_p_list p_list p_option

------------------------------------------------------------*/

/*----------------------------------------------------------+
 |                                                          |
 |        Result types for non-terminal productions         |
 |                                                          |
 |            (CONVEX EXTENSIONS and DIRECTIVES)            |
 |                                                          |
 +----------------------------------------------------------*/

/*--------- currently disabled in standard Octave release ---
%type <bf_node> cvx
%type <bf_node> cvx_options cvx_accept cvx_decode cvx_encode
%type <bf_node> cvx_type cvx_find
%type <bf_node> directive
%type <ll_node> cvx_opt_list cvx_opt cvx_fmt cvx_trip
%type <ll_node> dirlist diroption dir_arg_list dir_arg 
------------------------------------------------------------*/
%%


/*----------------------------------------------------------+
 |                                                          |
 |                 Production Rules                         |
 |                                                          |
 |              (STANDARD FORTRAN 77)                       |
 |                                                          |
 +----------------------------------------------------------*/

program:   statlist
            { 
	      YYACCEPT ;
	    }
        ;

statlist:  
	   stat NEW_STATEMENT
            { 
	      attach_statement ($1) ;
	    } 

        |  statlist stat NEW_STATEMENT
	    { 
	      attach_statement ($2) ;
	    }

	|  stat short_cmnt NEW_STATEMENT
            { 
	      attach_statement ($1) ;
	    } 

        |  statlist stat short_cmnt NEW_STATEMENT
	    { 
	      attach_statement ($2) ;
	    }
	;


short_cmnt: SHORT_COMMENT
	    {
	      /* create the comment and append to the list */
	      Comment *p ;
	      
	      p = make_comment ($1, HALF) ;
	      if (!stmt_comments)
		stmt_comments = cur_comment = p ;
	      else
		cur_comment = cur_comment->next = p ;
	    }
	;

stat:	  thislabel entry
	    { 
	      lastwasbranch = NO ;
	      $$ = $2 ;
	    }

	| thislabel spec
            {
	      if ($2 != 0) 
	      {	    
		$2->label.ftn = $1 ;
		$$ = $2 ;
	      }
	    }

	| thislabel exec
            { PTR_STMT p ;
  
	      // if (lastwasbranch && ! thislabel)
	      //   warn ("statement cannot be reached") ;
	      lastwasbranch = thiswasbranch ;
	      thiswasbranch = NO ;
	      if ($2) $2->label.ftn = $1 ;
	      if ($1) {
		$1->statbody = $2;
		if ($1->labtype == LABFORMAT)
		  error ("label already that of a format");
		else
		  $1->labtype = LABEXEC;
	      }
	      for (p = pred_stmt; $1 && (p->variant() == FOR_NODE) &&
		   (p->node.for_node.doend) &&
		   (p->node.for_node.doend->stateno == $1->stateno);
		   p = p->control_parent)
                ++end_group;
	      $$ = $2;
	    }

	| thislabel INCLUDE string_constant
            {
	      /* doinclude (*$3) ; */
	      
	      $$ = make_stmt (INCLUDE_STAT, 0, $3, 0, 0) ;
	    }

	| thislabel UNKNOWN_TOK
	    {
	      error ("unclassifiable statement");
	      /* flline(); */
	      $$ = 0;
	    } 

	| COMMENT
	    { 
	      Comment *p ;
	      
	      p = make_comment ($1, FULL) ;
	      if (!stmt_comments)
		stmt_comments = cur_comment = p ;
	      else
		cur_comment = cur_comment->next = p ;
	      $$ = 0 ;
	    }

/*---------------- Convex Fortran Disabled -----------------
        | directive
           {
	     $$ = $1 ;
	   }
-----------------------------------------------------------*/

	| error
            {
	      /* flline() ; */
	      needkwd = NO ;
	      inioctl = NO ; 
	      /* yyerrok ; */
	      yyclearin ;
	      $$ = 0 ;
	    }
	;

thislabel:  LABEL
	    {
	      yystno = $1 ;
	      if (yystno)
	      {
		$$ = thislabel = make_label (yystno) ;
		if (thislabel->labdefined)
		  erri ("label %d already defined", thislabel->stateno) ;
		else
		  thislabel->labdefined = YES ;
	      }
	      else
		$$ = thislabel = 0 ;
	    }
	;

entry:	  PROGRAM new_prog progname
	    {
	      if (pred_stmt != root_stmt)
		error ("misplaced PROGRAM statement") ;

	      $$ = make_stmt (PROG_HEDR, $3, 0, 0, 0) ;
	    }

	| BLOCKDATA new_prog blokname
	    {
	      $3->variant (PROCEDURE_NAME) ;
	      $3->decl = YES ;
	      $$ = make_stmt (BLOCK_DATA, $3, 0, 0, 0) ;
	    }

	| SUBROUTINE new_prog procname arglist
	    { 
              if (pred_stmt != root_stmt)
	         error ("misplaced SUBROUTINE statement") ;

	      $3->install_param_list ($4, 0, PROCEDURE_NAME) ;
	      $$ = make_stmt (PROC_HEDR, $3, 0, 0, 0) ;
	    }

	| RECURSIVE needkeyword SUBROUTINE new_prog procname arglist
	    { 
              if (pred_stmt != root_stmt)
	         error ("misplaced SUBROUTINE statement") ;

	      $5->install_param_list ($6, 0, PROCEDURE_NAME) ;
	      $$ = make_stmt (PROC_HEDR, $5, 0, 0, 0) ;
	    }

	| FUNCTION new_prog funcname arglist opt_result_clause
            {
	      if (pred_stmt != root_stmt)
		error ("misplaced FUNCTION statement") ;

              $3->install_param_list ($4, $5, FUNCTION_NAME) ;  
	      $$ = make_stmt (FUNC_HEDR, $3, 0, 0, 0) ;
  	      $$->node.fields.e1 = $5 ;
	      $$->idx = 0 ;
            }

	| typedfunc arglist opt_result_clause
	    {
	      if (pred_stmt != root_stmt)
		error ("misplaced FUNCTION statement") ;

              $1->install_param_list ($2, $3, FUNCTION_NAME); 
	      $$ = make_stmt (FUNC_HEDR, $1, 0, 0, 0) ;
	      $$->node.fields.e1 = $3;
	      $$->idx = 1 ;
	    }

	| ENTRY name arglist
            { 
	      PTR_STMT stmt ;
	      PTR_SYMB q ;
	      PTR_EXPR l ;
	      
	      if (parstate == OUTSIDE)
		error ("misplaced entry statement") ;

	      stmt = cur_proc () ;

	      if (stmt->stmt_variant() == FUNC_HEDR) {
	        q = make_function ($2, stmt->node.fields.symbol->type, YES) ;
	        l = q->construct_entry_list ($3, FUNCTION_NAME) ;
	      }
	      else if ((stmt->variant() == PROC_HEDR) || 
		       (stmt->variant() == PROG_HEDR)) {
		q = make_procedure ($2, YES) ;
		l = q->construct_entry_list ($3, PROCEDURE_NAME) ; 
	      }
	      $$ = make_stmt (ENTRY_STAT, q, l, 0, 0) ;
	    }

	| MODULE name
	    { PTR_SYMB s;
	      PTR_STMT p;

	      s = make_global_entity ($2, MODULE_NAME, global_default, NO) ;
	      s->decl = YES ;
	      $$ = make_stmt (MODULE_STMT, s, 0, 0, 0) ;
	      s->node.fields.func_hedr = $$ ;
	      position = IN_MODULE ;
            }
	;

new_prog:   { newprog () ;
	      if (position == IN_OUTSIDE)
	          position = IN_PROC ;
              else if (position != IN_INTERNAL_PROC)
	         position-- ;
              else
		error ("internal procedures cannot contain procedures") ;

	    }
	;


procname:  name
           {
	      $$ = make_procedure($1, LOCAL) ;
	      $$->decl = YES ;
           }        

funcname:  name
           { 
	     $$ = make_function($1, 0, LOCAL) ;
	     $$->decl = YES ;
	   }        

typedfunc: type FUNCTION new_prog name 
           { 
	     $$ = make_function ($4, $1, LOCAL) ;
	     $$->decl = YES ;
	   }
     
        | type needkeyword RECURSIVE FUNCTION new_prog name
           {
	     $$ = make_function ($6, $1, LOCAL) ;
	     $$->decl = YES ;
	     $$->attr = $$->attr | RECURSIVE_BIT ;
	   }

	| RECURSIVE needkeyword FUNCTION new_prog name
           {
	     $$ = make_function ($5, 0, LOCAL) ;
	     $$->decl = YES ;
	     $$->attr = $$->attr | RECURSIVE_BIT ;
	   }

	| RECURSIVE needkeyword type FUNCTION new_prog name
           {
	     $$ = make_function ($6, $3, LOCAL) ;
	     $$->decl = YES ;
	     $$->attr = $$->attr | RECURSIVE_BIT ;
	   }
	;

opt_result_clause: needkeyword keywordoff
	    { $$ = 0; }
	| needkeyword RESULT name
	    { PTR_SYMB s;

              s = make_scalar($3, 0, LOCAL);
              $$ = make_expr ( VAR_REF, 0, 0, s);
            }
	;

name:	  IDENTIFIER
	    { $$ = cur_scope()->look_up_ftn_sym ((*($1))()); }
	;

progname:   { $$ = make_program(cur_scope()->look_up_ftn_sym ("MAIN")); }
	| name
	    {
              $$ = make_program($1);
	      $$->decl = YES;   /* variable declaration has been seen. */
            }
	;

blokname:   { $$ = make_program(cur_scope()->look_up_ftn_sym ("BLOCK")); }
	| name
	    {
              $$ = make_program($1); 
	      $$->decl = YES;   /* variable declaration has been seen. */
	    }
	;

arglist:
	    { $$ = 0; }
	| LEFTPAR RIGHTPAR
	    { $$ = 0; }
	| LEFTPAR arg_scope args normal_scope RIGHTPAR
	    { $$ = $3 ; }
	;

args:	  arg
	| args COMMA arg
	    { $$ = $1->append_id_list($3); }
	;

arg:	  name
	    {
	      $$ = make_scalar($1, 0, IO);
            }
	| ASTER
	    { $$ = star_symb; }
	;

arg_scope:
            {
	      push_arg_scope () ;
	    }
         ;

normal_scope:
            {
	      pop_normal_scope () ;
	    }
         ;

needkeyword: 
            { needkwd = YES; }
         ;

keywordoff: 
            { needkwd = NO; }
         ;

/* The scanner checks if the keyword is ONLY. */
keyword_if_colon_follow: 
            { colon_flag = YES; }
         ;


/*----------------------------------------------------------+
 |                                                          |
 |               Grammar for DECLARATIONS                   |
 |                                                          |
 +----------------------------------------------------------*/

spec:	  type_dcl
	| end_type
	| dcl
/*
	| pcf_dcl
*/
	| common
/*
        | pcf_common
*/
        | dimension
	| external
	| intrinsic
	| equivalence
	| implicit
	| attrib
	| namelist
/*
	| identify
*/
	| data
	| SAVE in_dcl
	    {
	      saveall = YES;
	      $$ = make_stmt (SAVE_DECL, 0, 0, 0, 0);
	    }
	| SAVE in_dcl savelist
	    {
	      $$ = make_stmt (SAVE_DECL, 0, $3, 0, 0);
            }

	| FORMAT_TOKEN string_constant
            {
	      $$ = make_stmt (FORMAT_STAT, 0, $2, 0, 0);
	    }

	| PARAMETER in_dcl LEFTPAR paramlist RIGHTPAR
	    { $$ = make_stmt (PARAM_DECL, 0, $4, 0, 0); }
	| intent
	| optional
	| public
	| private
	| allocatable
	| pointer
	| target
	| interface
	| use_stat
	| module_proc_stmt
/*
	| decomposition
*/
	;

interface: INTERFACE in_dcl
	    { $$ = make_stmt (INTERFACE_STMT, 0, 0, 0, 0); }
	| INTERFACE in_dcl name
	    { PTR_SYMB s;

	      s = make_procedure($3, LOCAL);
	      s->variant(INTERFACE_NAME) ;
	      $$ = make_stmt (INTERFACE_STMT, s, 0, 0, 0);
	    }
	| INTERFACEOPERATOR in_dcl LEFTPAR operator RIGHTPAR 
	    { PTR_SYMB s;

	      s = make_function($4, global_default, LOCAL);
	      s->variant(INTERFACE_NAME) ;
	      $$ = make_stmt (INTERFACE_STMT, s, 0, 0, 0);
	    }
	| INTERFACEASSIGNMENT in_dcl LEFTPAR EQUAL RIGHTPAR
	    { PTR_SYMB s;


	      s = make_procedure(cur_scope()->look_up_ftn_sym ("="), LOCAL);
	      s->variant(INTERFACE_NAME) ;
	      $$ = make_stmt (INTERFACE_STMT, s, 0, 0, 0);
	    }
	| ENDINTERFACE in_dcl
	    { $$ = make_stmt (CONTROL_END, 0, 0, 0, 0); 
	      process_interface($$);
	    }
	;

defined_op: DEFINED_OPERATOR
	    { $$ = cur_scope()->look_up_ftn_sym ((*$1)()); }
	;

operator: defined_op
	    { $$ = $1; }
	| intrinsic_op
	    { $$ = $1; }
	;

intrinsic_op: PLUS
	    { $$ = cur_scope()->look_up_op(PLUS); }
	| MINUS
	    { $$ = cur_scope()->look_up_op(MINUS); }
	| ASTER
	    { $$ = cur_scope()->look_up_op(ASTER); }
	| DASTER
	    { $$ = cur_scope()->look_up_op(DASTER); }
	| SLASH
	    { $$ = cur_scope()->look_up_op(SLASH); }
	| DSLASH
	    { $$ = cur_scope()->look_up_op(DSLASH); }
	| AND
	    { $$ = cur_scope()->look_up_op(AND); }
	| OR
	    { $$ = cur_scope()->look_up_op(OR); }
	| XOR
	    { $$ = cur_scope()->look_up_op(XOR); }
	| NOT
	    { $$ = cur_scope()->look_up_op(NOT); }
	| EQ
	    { $$ = cur_scope()->look_up_op(EQ); }
	| NE
	    { $$ = cur_scope()->look_up_op(NE); }
	| GT
	    { $$ = cur_scope()->look_up_op(GT); }
	| GE
	    { $$ = cur_scope()->look_up_op(GE); }
	| NEQV
	    { $$ = cur_scope()->look_up_op(NEQV); }
	| EQV
	    { $$ = cur_scope()->look_up_op(EQV); }
	;


type_dcl: TYPE_DECL name
	   { PTR_EXPR q;
             PTR_SYMB s;
         
             type_var = s = make_derived_type($2, 0, LOCAL);	
             $$ = make_stmt (STRUCT_DECL, s, 0, 0, 0);
             push_scope_level ($$, NO);
	   }
	| TYPE_DECL COMMA needkeyword access_spec opt_double_colon name
	   { PTR_SYMB s;
         
             type_var = s = make_derived_type($6, 0, LOCAL);	
	     s->attr = s->attr | type_options;
             $$ = make_stmt (STRUCT_DECL, s, $4, 0, 0);
             push_scope_level ($$, NO);
	   }
	;

end_type: ENDTYPE in_dcl
	   {
	     $$ = make_stmt (CONTROL_END, 0, 0, 0, 0);
	     if (type_var != 0)
               type_var->process_type ();
             type_var = 0;
	     pop_scope_level (pred_stmt) ;
           }
	| ENDTYPE in_dcl name
	   {
             $$ = make_stmt (CONTROL_END, 0, 0, 0, 0);
	     if (type_var != 0)
               type_var->process_type ();
             type_var = 0;
	     pop_scope_level (pred_stmt);	
           }
	;

dcl:	 type options in_dcl name dims lengspec initial_value //fgg swapped in_dcl and name
	    { 
	      PTR_EXPR q, r, l, d = $5 ;
	      PTR_SYMB s;
	      PTR_TYPE t;
	      int type_opts;

	      vartype = $1;

	      if (d && $6 && vartype->type_variant() != T_STRING)
		error ("array length specifier misplaced") ;

              t = make_type_node (vartype, $6);
	      type_opts = type_options;

	      if (d && (type_opts & DIMENSION_BIT))
		{
		  errstr ("dimension of '%s' declared twice", $4->identifier()) ;
		  d = 0 ;
		}

	      if (d) type_opts = type_opts | DIMENSION_BIT;

	      if (d)
		 q = deal_with_options($4, t, type_opts, d, ndim, 
				       $7, ENTITY_DECL);
	      else 
		 q = deal_with_options($4, t, type_opts, attr_dims, attr_ndim,
				       $7, ATTRIB_DECL);

	      r = make_expr ( EXPR_LIST, q, 0, 0);
	      l = make_expr ( TYPE_OP, 0, 0, 0);
	      l->type = vartype;
	      $$ = make_stmt (VAR_DECL, 0, r, l, $2);
	    }
	| dcl COMMA name dims lengspec initial_value
	    { 
	      PTR_EXPR q, r, d = $4 ;
	      PTR_SYMB s;
              PTR_TYPE t;
	      int type_opts;
              
	      if (d && $5 && vartype->type_variant() != T_STRING)
		error ("array length specifier misplaced") ;

              t = make_type_node (vartype, $5);
	      type_opts = type_options;

	      if (d && (type_opts & DIMENSION_BIT))
		{
		  errstr ("dimension of '%s' declared twice", $3->identifier()) ;
		  d = 0 ;
		}

	      if (d) type_opts = type_opts | DIMENSION_BIT;

	      if (d)
		 q = deal_with_options($3, t, type_opts, d, ndim,
				       $6, ENTITY_DECL) ;
	      else 
		 q = deal_with_options($3, t, type_opts, attr_dims, attr_ndim,
				       $6, ATTRIB_DECL) ;

	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append (r) ;
       	    }

options: 
	    { $$ = 0; }
	| COLON COLON
            { $$ = 0; }
	| COMMA needkeyword attr_spec_list COLON COLON
	    { $$ = $3; }
	;

attr_spec_list:  attr_spec
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| attr_spec_list COMMA needkeyword attr_spec
	    { $$ = set_ll_list ($1, $4, EXPR_LIST); }
	;

attr_spec: PARAMETER
	    { 
	      if (type_options & PARAMETER_BIT)
		{
		  error ("multiple PARAMETER attributes") ;
		}
	      type_options = type_options | PARAMETER_BIT; 
              $$ = make_expr ( PARAMETER_OP, 0, 0, 0);
            }
	| access_spec
            { $$ = $1; }
	| ALLOCATABLE
	    { 
	      if (type_options & ALLOCATABLE_BIT)
		{
		  error ("multiple ALLOCATABLE attributes") ;
		}
	      type_options = type_options | ALLOCATABLE_BIT;
              $$ = make_expr ( ALLOCATABLE_OP, 0, 0, 0);
	    }
	| DIMENSION dims
	    { 
	      if (type_options & DIMENSION_BIT)
		{
		  error ("multiple DIMENSION attributes") ;
		}
	      type_options = type_options | DIMENSION_BIT;
	      attr_ndim = ndim;
	      attr_dims = $2;
              $$ = make_expr ( DIMENSION_OP, $2, 0, 0);
            }
	| EXTERNAL
	    { 
	      if (type_options & EXTERNAL_BIT)
		{
		  error ("multiple EXTERNAL attributes") ;
		}
	      type_options = type_options | EXTERNAL_BIT;
              $$ = make_expr ( EXTERNAL_OP, 0, 0, 0);
            }
	| INTENT LEFTPAR intent_spec RIGHTPAR
	    { 
	      if (type_options & (IN_BIT | OUT_BIT | INOUT_BIT))
		{
		  error ("multiple INTENT attributes") ;
		}
	      $$ = $3; 
	    }
	| INTRINSIC
	    { 
	      if (type_options & INTRINSIC_BIT)
		{
		  error ("multiple INTRINSIC attributes") ;
		}
	      type_options = type_options | INTRINSIC_BIT;
              $$ = make_expr ( INTRINSIC_OP, 0, 0, 0);
            }
	| OPTIONAL
	    { 
	      if (type_options & OPTIONAL_BIT)
		{
		  error ("multiple OPTIONAL attributes") ;
		}
	      type_options = type_options | OPTIONAL_BIT;
              $$ = make_expr ( OPTIONAL_OP, 0, 0, 0);
            }
	| POINTER
	    { 
	      if (type_options & POINTER_BIT)
		{
		  error ("multiple POINTER attributes") ;
		}
	      type_options = type_options | POINTER_BIT;
              $$ = make_expr ( POINTER_OP, 0, 0, 0);
            }
	| SAVE
	    { 
	      if (type_options & SAVE_BIT)
		{
		  error ("multiple SAVE attributes") ;
		}
	      type_options = type_options | SAVE_BIT; 
              $$ = make_expr ( SAVE_OP, 0, 0, 0);
            }
	| TARGET
	    { 
	      if (type_options & TARGET_BIT)
		{
		  error ("multiple TARGET attributes") ;
		}
	      type_options = type_options | TARGET_BIT; 
              $$ = make_expr ( TARGET_OP, 0, 0, 0);
            }
	;

intent_spec: needkeyword IN
	    { type_options = type_options | IN_BIT; 
              $$ = make_expr ( IN_OP, 0, 0, 0);
            }
        | needkeyword OUT
	    { type_options = type_options | OUT_BIT; 
              $$ = make_expr ( OUT_OP, 0, 0, 0);
            }
        | needkeyword INOUT
	    { type_options = type_options | INOUT_BIT; 
              $$ = make_expr ( INOUT_OP, 0, 0, 0);
            }
	;

access_spec: PUBLIC
	    { type_options = type_options | PUBLIC_BIT; 
              $$ = make_expr ( PUBLIC_OP, 0, 0, 0);
            }
	| PRIVATE
	    { type_options = type_options | PRIVATE_BIT;
              $$ = make_expr ( PRIVATE_OP, 0, 0, 0);
            }
        ;

intent:	 INTENT in_dcl LEFTPAR intent_spec RIGHTPAR opt_double_colon name
	    { 
	      PTR_EXPR q, r;
	      PTR_SYMB s;

              s = make_scalar($7, 0, LOCAL);
	      s->attr = type_options;	
	      q = make_expr (VAR_REF, 0, 0, s);
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (INTENT_STMT, 0, r, 0, 0);
	    }
	| intent COMMA name 
	    { 
	      PTR_EXPR q, r;
	      PTR_SYMB s;

              s = make_scalar($3, 0, LOCAL);	
	      s->attr = type_options;
	      q = make_expr (VAR_REF, 0, 0, s);
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append (r) ;
  	    }
	;

optional: OPTIONAL in_dcl opt_double_colon name
	    { 
	      PTR_EXPR q, r;
	      PTR_SYMB s;

              s = make_scalar($4, 0, LOCAL);	
	      s->attr = s->attr | OPTIONAL_BIT;
	      q = make_expr (VAR_REF, 0, 0, s);
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (OPTIONAL_STMT, 0, r, 0, 0);
	    }
	| optional COMMA name 
	    { 
	      PTR_EXPR q, r;
	      PTR_SYMB s;

              s = make_scalar($3, 0, LOCAL);	
	      s->attr = s->attr | OPTIONAL_BIT;
	      q = make_expr (VAR_REF, 0, 0, s);
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append (r) ;
  	    }
	;

private: PRIVATE in_dcl
	    {
	      saveall = YES;
	      $$ = make_stmt (PRIVATE_STMT, 0, 0, 0, 0);
	    }
	| PRIVATE in_dcl use_name_list
	    {
	      type_options = type_options | PRIVATE_BIT;
	      $$ = make_stmt (PRIVATE_STMT, 0, $3, 0, 0);
            }
        ;

public: PUBLIC in_dcl
	    {
	      saveall = YES;
	      $$ = make_stmt (PUBLIC_STMT, 0, 0, 0, 0);
	    }
	| PUBLIC in_dcl use_name_list
	    {
	      type_options = type_options | PUBLIC_BIT;
	      $$ = make_stmt (PUBLIC_STMT, 0, $3, 0, 0);
            }


type:	  typespec intonlyon selector intonlyoff
	    {
	      if ($3 && $1 == global_byte)
		err ("BYTE cannot have a selector") ;
	      type_options = 0;
	      attr_dims = 0 ;
              $$ = make_type_node ($1, $3);
            }
	| TYPE LEFTPAR name RIGHTPAR
	    { PTR_TYPE t;

	      type_options = 0;
	      attr_dims = 0 ;
              t = lookup_type ($3) ;
	      vartype = t;
	      $$ = make_type_node (t, 0);
            }
	;

attrib:   att_type name

	    { PTR_TYPE p;
	      PTR_EXPR q;
	      PTR_SYMB s;

	      if (s = $2->id_attr)
		   s->attr = $1;
	      else {
		p = undeftype ? global_unknown : impltype[*($2->identifier()) - 'a'];
                s = install_entry($2, SOFT);
		s->attr = $1;
                s->set_type(p, LOCAL);
	      }
	      q = make_expr (VAR_REF, 0, 0, $2->id_attr);
	      q = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (ATTR_DECL, 0, q, 0, 0);
	    }

	| attrib COMMA name

	    { PTR_TYPE p;
	      PTR_EXPR q, r;
	      PTR_SYMB s;
	      int att;

	      att = $1->node.fields.e1->node.fields.e1->
		    node.fields.symbol->attr;
	      if (s = $3->id_attr)
		   s->attr = att;
	      else {
		p = undeftype ? global_unknown : impltype[*($3->identifier()) - 'a'];
                s = install_entry($3, SOFT);
		s->attr = att;
                s->set_type(p, LOCAL);
	      }
	      q = make_expr (VAR_REF, 0, 0, $3->id_attr);
	      for (r = $1->node.fields.e1;
		   r->node.list.next;
		   r = r->node.list.next) ;
	      r->node.list.next = make_expr (EXPR_LIST, q, 0, 0);

	    }
	;

att_type: GLOBAL_A
	 { $$ = ATT_GLOBAL; }
	| CLUSTER
	 { $$ = ATT_CLUSTER; }
	;

/*
opt_attr:
	| TASK_GLOBAL
	| PROCESS_GLOBAL
	| TASK_CLUSTER
	| PROCESS_CLUSTER
	;
*/
typespec:  typename
		{
/*		  varleng = ($1<0 || $1==TYLONG ? 0 : typesize[$1]); */
		  vartype = $1;
		}

	;

typename: INTEGER	{ $$ = global_int; }
	| REAL		{ $$ = global_float; }
	| COMPLEX	{ $$ = global_complex; }
	| DOUBLEPRECISION { $$ = global_double; }
	| DOUBLECOMPLEX	{ $$ = global_complex; }
	| LOGICAL	{ $$ = global_bool; }
	| CHARACTER	{ $$ = global_string; }
	| BYTE          { $$ = global_byte ; }   /* CVX Extension */

/*      PCF FORTRAN's types.                     */
        | GATE          { $$ = global_gate; }
        | EVENT         { $$ = global_event; }
        | SEQUENCE      { $$ = global_sequence; }
	;

lengspec:
	    { $$ = 0; }
	| proper_lengspec
	    { $$ = $1; }
	;

proper_lengspec: ASTER expr 
	    { $$ = make_expr ( LEN_OP, $2, 0, 0); }
	| ASTER  LEFTPAR ASTER RIGHTPAR 
	    { Expression *l, *p ;

              l = make_expr ( KEYWORD_VAL, 0, 0, 0) ;
              l->node.string_val = new string (star_string) ;
	      p = make_expr ( PAREN_OP, l, 0, 0) ;
              $$ = make_expr ( LEN_OP, p, 0, 0) ;
            }
	;

selector:
	    { $$ = 0; }
 	| proper_lengspec
	    { $$ = $1; } 
        | LEFTPAR in_ioctl kind_selector RIGHTPAR 
	    { 
	      $$ = make_expr ( PAREN_OP, $3, 0, 0) ;
	      endioctl () ;
	    }
/*
        | LEFTPAR clause COMMA clause RIGHTPAR 
	    { PTR_EXPR l;

              l = make_expr ( CONS, $2, $4, 0);
	      $$ = make_expr ( PAREN_OP, l, 0, 0);
            }
*/
	;

kind_selector: nameeq expr
	    {
	      char *q = $1->node.string_val->operator()() ;

	      if (strcasecmp(q, "kind"))
		{
		  error ("kind selector expected") ;
		  $$ = 0 ;
		}
	      else
		$$ = make_expr ( SPEC_PAIR, $1, $2, 0) ;
	    }
	;

initial_value: 
	    { $$ = 0; }
	| EQUAL expr
	    { $$ = $2; }
	;

dimension: DIMENSION in_dcl opt_comma name dims lengspec
	    { 
	      PTR_SYMB s;
	      PTR_EXPR q, r;

	      if (! $5) {
		error ("no dimensions in Dimension statement");
		$$ = 0;
	      }
	      s = make_array($4, 0, $5, ndim, LOCAL);
	      s->attr = s->attr | DIMENSION_BIT;
	      q = make_expr (ARRAY_REF, $5, 0, s);
	      s->type->node.ar_decl.ranges = $5;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (DIM_STAT, 0, r, 0, 0);
	    }
	| dimension COMMA name dims lengspec
           {  
	     PTR_SYMB s;
	     PTR_EXPR q, r;

	     if (! $4) {
	       error ("no dimensions in Dimension statement");
	       $$ = 0;
	     }
	     s = make_array($3, 0, $4, ndim, LOCAL);
	     s->attr = s->attr | DIMENSION_BIT;
	     q = make_expr (ARRAY_REF, $4, 0, s);
	     s->type->node.ar_decl.ranges = $4;
	     r = make_expr (EXPR_LIST, q, 0, 0);
	     $1->node.fields.e1->append (r) ;
	   }
        ;

/* NOTE: Convex ALLOCATABLE has brackets around the name list
         and can omit the 'dims' part of the specification - in 
	 which case it must have been supplied in a type definition
         statement already.
*/

allocatable: ALLOCATABLE in_dcl LEFTPAR alloc_list RIGHTPAR
            {
	      $$ = make_stmt (ALLOCATABLE_STMT, 0, $4, 0, 0) ;
	      $$->note_transform (make_transform (ALLOC_PAREN, $$)) ;
	    }
         ;

alloc_list: name dims lengspec
	    { PTR_SYMB s;
	      PTR_EXPR q, r;

	      s = make_array($1, 0, $2, ndim, LOCAL);
	      s->attr = s->attr | ALLOCATABLE_BIT;
	      q = make_expr (ARRAY_REF, $2, 0, s);

	      if ($2)
		{
		  if (s->type->node.ar_decl.ranges)
		    error ("duplicate dimension in Allocatable statement") ;
		  else
		    s->type->node.ar_decl.ranges = $2 ;
		}
	      else
		if (! (s->type->node.ar_decl.ranges))
		  error ("no dimension in Allocatable statement") ;

	      $$ = make_expr (EXPR_LIST, q, 0, 0);
	    }
	| alloc_list COMMA name dims lengspec
           {
	     PTR_SYMB s;
	     PTR_EXPR q, r;
	     
	     s = make_array($3, 0, $4, ndim, LOCAL);
	     s->attr = s->attr | ALLOCATABLE_BIT;
	     q = make_expr (ARRAY_REF, $4, 0, s);
	     
	     if ($4)
	       {
		 if (s->type->node.ar_decl.ranges)
		   error ("duplicate dimension in Allocatable statement") ;
		 else
		   s->type->node.ar_decl.ranges = $4 ;
	       }
	     else
	       if (! (s->type->node.ar_decl.ranges))
		 error ("no dimension in Allocatable statement") ;
	     
	     r = make_expr (EXPR_LIST, q, 0, 0);
	     $1->append (r) ;
	   }
        ;

/* pointer: POINTER in_dcl opt_double_colon name dims lengspec */

pointer: POINTER in_dcl LEFTPAR var COMMA var RIGHTPAR
	    { PTR_SYMB s;
	      PTR_EXPR q, r;

	      if (! $4) {
		error ("Pointer variable missing in 'pointer' statement");
		$$ = 0;
	      }
	      if (! $6) {
		error ("Pointee variable missing in 'pointer' statement");
		$$ = 0;
	      }
	      s = $4 -> node.fields.symbol;
	      s->attr = s->attr | POINTER_BIT;
	      r = make_expr (EXPR_LIST, $4, $6, 0);
	      $$ = make_stmt (POINTER_STMT, 0, r, 0, 0);
	    }

/* ...we may add the capability to recognise multiple pointer pairs later...
	| pointer COMMA name dims lengspec
           {  PTR_SYMB s;
	      PTR_EXPR q, r;

	      if (! $4) {
		error ("No Dimensions in Dimension Statement");
		$$ = 0;
	      }
	      s = make_array($3, 0, $4, ndim, LOCAL);
	      s->attr = s->attr | POINTER_BIT;
	      q = make_expr (ARRAY_REF, $4, 0, s);
	      s->type->node.ar_decl.ranges = $4;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append(r) ;
	   }
*/
	;

target: TARGET in_dcl opt_double_colon name dims lengspec
	    { PTR_SYMB s;
	      PTR_EXPR q, r;


	      if (! $5) {
		error ("No Dimensions in Target Statement");
		$$ = 0;
	      }
	      s = make_array($4, 0, $5, ndim, LOCAL);
	      s->attr = s->attr | TARGET_BIT;
	      q = make_expr (ARRAY_REF, $5, 0, s);
	      s->type->node.ar_decl.ranges = $5;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (TARGET_STMT, 0, r, 0, 0);
	    }
	| target COMMA name dims lengspec
           {  PTR_SYMB s;
	      PTR_EXPR q, r;

	      if (! $4) {
		error ("No Dimensions in Dimension Statement");
		$$ = 0;
	      }
	      s = make_array($3, 0, $4, ndim, LOCAL);
	      s->attr = s->attr | TARGET_BIT;
	      q = make_expr (ARRAY_REF, $4, 0, s);
	      s->type->node.ar_decl.ranges = $4;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append(r) ;
	}
	;

common:	  COMMON in_dcl var
	    { PTR_EXPR p, q;
              PTR_SYMB s;

/*	      s = make_common(cur_scope()->look_up_ftn_sym ("BLANK")); */
              p = make_expr (EXPR_LIST, $3, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, 0);
	      $$ = make_stmt (COMM_STAT, 0, q, 0, 0);
	    }
	| COMMON in_dcl comblock var
	    { PTR_EXPR p, q;

              p = make_expr (EXPR_LIST, $4, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, $3);
	      $$ = make_stmt (COMM_STAT, 0, q, 0, 0);
	    }
	| TASK_COMMON in_dcl var
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_common(cur_scope()->look_up_ftn_sym ("BLANK"));
              p = make_expr (EXPR_LIST, $3, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, s);
	      $$ = make_stmt (TASK_COM, 0, q, 0, 0);
	    }
	| TASK_COMMON in_dcl comblock var
	    { PTR_EXPR p, q;

              p = make_expr (EXPR_LIST, $4, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, $3);
	      $$ = make_stmt (TASK_COM, 0, q, 0, 0);
	    }
	| common opt_comma comblock opt_comma var
	    { PTR_EXPR p, q;

              p = make_expr (EXPR_LIST, $5, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, $3);
	      $1->node.fields.e1->append (q) ;
	    }
	| common COMMA var
	    { PTR_EXPR p, q, r;

              p = make_expr (EXPR_LIST, $3, 0, 0);
	      q = make_expr (COMM_LIST, p, 0, 0);
	      for (r = $1->node.fields.e1;
		   r->node.list.next;
		   r = r->node.list.next);
	      r->node.fields.e1->append (q) ;
	    }
	;

namelist:  NAMELIST in_dcl name
	    { PTR_EXPR p, q, r;
	      PTR_SYMB s;

              s = make_scalar($3, 0, LOCAL);	
	      p = make_expr (VAR_REF, 0, 0, s);
              q = make_expr (EXPR_LIST, p, 0, 0);
	      r = make_expr (NAMELIST_LIST, q, 0, 0);
	      $$ = make_stmt (NAMELIST_STAT, 0, r, 0, 0);
	    }
	| NAMELIST in_dcl comblock name
	    { PTR_EXPR p, q, r;
	      PTR_SYMB s;

              s = make_scalar($4, 0, LOCAL);	
	      p = make_expr (VAR_REF, 0, 0, s);
              q = make_expr (EXPR_LIST, p, 0, 0);
	      r = make_expr (NAMELIST_LIST, q, 0, $3);
	      $$ = make_stmt (NAMELIST_STAT, 0, r, 0, 0);
	    }
	| namelist opt_comma comblock opt_comma name
	    { PTR_EXPR p, q, r;
              PTR_SYMB s;

              s = make_scalar($5, 0, LOCAL);	
	      p = make_expr (VAR_REF, 0, 0, s);
              q = make_expr (EXPR_LIST, p, 0, 0);
	      r = make_expr (NAMELIST_LIST, q, 0, $3);
	      $1->node.fields.e1->append (r) ;
	    }
	| namelist COMMA name
	    { PTR_EXPR p, q, r;
              PTR_SYMB s;

              s = make_scalar($3, 0, LOCAL);	
	      p = make_expr (VAR_REF, 0, 0, s);
              q = make_expr (EXPR_LIST, p, 0, 0);
	      for (r = $1->node.fields.e1;
		   r->node.list.next;
		   r = r->node.list.next);
	      r->node.fields.e1->append (q);
	    }
	;

comblock: DSLASH
	    { $$ = make_common(cur_scope()->look_up_ftn_sym ("BLANK")); }
	| SLASH name SLASH
	    { $$ = make_common($2); }
	;

var:	  name dims
          {  PTR_SYMB s;
	
	      if ($2) {
   	    	s = make_array($1, 0, $2, ndim, LOCAL);
                s->attr = s->attr | DIMENSION_BIT;
		$$ = make_expr (ARRAY_REF, $2, 0, s);
	      }
	      else {
		s = make_scalar($1, 0, LOCAL);	
		$$ = make_expr (VAR_REF, 0, 0, s);
	      }

          }
            
	;

external: EXTERNAL in_dcl name
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_external($3, 0);
	      s->attr = s->attr | EXTERNAL_BIT;
              q = make_expr ( VAR_REF, 0, 0, s);
	      p = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (EXTERN_STAT, 0, p, 0, 0);
	    }

	| external COMMA name
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_external($3, 0);
	      s->attr = s->attr | EXTERNAL_BIT;
              p = make_expr ( VAR_REF, 0, 0, s);
	      q = make_expr (EXPR_LIST, p, 0, 0);
	      $1->node.fields.e1->append (q) ;
	    }
	;

intrinsic: INTRINSIC in_dcl name
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_function($3, 0, NO);
	      s->attr = s->attr | INTRINSIC_BIT;
              q = make_expr ( VAR_REF, 0, 0, s);
	      p = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (INTRIN_STAT, 0, p,
			     0, 0);
	    }

	| intrinsic COMMA name
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_function($3, 0, NO);
	      s->attr = s->attr | INTRINSIC_BIT;
              p = make_expr ( VAR_REF, 0, 0, s);
	      q = make_expr (EXPR_LIST, p, 0, 0);
	      $1->node.fields.e1->append (q) ;
	    }

	;


equivalence:  EQUIVALENCE in_dcl equivset
	    {
	      $$ = make_stmt (EQUI_STAT, 0, $3,
			     0, 0);
	    }

	| equivalence COMMA equivset
	    { 
	      $1->node.fields.e1->append ($3) ;
	    }

	;

equivset: LEFTPAR equivlist RIGHTPAR
	  {
	      $$ = make_expr (EQUI_LIST, $2, 0, 0);
           }
	;

equivlist: equi_object COMMA equi_object
           { PTR_EXPR p;
	      p = make_expr (EXPR_LIST, $3, 0, 0);
	      $$ = make_expr (EXPR_LIST, $1, p, 0);
	    }

	| equivlist COMMA equi_object
	    { PTR_EXPR p;

	      p = make_expr (EXPR_LIST, $3, 0, 0);
	      $1->append (p) ;
	    }
	;

equi_object: var
       | asubstring
       ;


/*------ DATA Statements are currently not parsed into components -----*/

data:	  data1
            { 
	      $$ = make_stmt (DATA_DECL, 0, $1, 0, 0) ;
            }

/*------- Re-enable this version of data1 to fully parse DATA statements ---
data1:	  DATA in_data datapair 

	| data1 opt_comma datapair
	;
*/

data1:    DATA in_data string_constant
            {
	      $$ = $3 ;
	    }



in_data:  
	    {
	      if (parstate == OUTSIDE)
	         { PTR_STMT p;

		   p = make_stmt (PROG_HEDR,
                                make_program(cur_scope()->look_up_ftn_sym ("MAIN")),
                                0, 0, 0);
		   attach_statement (p) ;
/* fgg remarked on 2000/12/11 since it's wrong ?!
	           push_scope_level (p, NO);
*/
		   position = IN_PROC; 
	  	   parstate = INDCL;
	           break;
                 }
	      if (parstate < INDATA)
		{
		  enddcl();
		  parstate = INDATA;
		}
	    }
	;

/*--------- Re-enable the following rules to parse DATA statements -------

dataname: IDENTIFIER

	;

datapair: datalvals SLASH datarvals SLASH

	;

datalvals: datalval

	| datalvals COMMA datalval

	;

datarvals: datarval
	| datarvals COMMA datarval
	;

datalval: dataname

	| dataname datasubs

	| dataname datarange

	| dataname datasubs datarange

	| dataimplieddo
	;

datasubs: LEFTPAR iconexprlist RIGHTPAR

	;

datarange: LEFTPAR opticonexpr COLON opticonexpr RIGHTPAR

	;

iconexprlist:
	  iconexpr

	| iconexprlist COMMA iconexpr

	;

opticonexpr:
	| iconexpr

	;

dataimplieddo:	LEFTPAR dlist COMMA dataname EQUAL iconexprlist RIGHTPAR

	;

dlist:	  dataelt

	| dlist COMMA dataelt

	;

dataelt:  dataname datasubs

	| dataname datarange

	| dataname datasubs datarange

	| dataimplieddo
	;

datarval: datavalue

	| dataname ASTER datavalue

	| unsignedint ASTER datavalue
	;

datavalue: dataname

	| int_const
	| real_const
x	| complex_const
	| TRUE_VAL
	| FALSE_VAL
	| CHAR_CONSTANT
        | HOLLERITH
x	| STRING	
x	| bit_const	
	;

int_const: unsignedint
	| PLUS unsignedint

	| MINUS unsignedint
				
	;

unsignedint: INT_CONSTANT
	;

real_const: unsignedreal
	| PLUS unsignedreal

	| MINUS unsignedreal

	;

unsignedreal:
	  REAL_CONSTANT
	| DP_CONSTANT
	;

bit_const:HEX_CONSTANT	{ $$ = mkbitcon(4, YYLeng(), YYText()); }
	| OCT_CONSTANT	{ $$ = mkbitcon(3, YYLeng(), YYText()); }
	| BITCON	{ $$ = mkbitcon(1, YYLeng(), YYText()); }
	;

iconexpr: iconterm
	| PLUS iconterm

	| MINUS iconterm

	| iconexpr PLUS iconterm

	| iconexpr MINUS iconterm

	;

iconterm: iconfactor
	| iconterm ASTER iconfactor

	| iconterm SLASH iconfactor

	;

iconfactor: iconprimary
	| iconprimary DASTER iconfactor

	;

iconprimary: INT_CONSTANT

	| dataname
	| LEFTPAR iconexpr RIGHTPAR

	;
-----------------------------------------------*/


savelist: saveitem
         { $$ = make_expr (EXPR_LIST, $1, 0, 0); }
	| savelist COMMA saveitem
         { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

saveitem: var
	   { $$ = $1;
             $$->node.fields.symbol->attr = $$->node.fields.symbol->attr | SAVE_BIT;
           }
	| comblock
          { $$ = make_expr (COMM_LIST, 0, 0, $1); 
            $$->node.fields.symbol->attr = $$->node.fields.symbol->attr | SAVE_BIT;
          }
	;

use_name_list: name
	   { 
             PTR_SYMB s;
             PTR_EXPR q; 

             s = make_scalar($1, global_default, LOCAL);	
	     s->attr = type_options;
	     q = make_expr (VAR_REF, 0, 0, s);
	     $$ = make_expr (EXPR_LIST, q, 0, 0);
            }
	| use_name_list COMMA name
	   {
              PTR_SYMB s;
	      PTR_EXPR q, r;

              s = make_scalar($3, global_default, LOCAL);	
	      q = make_expr (VAR_REF, 0, 0, s);
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->append (r) ;
           }
         ;

paramlist:paramitem
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| paramlist COMMA paramitem
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

paramitem: name EQUAL expr
	    { PTR_SYMB p;

                /* The check if name and expr have compatible types has
                   not been done yet. */ 
		p = make_constant($1, 0);
 	        p->attr = p->attr | PARAMETER_BIT;
                p->node.const_value = $3;
		$$ = make_expr (CONST_REF, 0, 0, p);
	    }
	;

module_proc_stmt: MODULE needkeyword PROCEDURE proc_name_list
	    { $$ = make_stmt (MODULE_PROC_STMT, 0, $4, 0, 0); }

proc_name_list: name
	    { PTR_SYMB s;
 	      PTR_EXPR p, q;

	      s = make_function($1, 0, LOCAL);
	      s->symb_variant(ROUTINE_NAME);
              q = make_expr ( VAR_REF, 0, 0, s);
	      $$ = make_expr (EXPR_LIST, q, 0, 0);
	    }
	| proc_name_list COMMA name
	    { PTR_EXPR p, q;
              PTR_SYMB s;

	      s = make_function($3, 0, NO);
	      s->symb_variant(ROUTINE_NAME) ;
              p = make_expr ( VAR_REF, 0, 0, s);
	      q = make_expr (EXPR_LIST, p, 0, 0);
	      $1->node.fields.e1->append (q) ;
	    }
	;


use_stat: USE in_dcl module_name 
	    { $$ = make_stmt (USE_STMT, $3, 0, 0, 0);
              push_scope_level (module_header, YES);
            }
	| USE in_dcl module_name COMMA keyword_if_colon_follow rename_list
	    { $$ = make_stmt (USE_STMT, $3, $6, 0, 0); 
              push_scope_level (module_header, YES);
            }
	| USE in_dcl module_name COMMA keyword_if_colon_follow ONLY 
	    { PTR_EXPR l;

	      l = make_expr ( ONLY_NODE, 0, 0, 0);
              $$ = make_stmt (USE_STMT, $3, l, 0, 0);
            }
	| USE in_dcl module_name COMMA keyword_if_colon_follow ONLY only_list
	    { PTR_EXPR l;

	      l = make_expr ( ONLY_NODE, $7, 0, 0);
              $$ = make_stmt (USE_STMT, $3, l, 0, 0);
            }
	;

module_name: name
            {
              if ($1->id_attr == 0)
	         errstr("unknown module '%s'", $1->identifier()) ;
              $$ = make_global_entity($1, MODULE_NAME, global_default, NO) ;
	      module_header = $$->node.fields.func_hedr ;
	      module_scope = module_header->scoping () ;
            }
        ;

only_list: only_name
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| only_list COMMA only_name
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

only_name: name POINT_TO name
	    {  PTR_HASH oldhash;
	       PTR_SYMB oldsym, newsym;
	       PTR_EXPR l, m;

	       oldhash = module_scope->just_look_up_ftn_sym_in_scope ($3->identifier()) ;
	       if (oldhash == 0) {
                  errstr("unknown identifier '%s'", $3->identifier());
	          $$= 0;
	       }
	       else {
	         oldsym = oldhash->id_attr;
	         newsym = make_local_entity($1, oldsym->symb_variant(), 
					    oldsym->type, LOCAL);
	         // copies data in node.fields structure and attr
		 newsym = oldsym ;
	         newsym->node.fields.declared_name = oldsym;
	  	 l = make_expr ( VAR_REF, 0, 0, oldsym);
		 m = make_expr ( VAR_REF, 0, 0, newsym);
		 $$ = make_expr ( RENAME_NODE, m, l, 0);
 	      }
            }
	| name
	    {  PTR_HASH oldhash;
	       PTR_SYMB oldsym, newsym;
	       PTR_EXPR l, m;

	       oldhash = module_scope->just_look_up_ftn_sym_in_scope ($1->identifier());
	       if (oldhash == 0) {
                  errstr("unknown identifier '%s'", $1->identifier());
	          $$= 0;
	       }
	       else {
	         oldsym = oldhash->id_attr;
	         newsym = make_local_entity($1, oldsym->symb_variant(), oldsym->type,
LOCAL);
	         /* copies data in node.fields structure and attr */
		 newsym = oldsym ;
	         newsym->node.fields.declared_name = oldsym;
	  	 l = make_expr ( VAR_REF, 0, 0, oldsym);
		 m = make_expr ( VAR_REF, 0, 0, newsym);
		 $$ = make_expr ( RENAME_NODE, m, l, 0);
 	      }
            }
	;


rename_list: rename_name
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| rename_list COMMA rename_name
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

rename_name: name POINT_TO name
	    {  PTR_HASH oldhash;
	       PTR_SYMB oldsym, newsym;
	       PTR_EXPR l, m;

	       oldhash = module_scope->just_look_up_ftn_sym_in_scope ($3->ident());
	       if (oldhash == 0) {
                  errstr("unknown identifier '%s'.", $3->ident());
	          $$= 0;
	       }
	       else {
	         oldsym = oldhash->id_attr;
	         newsym = make_local_entity($1, oldsym->symb_variant(), oldsym->type,
LOCAL);
	         /* copies data in node.fields structure and attr */
		 newsym = oldsym ;
	         newsym->node.fields.declared_name = oldsym;
	  	 l = make_expr ( VAR_REF, 0, 0, oldsym);
		 m = make_expr ( VAR_REF, 0, 0, newsym);
		 $$ = make_expr ( RENAME_NODE, m, l, 0);
 	      }
            }
	;

dims:
	    { ndim = 0;	$$ = 0; }
	| LEFTPAR dimlist RIGHTPAR
	    { $$ = $2; }
	;

dimlist:   { ndim = 0; }   dim
	    {
	      $$ = make_expr (EXPR_LIST, $2, 0, 0);
	      $$->type = global_default;
	    }
	| dimlist COMMA dim
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

dim:	  ubound
	    {
	      if (ndim == maxdim)
		error ("too many dimensions");
	      else if (ndim < maxdim)
		$$ = $1;
	      ++ndim;
	    }
	| COLON
	    {
	      if (ndim == maxdim)
		error ("too many dimensions");
	      else if (ndim < maxdim)
		$$ = make_expr ( DDOT, 0, 0, 0);
	      ++ndim;
	    } 
	| expr COLON
	    {
	      if (ndim == maxdim)
		error ("too many dimensions");
	      else if (ndim < maxdim)
		$$ = make_expr (DDOT, $1, 0, 0);
	      ++ndim;
	    }
	| expr COLON ubound
	    {
	      if (ndim == maxdim)
		error ("too many dimensions");
	      else if (ndim < maxdim)
		$$ = make_expr (DDOT, $1, $3, 0);
	      ++ndim;
	    }
	;

ubound:	  ASTER
	    {
	      $$ = make_expr (STAR_RANGE, 0, 0, 0);
	      $$->type = global_default;
	    }
	| expr
	;

labellist: label
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| labellist COMMA label
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

label:	  INT_CONSTANT
	    {
	     PTR_FTN_LABEL p;
	     p = make_label ($1) ;
	     p->scope = cur_proc ();
	     $$ = make_expr (LABEL_REF, p);
	    }
	;

implicit: IMPLICIT in_dcl implist
	   { PTR_EXPR l;

             l = make_expr ( EXPR_LIST, $3, 0, 0);
             $$ = make_stmt (IMPL_DECL, 0, l, 0, 0);
           }
	| implicit COMMA implist
	  { PTR_EXPR l;

            l = make_expr ( EXPR_LIST, $3, 0, 0);
            $1->node.fields.e1->append (l) ;
          }
	| IMPLICITNONE
	  { 
            //FGG added on 2000/12/10 - to deal with implicit none 
            //when there was no 'program' declaration
            switch(parstate)
	    {
              case OUTSIDE:  
	      { 
                PTR_STMT p;

		p = make_stmt (PROG_HEDR,
                               make_program(cur_scope()->look_up_ftn_sym ("MAIN")),
                               0, 0, 0);
		attach_statement (p) ;
/* fgg remarked on 2000/12/11 since it's wrong ?!
                push_scope_level (p, NO);
*/
		position = IN_PROC; 
	  	parstate = INDCL;
	        break;
              }
	      case INDATA:
              case INSIDE:    parstate = INDCL;
              case INDCL:     break;

              default:
                error ("declaration among executables");
            }
          
            undeftype = YES;
	    // warn ("ignoring IMPLICIT NONE statement");
	    setimpl(0, (int)'a', (int)'z', cur_scope()) ;
	    $$ = make_stmt (IMPL_DECL, 0, 0, 0, 0);
	  }
	;
implist:  imptype LEFTPAR letgroups RIGHTPAR
          { PTR_EXPR l;

            $$ = make_expr ( IMPL_TYPE, $3, 0, 0);
            $$->type = $1 ;
          }
	| imptype
	 {
           $$ = make_expr ( IMPL_TYPE, 0, 0, 0);
           $$->type = $1 ;
         }
	; 

/* The draft specification leads to big trouble. Check that up. */
/* For the time being. */

imptype:   { needkwd = YES; } typespec lengspec
	    { 
	      if ($3 && $2 == global_byte)
		err ("BYTE cannot have a length specification") ;
	      type_options = 0 ;
	      attr_dims = 0 ;
	      $$ = make_type_node ($2, $3) ;
	    }
	;

letgroups: letgroup
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| letgroups COMMA letgroup
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

letgroup:  letter
	    {
	      setimpl(vartype, (int)$1, (int)$1, cur_scope());
	      $$ = make_expr (CHAR_VAL, 0, 0, 0);
	      $$->node.cval = $1;
	    }
	| letter MINUS letter
	    { PTR_EXPR p,q;
	      
	      setimpl(vartype, (int)$1, (int)$3, cur_scope());
	      p = make_expr (CHAR_VAL, 0, 0, 0);
	      p->node.cval = $1;
	      q = make_expr (CHAR_VAL, 0, 0, 0);
	      q->node.cval = $3;
	      $$= make_expr (DDOT, p, q, 0);
	    }
	;

letter:	 IDENTIFIER
	    {
	      string* ident = $1 ;
	      char letr = ident ? tolower((*ident)()[0]) : '\0' ;
	      int idlen = ident ? strlen((*ident)()) : 0 ;

	      if (idlen != 1 || letr < 'a' || letr > 'z')
		{
		  error ("implicit item must be single letter");
		  $$ = '\0';
		}
	      else $$ = letr ;
	    }
	;

in_dcl:
	{ switch(parstate)
		{
                case OUTSIDE:  
			{ PTR_STMT p;

                          newprog () ; //fgg?

	                  if (position == IN_OUTSIDE)
                                 position = IN_PROC ;
                          else if (position != IN_INTERNAL_PROC)
                     	         position-- ;
                          else
	                        error ("internal procedures cannot contain procedures") ;


			  p = make_stmt (PROG_HEDR,
                                       make_program(cur_scope()->look_up_ftn_sym ("MAIN")),
                                       0, 0, 0);
			  attach_statement (p) ;
/* fgg remarked on 2000/12/11 since it's wrong ?!
                          push_scope_level (p, NO);
*/
			  position = IN_PROC; 
	  		  parstate = INDCL;
	                  break;
			}
		case INDATA:
                case INSIDE:    parstate = INDCL;
                case INDCL:     break;
/*
                case INDATA:
		  error ("statement out of order: declaration after DATA");
		  break;
*/

                default:
                  error ("declaration among executables");
                }
        }

	;

opt_double_colon: 
	| COLON COLON
        ;         


/*----------------------------------------------------------+
 |                                                          |
 |               Grammar for EXPRESSIONS                    |
 |                                                          |
 +----------------------------------------------------------*/

funarglist:
	    { $$ = 0; }
	| funargs 
            { $$ = $1; }
	;
funarg:  expr
	    { $$ = $1; }
	| triplet
	    { $$ = $1; }
	| nameeq expr
	    { PTR_EXPR l;

	      l = make_expr ( KEYWORD_ARG, $1, $2, 0);
	      l->type = $2->type;
	    }
        ;

funargs:  in_ioctl funarg
            { $$ = set_ll_list ($2, 0, EXPR_LIST);
              endioctl(); 
            }
        | funargs COMMA in_ioctl funarg
	    { $$ = set_ll_list ($1, $4, EXPR_LIST);
              endioctl();
            }
        ;

/*-------------------- Now Obsolete -----------------------

subscript_list: expr
	    { $$ = set_ll_list ($1, 0, EXPR_LIST); }
        | subscript_list COMMA expr
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }
        ;
---------------------------------------------------------------*/

expr:	  uexpr
            { $$ = $1; }
	| LEFTPAR expr RIGHTPAR
	    { 
/* was before fgg changed on 2000/12/11 to deal with (,) correctly
              $$ = $2; 
*/
	      $$ = make_expr(PAREN_OP, $2, 0, 0) ;
	      $$->set_expr_type();
            }
        | LEFTPAR error RIGHTPAR
            { $$ = 0 ; }
	| complex_const
	    { $$ = $1; }
	;

uexpr:	  lhs
            { $$ = $1; }
	| simple_const
            { $$ = $1; }
/*        | string_constant opt_substring
            { $$ = make_expr ( ARRAY_OP, $1, $2, 0); }
*/	| vec
            { $$ = $1; }
	| expr PLUS expr   %prec PLUS
	    { $$ = intrinsic_op_node("+", ADD_OP, $1, $3); }
	| expr MINUS expr   %prec PLUS
	    { $$ = intrinsic_op_node("-", SUBT_OP, $1, $3); }
	| expr ASTER expr
	    { $$ = intrinsic_op_node("*", MULT_OP, $1, $3); }
	| expr SLASH expr
	    { $$ = intrinsic_op_node("/", DIV_OP, $1, $3); }
	| expr DASTER expr
	    { $$ = intrinsic_op_node("**", EXP_OP, $1, $3); }
	| defined_op expr %prec DASTER
	    { $$ = defined_op_node($1, $2, 0); }
	| PLUS expr  %prec ASTER
	    { $$ = intrinsic_op_node("+", UNARY_ADD_OP, $2, 0); }
	| MINUS expr  %prec ASTER
	    { $$ = intrinsic_op_node("-", MINUS_OP, $2, 0); }
	| expr EQ expr  %prec EQ
	    { $$ = intrinsic_op_node(".eq.", EQ_OP, $1, $3); }
	| expr GT expr  %prec EQ
	    { $$ = intrinsic_op_node(".gt.", GT_OP, $1, $3); }
	| expr LT expr  %prec EQ
	    { $$ = intrinsic_op_node(".lt.", LT_OP, $1, $3); }
	| expr GE expr  %prec EQ
	    { $$ = intrinsic_op_node(".ge.", GTEQL_OP, $1, $3); }
	| expr LE expr  %prec EQ
	    { $$ = intrinsic_op_node(".ge.", LTEQL_OP, $1, $3); }
	| expr NE expr  %prec EQ
	    { $$ = intrinsic_op_node(".ne.", NOTEQL_OP, $1, $3); }
	| expr EQV expr
	    { $$ = intrinsic_op_node(".eqv.", EQV_OP, $1, $3); }
	| expr NEQV expr
	    { $$ = intrinsic_op_node(".neqv.", NEQV_OP, $1, $3); }
	| expr XOR expr
	    { $$ = intrinsic_op_node(".xor.", XOR_OP, $1, $3); }
	| expr OR expr
	    { $$ = intrinsic_op_node(".or.", OR_OP, $1, $3); }
	| expr AND expr
	    { $$ = intrinsic_op_node(".and.", AND_OP, $1, $3); }
	| NOT expr
	    { $$ = intrinsic_op_node(".not.", NOT_OP, $2, 0); }
	| expr DSLASH expr
	    { $$ = intrinsic_op_node("//", CONCAT_OP, $1, $3); }
	| expr defined_op expr %prec BINARY_OP
	    { $$ = defined_op_node($2, $1, $3); }
	;

addop:	  PLUS	{ $$ = ADD_OP; }
	| MINUS	{ $$ = SUBT_OP; }
	;

ident: name %prec IDENT
	    { PTR_SYMB s;
	      PTR_TYPE t;
	      PTR_EXPR l;

       	      if (!(s = $1->id_attr))
              {
		// TBD: with 'implicit none' an intrinsic function call will get an error here!!!
	         s = make_scalar($1, 0, LOCAL);
	     	 s->decl = SOFT;
	      } 

#ifdef DEBUG_NAMES
	      printf ("%d: ident: name = '%s' symbol = 0x%08x, type = 0x%08x, variant = ", 
	        yylineno, $1->ident(), (unsigned)($1->id_attr), (int)(s->type)) ;
#endif

	      switch (s->symb_variant())
              {
	      case CONST_NAME:
#ifdef DEBUG_NAMES
		printf ("CONST_NAME\n") ;
#endif
		   $$ = make_expr (CONST_REF,0,0, s);
                   $$->type = s->type;
	           break;
	      case DEFAULT_NAME:   /* if common region with same name has been
                                 declared. */
#ifdef DEBUG_NAMES
		printf ("DEFAULT_NAME\n") ;
#endif
		   s = make_scalar($1, 0, LOCAL);
	     	   s->decl = SOFT;
	      case VARIABLE_NAME:
#ifdef DEBUG_NAMES
		printf ("VARIABLE_NAME\n") ;
#endif
                   $$ = make_expr (VAR_REF,0,0, s);
	           t = s->type;
	           if (t != 0) {
	              if ((t->type_variant() == T_ARRAY) ||
                          (t->type_variant() == T_STRING))
  		         $$->expr_variant(ARRAY_REF) ;
/*  	              if (t->type_variant() == T_DERIVED_TYPE)
                         $$->expr_variant(RECORD_REF) ; */
	           }

                   $$->type = t;
	           break;
	      case TYPE_NAME:
#ifdef DEBUG_NAMES
		printf ("TYPE_NAME\n") ;
#endif
  	           $$ = make_expr (TYPE_REF,0,0, s);
	           $$->type = s->type;
	           break;
	      case INTERFACE_NAME:
#ifdef DEBUG_NAMES
		printf ("INTERFACE_NAME\n") ;
#endif
  	           $$ = make_expr ( INTERFACE_REF,0,0, s);
	           $$->type = s->type;
	           break;
	      default:
#ifdef DEBUG_NAMES
		printf ("OTHER (%d)\n",s->variant()) ;
#endif
  	           $$ = make_expr (VAR_REF,0,0, s);
	           $$->type = s->type;
	           break;
	      }
              if ($$->variant() == T_POINTER) {
	         l = $$;
	         $$ = make_expr ( DEREF_OP, l, 0, 0);
	         $$->type = l->type->node.fields.base_type;
	      }
           } 
        ;

lhs:      ident %prec IDENT
            {
		Expression *varef = $1 ;
		Symbol *sym = varef->node.var_ref.symbol ;
   
		/* Check if this is an intrinsic function name which is being used
		 * as a variable with implicit typing.
		 */

		if (sym->attr & INTRINSIC_BIT)
		{
		  /* Need to implicitly declare this as a new symbol in current scope */
		  sym = make_local_symbol (sym->ident(), VARIABLE_NAME, 0, LOCAL);
		  varef->node.fields.symbol = sym ;
		}
		
		varef->set_expr_type() ;
		$$ = varef ;
	    }
	| structure_component %prec IDENT
            { $$ = $1 ; }
	| array_ele_substring_func_ref %prec IDENT
            { 
		$$ = $1 ; 
		$$->set_expr_type() ;
	    }
	;

array_ele_substring_func_ref:  ident LEFTPAR in_ioctl funarglist RIGHTPAR %prec IDENT
	    { 
	      int num_triplets ;
	      PTR_SYMB s, sym ;
	      PTR_EXPR l ;
	
	      l = $1 ;
	      s = $1->node.fields.symbol ;

	      // Handle variable-to-function conversion

	      if ( ($1->expr_variant() == VAR_REF) && 
		   (((s->symb_variant() == VARIABLE_NAME) && (s->type) &&
		     (s->type->type_variant() != T_ARRAY)) ||
		    (s->symb_variant() == ROUTINE_NAME))) {
/* was before fgg changed on 2000/12/11
	        s = $1->node.fields.symbol =  make_function(s->parent, 0, NO) ;
*/
	        s = $1->node.fields.symbol =  make_function(s->parent, 0, LOCAL) ;
	        $1->expr_variant(FUNC_CALL) ;
              }
	      if ( ($1->expr_variant() == VAR_REF) && 
		   (s->symb_variant() == FUNCTION_NAME)) 
	        $1->expr_variant(FUNC_CALL) ;
	      
              l = $1 ;
	      num_triplets = $4->is_array_section_ref() ;

	      switch ($1->variant())
              {
	      case TYPE_REF:
	           $$ = make_expr ( STRUCTURE_CONSTRUCTOR, $1, $4, 0);
	           $$->type = $1->type;
	           break;
	      case INTERFACE_REF:
	           sym = s->resolve_overloading ($4);
	           if (sym != 0)
	  	   {
	              l = make_expr ( FUNC_CALL, $4, 0, sym);
	              l->type = sym->type;
	              $$ = $1; 
		      $$->variant(OVERLOADED_CALL) ;
	              $$->node.fields.e1 = l;
	              $$->type = sym->type;
	           }
	           else {
	             errstr("can't resolve call '%s'", s->ident());
	           }
	           break;
	      case FUNC_CALL:
                   l->node.fields.e1 = $4;
		   $$ = deal_with_func_call (l) ;
	           // late_bind_if_needed($$);
	           break;
	      case DEREF_OP:
              case ARRAY_REF:
	           /* array element */
	           if (num_triplets == 0) {
       	                   $1->node.fields.e1 = $4;
	                   $$ = $1;
                           $$->type = $1->type->node.ar_decl.base_type;
                   }
                   /* substring */
	           else if ((num_triplets == 1) && 
                            ($1->type->type_variant() == T_STRING)) {
    	             $1->node.fields.e1 = $4;
	             $$ = $1; $$->type = global_string;
                   }           
                   /* array section */
                   else {
    	             $1->node.fields.e1 = $4;
	             $$ = $1; $$->type = make_type ( T_ARRAY);
                     $$->type->node.ar_decl.base_type = $1->type;
	             $$->type->node.ar_decl.num_dimensions = num_triplets;
                   }
	           break;
	      default:
	             error ("can't subscript");
             }
             if ($$->type_variant() == T_POINTER) {
	        l = $$;
	        $$ = make_expr ( DEREF_OP, l, 0, 0);
	        $$->type = l->type->node.fields.base_type;
	     }
	     endioctl();
           }
        | ident LEFTPAR in_ioctl funarglist RIGHTPAR substring %prec HYPER
	    { int num_triplets;
	      PTR_SYMB s;
	      PTR_EXPR l;

	      s = $1->node.fields.symbol;
/*              if ($1->type->type_variant() == T_POINTER) {
	         l = $1;
	         $1 = make_expr ( DEREF_OP, l, 0, 0);
	         $1->type = l->type->node.fields.base_type;
	      } */
	      if (($1->type->type_variant() != T_ARRAY) ||
                  ($1->type->node.ar_decl.base_type->type_variant() != T_STRING)) {
	         errstr("can't take substring of '%s'", s->ident());
              }
              else {
  	        num_triplets = $4->is_array_section_ref();
	           /* array element */
                if (num_triplets == 0) {
                   $1->node.fields.e1 = $4;
                   $1->node.fields.e2 = $6;
	           $$ = $1;
                   $$->type = $1->type->node.ar_decl.base_type;
                }
                /* array section */
                else {
    	           $1->node.fields.e1 = $4;
    	           $1->node.fields.e2 = $6;
	           $$ = $1; $$->type = make_type ( T_ARRAY);
                   $$->type->node.ar_decl.base_type = $1->type;
	           $$->type->node.ar_decl.num_dimensions = num_triplets;
               }
               if ($$->type_variant() == T_POINTER) {
	          l = $$;
	          $$ = make_expr ( DEREF_OP, l, 0, 0);
	          $$->type = l->type->node.fields.base_type;
	       }
             }
             endioctl();
          }
        | structure_component LEFTPAR funarglist RIGHTPAR
           {  int num_triplets;
	      PTR_EXPR l;

/*              if ($1->type_variant() == T_POINTER) {
	         l = $1;
	         $1 = make_expr ( DEREF_OP, l, 0, 0);
	         $1->type = l->type->node.fields.base_type;
	      } */
	      if ($1->type->type_variant() == T_ARRAY)
              {
  	         num_triplets = $3->is_array_section_ref();
	         /* array element */
	         if (num_triplets == 0) {
	            $$ = make_expr ( ARRAY_OP, 0, 0, 0);
       	            $$->node.fields.e1 = $1;
       	            $$->node.fields.e2 = $3;
                    $$->type = $1->type->node.ar_decl.base_type;
                 }
                 /* substring */
	         else if ((num_triplets == 1) && 
                          ($1->type->type_variant() == T_STRING)) {
	                  $$ = make_expr ( ARRAY_OP, 0, 0, 0);
    	                  $$->node.fields.e1 = $1;
       	                  $$->node.fields.e2 = $3;
	                  $$->type = global_string;
                 }           
                 /* array section */
                 else {
                     $$ = make_expr ( ARRAY_OP, 0, 0, 0);
    	             $$->node.fields.e1 = $1;
    	             $$->node.fields.e1 = $3;
	             $$->type = make_type ( T_ARRAY);
                     $$->type->node.ar_decl.base_type = $1->type;
	             $$->type->node.ar_decl.num_dimensions = num_triplets;
                 }
                 if ($$->type_variant() == T_POINTER) {
	            l = $$;
	            $$ = make_expr ( DEREF_OP, l, 0, 0);
	            $$->type = l->type->node.fields.base_type;
	         }
             } 
             else error ("can't subscript");
           }
        | structure_component LEFTPAR funarglist RIGHTPAR substring
	    { int num_triplets;
	      PTR_EXPR l;

/*              if ($1->type_variant() == T_POINTER) {
	         l = $1;
	         $1 = make_expr ( DEREF_OP, l, 0, 0);
	         $1->type = l->type->node.fields.base_type;
	      } */
	      if (($1->type->variant() != T_ARRAY) &&
                  ($1->type->node.ar_decl.base_type->variant() != T_STRING)) {
	         error ("can't take substring");
              }
              else {
  	        num_triplets = $3->is_array_section_ref();
                /* array element */
	        if (num_triplets == 0) {
	           l = make_expr ( ARRAY_OP, 0, 0, 0);
       	           l->node.fields.e1 = $1;
       	           l->node.fields.e2 = $3;
                   l->type = global_string;
                }
                /* array section */
                else {
	             l = make_expr ( ARRAY_REF, 0, 0, 0);
    	             l->node.fields.e1 = $1;
    	             l->node.fields.e2 = $3;
	             l->type = make_type ( T_ARRAY);
                     l->type->node.ar_decl.base_type = global_string;
	             l->type->node.ar_decl.num_dimensions = num_triplets;
                }
	        $$ = make_expr ( ARRAY_OP, l, $5, 0);
	        $$->type = l->type;
             }
          }
	;

structure_component: lhs PERCENT IDENTIFIER
	    { PTR_TYPE t;
	      PTR_SYMB s, field;
	      PTR_STMT at_scope;
              PTR_EXPR l;

/*              if ($1->variant() == T_POINTER) {
	         l = $1;
	         $1 = make_expr ( DEREF_OP, l, 0, 0);
	         $1->type = l->type->node.fields.base_type;
	      } */
	      t = $1->type;
              field = t->name->component((*$3)());
	      if (((($1->variant() == VAR_REF) ||
	            ($1->variant() == CONST_REF) || ($1->variant() == ARRAY_REF))
                   && (t->variant() == T_DERIVED_TYPE)) ||
	          (($1->variant() == ARRAY_REF) && (t->variant() == T_ARRAY) &&
                   (t = t->node.ar_decl.base_type) && (t->variant() == T_DERIVED_TYPE))) {
	         if (field = $1->type->name->component((*$3)())) {
	            l =  make_expr ( VAR_REF, 0, 0, field);
                    $$ = make_expr ( RECORD_REF, $1, l, 0);
	            if ($1->variant() == ARRAY_REF) {
                       l->type = field->type;
   	               $$->type = make_type ( T_ARRAY);
	               $$->type->node.ar_decl.base_type = field->type;
	               $$->type->node.ar_decl.num_dimensions = t->node.ar_decl.num_dimensions;
                   }
	           else $$->type = l->type = field->type;
                 }
                 else errstr("can't take component '%s'", (*$3)());
                 if ($$->variant() == T_POINTER) {
	            l = $$;
	            $$ = make_expr ( DEREF_OP, l, 0, 0);
	            $$->type = l->type->node.fields.base_type;
	         }
              }
              else errstr("can't take component '%s'", (*$3)());
	    }
        ;

/*-------------------- Now Obsolete -----------------------

array_element: ident LEFTPAR subscript_list RIGHTPAR
            {  int num_triplets;

	      if ($1->type->variant() == T_ARRAY)
              {
  	         num_triplets = $3->is_array_section_ref();
	         // array element
	         if (num_triplets == 0) {
       	            $1->node.fields.e1 = $3;
       	            $$ = $1;
                    $$->type = $1->type->node.ar_decl.base_type;
                 }
                 // substring
	         else if ((num_triplets == 1) && 
                          ($1->type->variant() == T_STRING)) {
    	                  $1->node.fields.e1 = $3;
	                  $$ = $1; $$->type = global_string;
                 }           
                 // array section
                 else {
    	             $1->node.fields.e1 = $3;
	             $$ = $1; $$->type = make_type ( T_ARRAY);
                     $$->type->node.ar_decl.base_type = $1->type;
	             $$->type->node.ar_decl.num_dimensions = num_triplets;
                 }
             } 
             else error ("can't subscript");
           }
        ;
---------------------------------------------------------------*/

asubstring: ident substring
            { 
	      if ($1->type->variant() == T_STRING) {
                 $1->node.fields.e1 = $2;
                 $$ = $1; $$->type = global_string;
              }
              else errstr("can't subscript '%s'", $1->node.fields.symbol->ident());
            }
        ;

opt_substring:
          /* empty */ %prec EPSILON
            { $$ = 0; } 
        | substring
            { $$ = $1; }
        ;

substring:  LEFTPAR opt_expr COLON opt_expr RIGHTPAR
	    { $$ = make_expr ( DDOT, $2, $4, 0); }
	;

opt_expr:
          /* empty */ %prec EPSILON
	    { $$ = 0; }
	| expr
	    { $$ = $1; }
	;


simple_const:
	  TRUE_VAL
	    {
	      $$ = make_expr (BOOL_VAL, 0, 0, 0);
	      $$->node.bval = true;
	      $$->type = global_bool;
	    }
	| FALSE_VAL
	    {
	      $$ = make_expr (BOOL_VAL, 0, 0, 0);
	      $$->node.bval = false;
	      $$->type = global_bool;
	    }
	| INT_CONSTANT
	    {
	      $$ = make_expr (INT_VAL, 0, 0, 0);
	      $$->node.ival = $1 ;
	      $$->type = global_int;
	    }
	| REAL_CONSTANT
	    {
	      $$ = make_expr (FLOAT_VAL, 0, 0, 0);
	      $$->node.string_val = $1 ;
	      $$->type = global_float;
	    }
	| DP_CONSTANT
	    {
	      $$ = make_expr (DOUBLE_VAL, 0, 0, 0);
	      $$->node.string_val = $1 ;
	      $$->type = global_double;
	    }
        | OCTAL_CONSTANT
	    {
	      $$ = make_expr (OCTAL_VAL, 0, 0, 0);
	      $$->node.string_val = $1 ;
	      $$->type = global_double;
	    }
        | HEX_CONSTANT
	    {
	      $$ = make_expr (HEX_VAL, 0, 0, 0);
	      $$->node.string_val = $1 ;
	      $$->type = global_double;
	    }
        | HOLLERITH
           {
	     $$ = make_expr (STRING_VAL, 0, 0, 0) ;
	     $$->node.string_val = $1 ;
	     $$->type = global_string ;
	     $$->note_transform (make_transform (HOLL_TRANS, $$)) ;
	   }
        | string_constant opt_substring
            {
              if ($2 != 0)
		 $$ = make_expr ( ARRAY_OP, $1, $2, 0); 
	      else $$ = $1;
            }
	;

string_constant: CHAR_CONSTANT
	    {
	      $$ = make_expr (STRING_VAL, 0, 0, 0);
	      $$->node.string_val = $1 ;
	      $$->type = global_string;
	    }
        ;

complex_const:	LEFTPAR uexpr COMMA uexpr RIGHTPAR
	    {
	      $$ = make_expr (COMPLEX_VAL, $2, $4, 0);
	      $$->type = global_complex;
	    }
	;
/*
section: LEFTPAR triplets RIGHTPAR
	    { $$ = $2; }
	;

triplets:triplet
	    { $$ = make_expr (EXPR_LIST, $1, 0, 0); }

	| triplets COMMA triplet
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); }*/
/* I changed uexpr to expr in below rule and triplet rules.
    Don't know why Daya wrote it as uexpr. Srinivas. */
/*	| triplets COMMA expr
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); } */
/* Deleted. Purpose unknown. Srinivas. 
	| expr COMMA triplets
	    { $$ = set_ll_list ($1, $3, EXPR_LIST); } 
	;*/

triplet: expr COLON expr
	    { $$ = make_expr (DDOT,$1,$3,0); }
	| expr COLON expr COLON expr
	    { $$ = make_expr (DDOT,make_expr (DDOT,$1,$3,0),$5,0); }
	| COLON expr COLON expr
	    { $$ = make_expr (DDOT, make_expr (DDOT,0,$2,0),$4,0); }
	| COLON COLON expr
	    { $$ = make_expr (DDOT,make_expr (DDOT,0,0,0),$3,0); }
	| COLON expr
	    { $$ = make_expr (DDOT,0,$2,0); }
	| COLON
	    { $$ = make_expr (DDOT,0,0,0); }
	;

vec:	  LEFTAB outlist RIGHTAB
           { $$ = make_expr (CONSTRUCTOR_REF,$2,0,0); 
             $$->type = $2->type; }
	;

/* Not used ...
complist: comp
	   { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| complist COMMA comp
           { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

comp:	  expr
	   { $$ = $1; }
	| LEFTPAR complist COMMA dospec RIGHTPAR
	   { $$ = 0; }
	;
*/
  
allocate_object: ident
	   { $$ = $1; }
	| structure_component
	   { $$ = $1; }
	;

allocation_list: var
	   { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| allocation_list COMMA var
	   { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

allocate_object_list: allocate_object 
	   { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| allocate_object_list COMMA allocate_object 
	   { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;

opt_stat_spec: 
	   { $$ = 0; }
	| COMMA needkeyword STAT EQUAL ident
	   { $$ = $5; }
	;

pointer_name_list: ident
	   { $$ = set_ll_list ($1, 0, EXPR_LIST); }
	| pointer_name_list COMMA ident
	   { $$ = set_ll_list ($1, $3, EXPR_LIST); }
	;


/*----------------------------------------------------------+
 |                                                          |
 |            Grammar for EXECUTABLE STATEMENTS             |
 |                                                          |
 +----------------------------------------------------------*/

exec:	  iffable
	    { $$ = $1; }
	| ALLOCATE LEFTPAR allocation_list opt_stat_spec RIGHTPAR
	    { $$ = make_stmt (ALLOCATE_STMT, 0, $3, $4, 0); }
	| DEALLOCATE LEFTPAR allocate_object_list opt_stat_spec RIGHTPAR
	    { $$ = make_stmt (DEALLOCATE_STMT, 0, $3, $4 , 0); }
	| NULLIFY LEFTPAR pointer_name_list RIGHTPAR
	    { $$ = make_stmt (NULLIFY_STMT, 0, $3, 0, 0); }
	| whereable
            { $$ = $1; }
	| opt_construct_name_colon PLAINDO end_spec intonlyon 
          dotarget intonlyoff opt_comma do_var EQUAL expr COMMA expr
	    {
	      if ( !do_name_err ) {
		if ($5 && $5->labdefined)
		  error ("no backward DO loops");
	        $$ = make_do(FOR_NODE, $5, $8, $10, $12, 0);
	        $$->node.fields.e3 = $1;
	      }
	    }
	| opt_construct_name_colon PLAINDO end_spec intonlyon 
          dotarget intonlyoff opt_comma do_var EQUAL expr COMMA expr COMMA expr
	    {
	      if ( !do_name_err ) {
		if ($5 && $5->labdefined)
		  error ("no backward DO loops");
	        $$ = make_do(FOR_NODE, $5, $8, $10, $12, $14);
 	        $$->node.fields.e3 = $1;
	      }
	    }

/*------------ PCF Fortran disabled -----
	| forall
-----------------------------------------*/

	| ENDUNIT end_spec
	    { PTR_STMT stmt;

	      $$ = make_stmt (CONTROL_END, 0, 0, 0, 0) ; 
              // bind() ;
	      stmt = cur_proc () ;
	      if ((stmt->variant() == FUNC_HEDR) 
                  || (stmt->variant() == PROC_HEDR)
	          || (stmt->variant() == PROG_HEDR)
                  || (stmt->variant() == BLOCK_DATA)) {
		 position++ ;
		 pop_scope_level (stmt) ;
		 while (pred_stmt && pred_stmt != stmt) {
		   switch (pred_stmt->stmt_variant()) {
		   case FOR_NODE:
		     erri ("DO statement at line %d has badly positioned end-point",
                           pred_stmt->g_line) ;
                     break ;
		   case IF_NODE:
	 	   case ELSEIF_NODE:
		     erri ("IF statement at line %d has a missing or badly-nested ENDIF",
                           pred_stmt->g_line) ;
                     break ;
		   default:
		     erri ("block at line %d has badly positioned end-point",
                           pred_stmt->g_line) ;
                     break ;
		   }
		   pred_stmt = pred_stmt->control_parent ;
		 }
	       }
	      else 
                 error ("unexpected END statement") ;
            }
	| ENDDO end_spec
           { $$ = make_stmt (CONTROL_END, 0, 0, 0, 0); }

	  /* end module is bit different. So shouldn't have end_spec. */
	  /* It may be better to move it to spec. */
	| ENDMODULE 
          { $$ = make_stmt (CONTROL_END, 0, 0, 0, 0); 
	    // bind() ;
	    pop_scope_level (pred_stmt);
	    position = IN_OUTSIDE;
          }
	| opt_construct_name_colon DOWHILE end_spec needkeyword intonlyon 
          dotarget intonlyoff WHILE expr
          { 
	     if (!do_name_err ) {
	       if ($6 && $6->labdefined)
		 error ("no backward DO WHILE loops");

	       $$ = make_stmt (WHILE_NODE, 0, 0, 0, 0) ;
	       $$->node.for_node.doend = $6 ;
	       $$->node.while_node.condition = $9 ;
	       $$->node.fields.e3 = $1 ;

               //fgg added on 2000/12/11
               needkwd = NO ;
	     }
           }
	| CDOALL do_var EQUAL expr COMMA expr
	        {$$ = make_do(CDOALL_NODE, 0, $2, $4, $6, 0);
                 push_scope_level ($$, NO);
                }
	| CDOALL do_var EQUAL expr COMMA expr COMMA expr
	        {$$ = make_do(CDOALL_NODE, 0, $2, $4, $6, $8);
                 push_scope_level ($$, NO);
                }
	| SDOALL do_var EQUAL expr COMMA expr
	        {$$ = make_do(SDOALL_NODE, 0, $2, $4, $6, 0);
                 push_scope_level ($$, NO);
                }
	| SDOALL do_var EQUAL expr COMMA expr COMMA expr
	        {$$ = make_do(SDOALL_NODE, 0, $2, $4, $6, $8);
                 push_scope_level ($$, NO);
                }
	| DOACROSS do_var EQUAL expr COMMA expr
	        {$$ = make_do(DOACROSS_NODE, 0, $2, $4, $6, 0);
                 push_scope_level ($$, NO);
                }
	| DOACROSS do_var EQUAL expr COMMA expr COMMA expr
	        {$$ = make_do(DOACROSS_NODE, 0, $2, $4, $6, $8);
                 push_scope_level ($$, NO);
                }
	| CDOACROSS do_var EQUAL expr COMMA expr
	        {$$ = make_do(CDOACROSS_NODE, 0, $2, $4, $6, 0);
                 push_scope_level ($$, NO);
                }
	| CDOACROSS do_var EQUAL expr COMMA expr COMMA expr
	        {$$ = make_do(CDOACROSS_NODE, 0, $2, $4, $6, $8);
                 push_scope_level ($$, NO);
                }
	| LOOP
          {
	   make_loop();
	   $$ = 0;
	  }
	| enddoall
	    { $$ = make_enddoall();
              pop_scope_level (pred_stmt);
            }
	| logif iffable
	    { 
	      thiswasbranch = NO;
	      $$ = make_logif ($1, $2);
	    }
	| logif THEN
	    {
              $$ = $1 ;
            }
	| CONSTRUCT_ID COLON logif THEN
	    { PTR_SYMB s;
	      PTR_EXPR l;

	      s = make_local_entity(cur_scope()->look_up_ftn_sym ((*$1)()), 
				    CONSTRUCT_NAME, global_default, LOCAL);
              l = make_expr ( VAR_REF, 0, 0, s);
	      /* $$ = make_if ($3); */
	      $3->node.fields.e3 = l;
              $$ = $3;
            }
	| ELSEIF end_spec LEFTPAR expr RIGHTPAR THEN
	    { $$ = make_elseif ($4); lastwasbranch = NO; }
	| ELSE end_spec
	    { make_else(); lastwasbranch = NO; $$ = 0; }
	| ENDIF end_spec
	    { make_endif (); $$ = 0; }
	| case
	    { $$ = $1; }
	| CYCLE opt_construct_name
	    { $$ = make_stmt (CYCLE_STMT, $2, 0, 0, 0); }
	| EXIT opt_construct_name
	    { $$ = make_stmt (EXIT_STMT, $2, 0, 0, 0); }
	| CONTAINS
	    { $$ = make_stmt (CONTAINS_STMT, 0, 0, 0, 0); }

/*---------------- PCF Fortran Disabled --------------------
	| pcf
	    { $$ = $1; }
-----------------------------------------------------------*/

/*---------------- Convex Fortran Disabled -----------------
	| cvx
	    { $$ = $1; }
-----------------------------------------------------------*/
	;

case:	CASE case_selector opt_construct_name
	    { $$ = make_stmt (SWITCH_NODE, $3, $2, 0, 0); }
	| DEFAULT_CASE opt_construct_name
	    { PTR_EXPR p;

	      p = make_expr ( DEFAULT_EXPR, 0, 0, 0); 
	      $$ = make_stmt (SWITCH_NODE, $2, p, 0, 0); }
	| ENDSELECT opt_construct_name
	    { $$ = make_stmt (CONTROL_END, $2, 0, 0, 0); }
	| opt_construct_name_colon SELECT needkeyword CASE LEFTPAR expr RIGHTPAR
	    { $$ = make_stmt (CASE_NODE, 0, $6, 0, $1) ; }
	;

case_selector: LEFTPAR case_value_range_list RIGHTPAR
	         { $$ = $2; }
/*	| DEFAULT_CASE
	    { $$ = make_expr ( DEFAULT_CASE, 0, 0, 0); }
*/	;

case_value_range: expr
	            { $$ = $1; }
	| expr COLON
	    { $$ = make_expr ( DDOT, $1, 0, 0); }
	| COLON expr
	    { $$ = make_expr ( DDOT, 0, $2, 0); }
	| expr COLON expr
	    { $$ = make_expr ( DDOT, $1, $3, 0); }
	;

case_value_range_list: case_value_range
	                 { $$ = make_expr ( EXPR_LIST, $1, 0, 0); }
	| case_value_range_list COMMA case_value_range
	    { PTR_EXPR p;
	      
	      p = make_expr ( EXPR_LIST, $3, 0, 0);
	      $1->append (p) ;
	    }
	;

opt_construct_name:
	    { $$ = 0; }
	| name
	    { $$ = make_local_entity($1, CONSTRUCT_NAME, global_default,
                                     LOCAL); } 

	;

opt_construct_name_colon:
	    { $$ = 0; }
	| CONSTRUCT_ID COLON
	    { PTR_SYMB s;

	      s = make_local_entity(cur_scope()->look_up_ftn_sym ((*$1)()), 
				    CONSTRUCT_NAME, global_default, LOCAL);
              $$ = make_expr ( VAR_REF, 0, 0, s);
            }
	;

logif:	  LOGICALIF end_spec LEFTPAR expr RIGHTPAR
	    { $$ = make_if ($4); }
	;

enddoall: ENDCDOALL
        | ENDSDOALL
	| ENDDOACROSS
	| ENDCDOACROSS
	;

do_var:	  name
	    { 
              $$ = make_scalar($1, 0, LOCAL);
            }
	;


dospec:	  name EQUAL expr COMMA expr
	    { PTR_SYMB s;
              PTR_EXPR l;

              s = make_scalar($1, 0, LOCAL);
	      if (s->variant() != CONST_NAME) {
                l = make_expr ( SEQ, make_expr ( DDOT, $3, $5, 0),
                              0, 0);
		$$ = make_expr (IOACCESS, 0, l, s);
		do_name_err = NO;
	      }
	      else {
		error ("symbolic constant not allowed as DO variable");
		do_name_err = YES;
	      }
	    }
	| name EQUAL expr COMMA expr COMMA expr
	    { PTR_SYMB s;
              PTR_EXPR l;

              s = make_scalar($1, 0, LOCAL);
	      if ( s->variant() != CONST_NAME ) {
                l = make_expr ( SEQ, make_expr ( DDOT, $3, $5, 0), $7,
                              0);
		$$ = make_expr (IOACCESS, 0, l, s);
		do_name_err = NO;
	      }
	      else {
		error ("symbolic constant not allowed as DO variable");
		do_name_err = YES;
	      }
	    }
	;


dotarget:   { $$ = 0; }
          | INT_CONSTANT
            {
	       $$  = make_label ($1) ;
	    }
       ;


whereable: WHERE_ASSIGN LEFTPAR expr RIGHTPAR lhs EQUAL expr
           { $$ = make_stmt (WHERE_NODE, 0, $3, $5, $7); }
	| ENDWHERE
           { make_endwhere(); $$ = 0; }
	| ELSEWHERE
           { make_elsewhere(); lastwasbranch = NO; $$ = 0; }
	| WHERE LEFTPAR expr RIGHTPAR 
           { $$ = make_stmt (WHERE_BLOCK_STMT, 0, $3, 0, 0); }
	;

iffable:  let expr EQUAL expr
          /* let lhs EQUAL expr */
           { PTR_EXPR p, q;
             PTR_SYMB s1, s2, s3, arg_list;
	     PTR_HASH hash_entry;

	     if (cur_scope()->just_look_up_ftn_sym ("=") != 0) {
	        p = intrinsic_op_node("=", ASSGN_OP, $2, $4);
   	        $$ = make_stmt (OVERLOADED_ASSIGN_STAT, 0, p, $2, $4);
             }	       
             else if ($2->variant() == FUNC_CALL) {
	         q = $2;
#ifdef DEBUG_NAMES
		 printf ("----- parser reducing 'FUNC_CALL() = expr' -----\n\n") ;
		 cur_scope()->dump(1); 
#endif DEBUG_NAMES
  	         $2->variant(STMTFN_DECL) ;
		 /* $2->node.fields.e2 = $4; */
		 s1 = $2->node.fields.e1->node.fields.e1
		        ->node.fields.symbol;
	         s1->scope = cur_scope();
 	         $$ = make_stmt (STMTFN_STAT, 0, $2, 0, 0);
	         push_scope_level ($$, NO);
		 if (s1) 
                 {
	            arg_list = 0;
                    p = $2->node.fields.e1;
                    while (p != 0)
                    {
		    /*   if (p->node.fields.e1->variant() != VAR_REF) {
			  errstr("illegal statement function %s.", 
			          $2->node.fields.symbol->ident());
			  break;
		       } 
                    */
	               hash_entry = cur_scope()->look_up_ftn_sym (p->node.fields.e1->
                                        node.fields.symbol->parent->ident());
	               s3 = make_scalar(hash_entry, s1->type, IO);
	               if (arg_list == 0) 
                          s2 = arg_list = s3;
             	       else 
                       {
                          s2->id_list = s3;
                          s2 = s3;
                       }
                       p = p->node.fields.e2;
                    }
  		    $2->node.fields.e1 = $4;
		    $2->node.fields.symbol->install_param_list
		      (arg_list, 0, FUNCTION_NAME);
	            pop_scope_level ($$);
		 }
		 else
		    errstr("illegal statement function declaration %s.", 
			   $2->node.fields.symbol->ident());
	     }
	     else
		$$ = make_stmt (ASSIGN_STAT,0, $2, $4, 0);
	  }
	| POINTERLET name POINT_TO lhs
	    { PTR_SYMB s;
	
	      s = make_scalar($2, 0, LOCAL);
  	      $$ = make_stmt (POINTER_ASSIGN_STAT, s, $4, 0, 0);
	    }
/*        | let lhs EQUAL vec
		
	       { if ($2->variant() == ARRAY_REF)
		    $$ = make_stmt (ASSIGN_STAT,0, $2, $4, 0);
                 else error ("Constructor reference");
		}
*/	| ASSIGN end_spec label TO name
	    { PTR_SYMB p;

	      p = make_scalar($5, 0, LOCAL);
	      p->symb_variant(LABEL_VAR) ;
  	      $$ = make_stmt (ASSLAB_STAT, p, $3,0,0);
            }
	| CONTINUE end_spec
	    { $$ = make_stmt (CONT_STAT,0,0,0,0); }
	| goto
	| io
	    { inioctl = NO; }
	| ARITHIF end_spec LEFTPAR expr RIGHTPAR label COMMA label COMMA label
	    { PTR_EXPR	p;

	      p = make_expr (EXPR_LIST, $10, 0, 0);
	      p = make_expr (EXPR_LIST, $8, p, 0);
	      $$= make_stmt (ARITHIF_NODE, 0, $4,
			    make_expr (EXPR_LIST, $6, p, 0), 0);
	      thiswasbranch = YES;
            }
	| call
	    {
	      $$ = $1->subroutine_call(0);
/*	      match_parameters($1, 0);
	      $$= make_stmt (PROC_STAT, $1, 0, 0, 0);
*/	      endioctl(); 
            }
	| call LEFTPAR RIGHTPAR
	    {
	      $$ = $1->subroutine_call(0);
/*	      match_parameters($1, 0);
	      $$= make_stmt (PROC_STAT,$1,0,0,0);
*/	      endioctl(); 
	    }
	| call LEFTPAR callarglist RIGHTPAR
	    {
	      $$ = $1->subroutine_call($3);
/*	      match_parameters($1, $3);
	      $$= make_stmt (PROC_STAT,$1,$3,0,0);
*/	      endioctl(); 
	    }
	| RETURN end_spec opt_expr
	    {
	      $$ = make_stmt (RETURN_STAT,0,$3,0,0);
	      thiswasbranch = YES;
	    }
	| stop end_spec opt_expr
	    {
	      $$ = make_stmt ($1,0,$3,0,0);
	      thiswasbranch = ($1 == STOP_STAT);
	    }
	;

let:	  LET
	    { if (parstate == OUTSIDE)
		{ PTR_STMT p;

		  p = make_stmt (PROG_HEDR, 
				 make_program(cur_scope()->look_up_ftn_sym ("MAIN")), 
				 0, 0, 0);
		  attach_statement (p) ;
/* fgg remarked on 2000/12/11 since it's wrong ?!
                  push_scope_level (p, NO);
*/
		  position = IN_PROC; 
		}
		if (parstate < INDATA) enddcl();
		parstate = INEXEC;
		yystno = 0;
	      }
	;

goto:	  PLAINGOTO end_spec label
	    {
	      $$=make_stmt (GOTO_NODE,0,0,0,(PTR_EXPR)$3);
	      thiswasbranch = YES;
	    }
	| ASSIGNGOTO end_spec name
	    { PTR_SYMB p;

	      if ($3->id_attr)
		p = $3->id_attr;
	      else {
	        p = make_scalar($3, 0, LOCAL);
		p->symb_variant(LABEL_VAR) ;
	      }

	      if (p->variant() == LABEL_VAR) {
		  $$ = make_stmt (GOTO_NODE,0,0,0,0);
		  thiswasbranch = YES;
	      }
	      else {
		  error ("must go to label or assigned variable");
		  $$ = 0;
	      }
	    }
	| ASSIGNGOTO end_spec name opt_comma LEFTPAR labellist RIGHTPAR
	    { PTR_SYMB p;

	      if ($3->id_attr)
		p = $3->id_attr;
	      else {
	        p = make_scalar($3, 0, LOCAL);
		p->symb_variant(LABEL_VAR) ;
	      }

	      if (p->variant() == LABEL_VAR) {
		 $$ = make_stmt (ASSGOTO_NODE,p,$6,0,0);
		 thiswasbranch = YES;
	      }
	      else {
		error ("must go to label or assigned variable");
		$$ = 0;
	      }
	    }
	| COMPGOTO end_spec LEFTPAR labellist RIGHTPAR opt_comma expr
	    { $$ = make_stmt (COMGOTO_NODE, 0, $4, $7, 0); }
	;

opt_comma:
	| COMMA
	;

call:	  CALL end_spec name 
	    { $$ = make_procedure($3, YES); }
	;

callarglist:  in_ioctl callarg
	    { $$ = set_ll_list ($2, 0, EXPR_LIST);
              endioctl();
            }
	| callarglist COMMA in_ioctl callarg
	    { $$ = set_ll_list ($1, $4, EXPR_LIST); }
	;

callarg:  expr
	    { $$ = $1; }
	| nameeq expr
	    { $$  = make_expr ( KEYWORD_ARG, $1, $2, 0); }
	| ASTER label
	    { $$ = make_expr (LABEL_REF,0,0,(PTR_SYMB)$2); }
	;

stop:	  PAUSE	{ $$ = PAUSE_NODE; }
	| STOP	{ $$ = STOP_STAT; }
	;


end_spec:
	    { if (parstate == OUTSIDE)
		{ PTR_STMT p;
		  
		  p = make_stmt (PROG_HEDR, 
				 make_program(cur_scope()->look_up_ftn_sym ("MAIN")), 
				 0, 0, 0);
		  attach_statement (p) ;
/* fgg remarked on 2000/12/11 since it's wrong ?!
                  push_scope_level (p, NO);
*/
		  position = IN_PROC; 
		}
	      if (parstate < INDATA) enddcl();
	      parstate = INEXEC;
	      yystno = 0;
	    }
        ;

intonlyon:
	    { intonly = YES; }
	;

intonlyoff:
	     { intonly = NO; }

	;


/*----------------------------------------------------------+
 |                                                          |
 |           Grammar for INPUT/OUTPUT STATEMENTS            |
 |                                                          |
 +----------------------------------------------------------*/

io:	  iofmove ioctl
		{ $1->node.fields.e2 = $2;
		  $$ = $1; }
	| iofmove unpar_fexpr
		{ PTR_EXPR p, q;

		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  q->node.string_val = new string (unit_string) ;
		  q->type = global_string;
		  p = make_expr ( SPEC_PAIR, q, $2, 0);
		  $1->node.fields.e2 = p;
		  endioctl();
		  $$ = $1; }
	| iofmove ASTER
		{ PTR_EXPR p, q, r;

		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (star_string) ;
		  p->type = global_string;
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  q->node.string_val = new string (unit_string) ;
		  q->type = global_string;
		  r = make_expr ( SPEC_PAIR, p, q, 0);
		  $1->node.fields.e2 = r;
		  endioctl();
		  $$ = $1; }
 	| iofmove DASTER
		{ PTR_EXPR p, q, r;

		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (dbl_star_string) ;
		  p->type = global_string;
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  q->node.string_val = new string (unit_string) ;
		  q->type = global_string;
		  r = make_expr ( SPEC_PAIR, p, q, 0);
		  $1->node.fields.e2 = r;
		  endioctl();
		  $$ = $1; }
	| iofctl ioctl
		{ $1->node.fields.e2 = $2;
		  $$ = $1; }

	| read ioctl
		{ $1->node.fields.e2 = $2;
		  $$ = $1; }
	| read infmt
		{ $1->node.fields.e2 = $2;
		  $$ = $1; }
	| read ioctl inlist
		{ $1->node.fields.e2 = $2;
		  $1->node.fields.e1 = $3;
		  $$ = $1; }
	| read infmt COMMA inlist
		{ $1->node.fields.e2 = $2;
		  $1->node.fields.e1 = $4;
		  $$ = $1; }
	| write ioctl
		{ $1->node.fields.e2 = $2;
		  $$ = $1; }
	| write ioctl outlist
		{ $1->node.fields.e2 = $2;
		  $1->node.fields.e1 = $3;
		  $$ = $1; }
	| print
		{ $$ = $1; }
	| print COMMA outlist
		{ $1->node.fields.e1 = $3;
		  $$ = $1; }
	;


iofmove:   fmkwd end_spec in_ioctl
		{ $$ = $1; }
	;

fmkwd:	  BACKSPACE
           {$$ = make_stmt (BACKSPACE_STAT, 0, 0, 0, 0);}
	| REWIND
           {$$ = make_stmt (REWIND_STAT, 0, 0, 0, 0);}
	| ENDFILE
           {$$ = make_stmt (ENDFILE_STAT, 0, 0, 0, 0);}
        | SKIPPASTEOF
           {$$ = make_stmt (SKIPPASTEOF_NODE, 0, 0, 0, 0); }
	;

iofctl:  ctlkwd end_spec in_ioctl
		{ $$ = $1; }
	;

ctlkwd:	  INQUIRE
           {$$ = make_stmt (INQUIRE_STAT, 0, 0, 0, 0);}
	| OPEN
           {$$ = make_stmt (OPEN_STAT, 0, 0, 0, 0);}
	| CLOSE
           {$$ = make_stmt (CLOSE_STAT, 0, 0, 0, 0);}
	;

infmt:	  unpar_fexpr
		{ PTR_EXPR p;
		  PTR_EXPR q;

		  if ($1->variant() == INT_VAL)
 	          {
		        PTR_FTN_LABEL r;

			r = make_label ((long) $1->node.ival);
			p = make_expr (LABEL_REF, r);
		  }
		  else p = $1; 
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  q->node.string_val = new string (fmt_string) ;
		  q->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, q, p, 0);
		  endioctl();
		}
	| ASTER
		{ PTR_EXPR p;
		  PTR_EXPR q;

		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (star_string) ;
		  p->type = global_string;
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  q->node.string_val = new string (fmt_string) ;
		  q->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, q, p, 0);
		  endioctl();
		}
	;


ioctl:	LEFTPAR fexpr RIGHTPAR
		{ PTR_EXPR p;

		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (unit_string) ;
		  p->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, p, $2, 0);
		  endioctl();
		}
	| LEFTPAR ctllist RIGHTPAR
		{ 
		  $$ = $2;
		  endioctl();
		 }
	;

ctllist:  ioclause
		{ $$ = $1; }
	| ctllist COMMA still_ioctl ioclause
		{ $$ = set_ll_list ($1, $4, EXPR_LIST);}
	;

ioclause:  fexpr %prec IDENT
		{ PTR_EXPR p ;
		  PTR_EXPR q = make_expr (KEYWORD_VAL, 0, 0, 0) ;
		  q->type = global_string;
 
		  ++nioctl ;
		  if (nioctl == 1)
		    {
		      p = $1 ;
		      q->node.string_val = new string (unit_string) ;
		    }
		  else if ( (nioctl == 2) && ($1->variant() == INT_VAL) )
		    {
		      PTR_FTN_LABEL r;
		      
		      r = make_label ((long) $1->node.ival);
		      p = make_expr ( LABEL_REF, r);
		      q->node.string_val = new string (fmt_string) ;
		    }
		  else if ((nioctl == 2) && ($1->variant() == VAR_REF))
		    {
		      // namelist (4, llist).
		      // This should be implemented more correctly
		      // llist - will be defined as a variable,
		      // although it is a reference to namelist.
		      // Current implementati4on just prevents crashing
		      // (FGG) 
		      //$1->Tag(KEYWORD_VAL); // ??? it shouldn't be VAR_REF ! ???
		      p = make_expr (KEYWORD_VAL, 0, 0, 0) ;
		      p->node.string_val = new string ($1->node.fields.symbol->ident()) ;
		      p->type = global_string;
		      q->node.string_val = new string (nml_string) ;
		    }
		  else
		    {
		      p = $1; 
		      q->node.string_val = new string (fmt_string) ;
		    }
		  $$ = make_expr ( SPEC_PAIR, q, p, 0);
		}
	| ASTER
		{ PTR_EXPR p;
		  PTR_EXPR q;

		  nioctl++;
		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (star_string) ;
		  p->type = global_string;
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  if (nioctl == 1)
		    q->node.string_val = new string (unit_string) ;
		  else
		    q->node.string_val = new string (fmt_string) ;
		  q->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, q, p, 0);
		}
	| DASTER
		{ PTR_EXPR p;
		  PTR_EXPR q;

		  nioctl++;
		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (dbl_star_string) ;
		  p->type = global_string;
		  q = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  if (nioctl == 1)
		    q->node.string_val = new string (unit_string) ;
		  else
		    q->node.string_val = new string (fmt_string) ;
		  q->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, q, p, 0);
		}
	| nameeq expr
		{ 
		  PTR_EXPR p;
		  char *q;

		  q = $1->node.string_val->operator()();
  		  if ((strcasecmp(q, "end") == 0) || (strcasecmp(q, "err") == 0))
 	          {
		        PTR_FTN_LABEL r;

			r = make_label ((long) $2->node.ival);
			p = make_expr ( LABEL_REF, r);
		  }
		  else p = $2;

		  $$ = make_expr ( SPEC_PAIR, $1, p, 0); }
	| nameeq ASTER
		{ PTR_EXPR p;
		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (star_string) ;
		  p->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, $1, p, 0);
		}
	| nameeq DASTER
		{ PTR_EXPR p;
		  p = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  p->node.string_val = new string (dbl_star_string) ;
		  p->type = global_string;
		  $$ = make_expr ( SPEC_PAIR, $1, p, 0);
		}
	;

nameeq:  NAMEEQ
		{ $$ = make_expr ( KEYWORD_VAL, 0, 0, 0);
		  $$->node.string_val = $1 ;
		  $$->type = global_string;
	        }
	;


read:	  READ end_spec in_ioctl
           {$$ = make_stmt (READ_STAT, 0, 0, 0, 0);}
	;


write:	  WRITE end_spec in_ioctl
           {$$ = make_stmt (WRITE_STAT, 0, 0, 0, 0);}
	;


print:	  PRINT end_spec fexpr in_ioctl
           {
	    PTR_EXPR p, q, l;

	    if ($3->variant() == INT_VAL)
		{
		        PTR_FTN_LABEL r;

			r = make_label ((long) $3->node.ival);
			p = make_expr ( LABEL_REF, r);
		}
	    else p = $3;
	    
            q = make_expr ( KEYWORD_VAL, 0, 0, 0);
	    q->node.string_val = new string (fmt_string) ;
            q->type = global_string;
            l = make_expr ( SPEC_PAIR, q, p, 0);

            $$ = make_stmt (PRINT_STAT, 0, 0, l, 0);
	    endioctl();
	   }	
	| PRINT end_spec ASTER in_ioctl
	   { PTR_EXPR p, q, r;
		
	     p = make_expr ( KEYWORD_VAL, 0, 0, 0);
	     p->node.string_val = new string (star_string) ;
	     p->type = global_string;
	     q = make_expr ( KEYWORD_VAL, 0, 0, 0);
	     q->node.string_val = new string (fmt_string) ;
             q->type = global_string;
             r = make_expr ( SPEC_PAIR, q, p, 0);
	     $$ = make_stmt (PRINT_STAT, 0, 0, r, 0);
	     endioctl();
           }
	;


inlist:	  inelt
		{ $$ = set_ll_list ($1, 0, EXPR_LIST);}
	| inlist COMMA inelt
		{ $$ = set_ll_list ($1, $3, EXPR_LIST);}
	;

inelt:	  lhs
		{ $$ = $1; }
	| LEFTPAR inlist COMMA dospec RIGHTPAR
		{
		  $4->node.fields.e1 = $2;
		  $$ = $4;
		}
	;

outlist:  uexpr
		{ $$ = set_ll_list ($1, 0, EXPR_LIST);}
	| other
		{ $$ = $1; }
	| out2
		{ $$ = $1; }
	;

out2:	  uexpr COMMA uexpr
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	| uexpr COMMA other
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	| other COMMA uexpr
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	| other COMMA other
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	| out2  COMMA uexpr
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	| out2  COMMA other
		{ $$ = set_ll_list ($1, $3, EXPR_LIST); $$->type = $1->type;}
	;

other:	  complex_const
		{ $$ = set_ll_list ($1, 0, EXPR_LIST);
	          $$->type = global_complex; }
	| LEFTPAR expr RIGHTPAR
		{ $$ = set_ll_list ($2, 0, EXPR_LIST);
                  $$->type = $2->type; }
	| LEFTPAR uexpr COMMA dospec RIGHTPAR
		{
		  $4->node.fields.e1 = $2;
		  $$ = set_ll_list ($4, 0, EXPR_LIST);
                  $$->type = $2->type; 
		}
	| LEFTPAR other COMMA dospec RIGHTPAR	
		{
		  $4->node.fields.e1 = $2;
		  $$ = set_ll_list ($4, 0, EXPR_LIST);
                  $$->type = $2->type; 
		}
	| LEFTPAR out2  COMMA dospec RIGHTPAR
		{
		  $4->node.fields.e1 = $2;
		  $$ = set_ll_list ($4, 0, EXPR_LIST);
                  $$->type = $2->type; 
		}
	;

in_ioctl:
		{ startioctl(); }
	;

still_ioctl:
                { inioctl = YES; }
        ;

/*
 * used by I/O statement 
 */
fexpr:	unpar_fexpr
            { $$ = $1; }
	| LEFTPAR fexpr RIGHTPAR
	    { 
/* was before fgg changed on 2000/12/11 to deal with () correctly
              $$ = $2; 
*/
	      $$ = make_expr ( PAREN_OP, $2, 0, 0) ;
	      $$->set_expr_type();
            }
	;

unpar_fexpr:	  lhs
        { $$ = $1; }
	| simple_const
        { $$ = $1; }
	| fexpr addop fexpr   %prec PLUS
	    {
	      $$ = make_expr ($2, $1, $3, 0);
	      $$->set_expr_type();
	    }
	| fexpr ASTER fexpr
	    {
	      $$ = make_expr (MULT_OP, $1, $3, 0);
	      $$->set_expr_type();
	    }
	| fexpr SLASH fexpr
	    {
	      $$ = make_expr (DIV_OP, $1, $3, 0);
	      $$->set_expr_type ();
	    }
	| fexpr DASTER fexpr
	    {
	      $$ = make_expr (EXP_OP, $1, $3, 0);
	      $$->set_expr_type ();
	    }
	| addop fexpr  %prec ASTER
	    {
	      if ($1 == SUBT_OP)
		{
		  $$ = make_expr (SUBT_OP, $2, 0, 0);
		  $$->set_expr_type ();
		}
	      else	$$ = $2;
	    }
	| fexpr DSLASH fexpr
	    {
	      $$ = make_expr (CONCAT_OP, $1, $3, 0);
	      $$->set_expr_type ();
	    }
        | IDENTIFIER EQUAL expr  
           { $$ = 0; }
	;



/*----------------------------------------------------------+
 |                                                          |
 |                  (FORTRAN-D EXTENSIONS)                  |
 |                                                          |
 +----------------------------------------------------------*/

/*----------------- Currently Unused ------------------------

decomposition: DECOMPOSITION opt_comma name in_dcl dims lengspec
	    { PTR_SYMB s;
	      PTR_EXPR q, r;


	      if (! $5) {
		error ("No Dimensions in Decomposition Statement");
		$$ = 0;
	      }
	      s = make_array($3, 0, $5, ndim, LOCAL);
	      s->attr = s->attr | DIMENSION_BIT;
	      q = make_expr (ARRAY_REF, $5, 0, s);
	      s->type->node.ar_decl.ranges = q;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $$ = make_stmt (DECOMPOSITION_STMT, 0, r, 0, 0);
	    }
	| decomposition COMMA name dims lengspec
           {  PTR_SYMB s;
	      PTR_EXPR q, r;

	      if (! $4) {
		error ("No Dimensions in Decomposition Statement");
		$$ = 0;
	      }
	      s = make_array($3, 0, $4, ndim, LOCAL);
	      s->attr = s->attr | DIMENSION_BIT;
	      q = make_expr (ARRAY_REF, $4, 0, s);
	      s->type->node.ar_decl.ranges = q;
	      r = make_expr (EXPR_LIST, q, 0, 0);
	      $1->node.fields.e1->append (r) ;
	}
	;

forall:	  FORALLDO end_spec intonlyon dotarget intonlyoff opt_comma do_var
          EQUAL expr COMMA expr on_clause
	    {
	      if ( !do_name_err ) {
		if ($4 && $4->labdefined)
		  error ("no backward DO loops");
	        $$ = make_do(FORALL_NODE, $4, $7, $9, $11, 0);	
	        $$->node.fields.e3 = $12;
      }
	    }
	| FORALLDO end_spec intonlyon dotarget intonlyoff opt_comma do_var
          EQUAL expr COMMA expr COMMA expr on_clause
	    {
	      if ( !do_name_err ) {
		if ($4 && $4->labdefined)
		  error ("no backward DO loops");
	        $$ = make_do(FORALL_NODE, $4, $7, $9, $11, $13);
	        $$->node.fields.e3 = $14;
	      }
	    }
	;

on_clause: needkeyword keywordoff
	    { $$ = 0; }
	| needkeyword ON expr
	    { $$ = $3; }
	;

------------------------------------------------------------*/


/*----------------------------------------------------------+
 |                                                          |
 |                    (PCF EXTENSIONS)                      |
 |                                                          |
 +----------------------------------------------------------*/

/*----------------- Currently Unused ------------------------

pcf_dcl:  sync_dcl
        ;

sync_dcl: GUARDS name_list LEFTPAR object_name_list RIGHTPAR
          { $$ = make_stmt (GUARDS_NODE, 0, $2, $4, 0); }
        ;

name_list: name
	   { PTR_EXPR p;

             p = check_and_install($1, 0, 0);
             $$ = set_ll_list(p, 0, EXPR_LIST);
            }
	| name_list COMMA name
	   { PTR_EXPR p;

             p = check_and_install($3, 0, 0);
             $$ = set_ll_list ($1, p, EXPR_LIST);
           }
         ;

pcf:	  parallel_do
	| parallel_sections
        | parallel_region
        | pregion_do
	| par_end_stmts
          { $$ = make_enddoall(); }
        | unstruc_lock_sync
	| critical_section
        | event_sync
        | sequence_sync
        | sync_performance
        | data_sharing
        | single_process_section
	;

parallel_do: PARALLELDO end_spec opt_qualities intonlyon dotarget intonlyoff 
	     opt_comma do_var EQUAL expr COMMA expr opt_comma_expr
	    {
	      if ( !do_name_err ) {
	         if ($5 && $5->labdefined)
		  error ("no backward DO loops");
	        $$ = make_pardo(PARDO_NODE, $5, $8, $10, $12, $13, $3);
	      }
	    }
	;

parallel_sections: PARALLELSECTIONS opt_qualities
            { $$ = make_stmt (PARSECTIONS_NODE, 0, $2, 0, 0);
              needkwd = NO;
            }
	| SECTIONWAIT LEFTPAR section_name_list RIGHTPAR
            { make_section(0, $3); needkwd= NO;
              $$ = 0; }
	| SECTION LEFTPAR section_name RIGHTPAR needkeyword WAIT LEFTPAR section_name_list RIGHTPAR
            { make_section($3, $8); needkwd = NO; 
              $$ = 0; }
	| SECTION LEFTPAR section_name RIGHTPAR needkeyword
            { make_section($3, 0); needkwd = NO; 
              $$ = 0; }
	| SECTION
            { make_section(0, 0); needkwd = NO;
              $$ = 0; }
        | ENDSECTIONS
            { make_section_extend(); $$ = 0; }
        | ENDPARALLELSECTIONS
            { make_section_extend(); $$ = 0; }
        | PSECTIONS opt_p_list
            { $$ = make_stmt (PSECTIONS_NODE, 0, $2, 0, 0); 
              if (extend_flag)
                 $$->idx = EXTEND_NODE;
              else $$->idx = 0;
              extend_flag = 0;
            }
        | ENDPSECTIONS
            { make_section_extend(); $$ = 0; }
        ;

section_name: name
            {  PTR_SYMB s;
         
             s = install_entry($1, HARD);
             s->set_type(global_default, LOCAL);
             $$ = make_expr ( VAR_REF, 0, 0, s);
             // $$ = make_expr ( EXPR_LIST, q, 0, 0);
           }
        ;

section_name_list: section_name
            { $$ = set_ll_list ($1, 0, EXPR_LIST); }
        | section_name_list COMMA section_name
            { $$ = set_ll_list ($1, $3, EXPR_LIST); }
        ;

parallel_region: PARALLEL opt_r_list
            { $$ = make_stmt (PARREGION_NODE, 0, $2, 0, 0); }
        | ENDPARALLEL
            { $$ = make_stmt (CONTROL_END, 0, 0, 0, 0); }
        ;

r_option: MAXPARALLEL EQUAL expr
          { $$ = make_expr ( MAXPARALLEL_OP, $3, 0, 0); }
        ;

r_list: needkeyword r_option
	   { $$ = $2; }
	| r_list COMMA needkeyword r_option
          { $$ = set_ll_list ($1, $4, EXPR_LIST); }
	;

opt_r_list:
          { $$ = 0; }
	| LEFTPAR r_list RIGHTPAR
          { $$ = $2; }
        ;

pregion_do: PDO end_spec opt_p_list intonlyon dotarget intonlyoff 
	     opt_comma do_var EQUAL expr COMMA expr opt_comma_expr
	    {
	      if ( !do_name_err ) {
		if ($5 && $5->labdefined)
		  error ("no backward DO loops") ;
	        $$ = make_pardo(PDO_NODE, $5, $8, $10, $12, $13, $3);
  	        if (extend_flag)
                   $$->idx = EXTEND_NODE;
	        else $$->idx = 0;
	        extend_flag = 0;
	      }
	    }
	| ENDPDO
	  { exit_group (0); $$ = 0; }
	| ENDEXTENDED
          { make_endextend(); $$ = 0; }
	| ENDEXTEND
          { make_endextend(); $$ = 0; }
	;

p_option: ORDERED
	  { $$ = make_expr ( ORDERED_OP, 0, 0, 0); }
	| EXTEND
	  { extend_flag = 1;
	    $$ = make_expr ( EXTEND_OP, 0, 0, 0); }
	| EXTENDED
	  { extend_flag = 1;
            $$ = make_expr ( EXTEND_OP, 0, 0, 0); }
	| MAXPARALLEL EQUAL expr
          { $$ = make_expr ( MAXPARALLEL_OP, $3, 0, 0); }
	;

p_list: needkeyword p_option
	   { $$ = $2; }
	| p_list COMMA needkeyword p_option
          { $$ = set_ll_list ($1, $4, EXPR_LIST); }
	;


opt_p_list: 
          { $$ = 0; }
	| LEFTPAR p_list RIGHTPAR
          { $$ = $2; }
	;

par_end_stmts: ENDPARALLELDO
        ;

opt_comma_expr: 
	{ $$ = 0; }
	| COMMA expr
          { $$ = $2; }
        ;

qual:	  ORDERED
	  { $$ = make_expr ( ORDERED_OP, 0, 0, 0); }
	| MAXPARALLEL EQUAL expr
          { $$ = make_expr ( MAXPARALLEL_OP, $3, 0, 0); }
	;

qual_list: needkeyword qual
	   { $$ = $2; }
	| qual_list COMMA needkeyword qual
          { $$ = set_ll_list ($1, $4, EXPR_LIST); }
	;


opt_qualities: 
          { $$ = 0; }
	| LEFTPAR qual_list RIGHTPAR
          { $$ = $2; }
	;

unstruc_lock_sync: LOCK LEFTPAR gate RIGHTPAR needkeyword opt_gaurd_list
          { $$ = make_stmt (LOCK_NODE, 0, $3, $6, 0);
            needkwd= NO;
          }
        | UNLOCK LEFTPAR gate RIGHTPAR needkeyword  opt_gaurd_list
          { $$ = make_stmt (UNLOCK_NODE, 0, $3, $6, 0);
            needkwd = NO;
          }
        ;
gate:     var
	  { $$ = $1; }
        ;

opt_gaurd_list: 
          { $$ = 0; }
        | GUARDS LEFTPAR object_name_list RIGHTPAR
          { $$ = $3; }
        ;
object:   var
          { $$ = $1; }
        | SLASH name SLASH
          { $$ = make_expr ( VAR_REF, 0, 0, 
                                                    install_entry($2, SOFT)); }
        ;

object_name_list: object
          { $$ = set_ll_list ($1, 0, EXPR_LIST); }
        | object_name_list COMMA object
          { $$ = set_ll_list ($1, $3, EXPR_LIST); }
        ;

critical_section: CRITICALSECTION
          { $$ = make_stmt (CRITSECTION_NODE, 0, 0, 0, 0); }
        | CRITICALSECTION LEFTPAR gate RIGHTPAR needkeyword opt_gaurd_list
          { $$ = make_stmt (CRITSECTION_NODE, 0, $3, $6, 0); 
            needkwd = NO;
          }
        | ENDCRITICALSECTION opt_gate
          { $$ = make_stmt (CONTROL_END, 0, $2, 0, 0); }
        ;

opt_gate: 
          { $$ = 0; }
        | LEFTPAR gate RIGHTPAR
          { $$ = $2; }

event_sync: POST LEFTPAR event RIGHTPAR needkeyword opt_gaurd_list
	  { $$ = make_stmt (POST_NODE, 0, $3, $6, 0);
            needkwd = NO;
          }
        | WAIT LEFTPAR event RIGHTPAR needkeyword opt_gaurd_list
	  { $$ = make_stmt (WAIT_NODE, 0, $3, $6, 0);
            needkwd = NO;
          }
        | CLEAR LEFTPAR event RIGHTPAR needkeyword opt_gaurd_list
	  { $$ = make_stmt (CLEAR_NODE, 0, $3, $6, 0);
            needkwd = NO;
          }
        ;

event:    gate
          { $$ = $1; }
        ;

sequence_sync: POST LEFTPAR seq COMMA expr RIGHTPAR needkeyword opt_gaurd_list
          { $$ = make_stmt (POSTSEQ_NODE, 0, $3, $5, $8); 
            needkwd = NO;
          }
 	| WAIT LEFTPAR seq COMMA expr RIGHTPAR needkeyword opt_gaurd_list
          { $$ = make_stmt (WAITSEQ_NODE, 0, $3, $5, $8); 
            needkwd = NO;
          }
	| SET LEFTPAR seq opt_seq_exprs34 RIGHTPAR needkeyword opt_gaurd_list
          { $$ = make_stmt (SETSEQ_NODE, 0, $3, $4, $7); 
            needkwd = NO;
          }
        ;

seq:      var
          { $$ = $1; }
        ;

opt_seq_exprs34:
          { $$ = 0; }
        | COMMA expr
          { $$ = $2; }
	| COMMA expr COMMA expr
          { $$ = make_expr ( DDOT, $2, $4, 0); }
        ;

sync_performance: ASSIGN LEFTPAR sync_list RIGHTPAR
          { $$ = make_stmt (ASSIGN_NODE, 0, $3, 0, 0); }
        | RELEASE LEFTPAR sync_list RIGHTPAR
          { $$ = make_stmt (RELEASE_NODE, 0, $3, 0, 0); }
        ;

sync_list: var
          { $$ = set_ll_list ($1, 0, EXPR_LIST); }
        | sync_list COMMA var
          { $$ = set_ll_list ($1, $3, EXPR_LIST); }
        ;

data_sharing: PRIVATE object_name_list
          { $$ = make_stmt (PRIVATE_NODE, 0, $2, 0, 0); }
        ;

pcf_common: SCOMMON
          { $$ = 0; }
        ;

single_process_section: SINGLEPROCESS
          { $$ = make_stmt (SINGLEPROCESS_NODE, 0, 0, 0, 0); }
        | ENDSINGLEPROCESS
          { $$ = make_stmt (CONTROL_END, 0, 0, 0, 0); }
        ;

-----------------------------------------------------------*/

%%


/*--------------------------------------------------------------------+
 |                                                                    |
 |                  FortranParser::stack_overflow                     |
 |                                                                    |
 +--------------------------------------------------------------------*/

void FortranParser::stack_overflow (char* msg, short** yys, int yysz, 
				    YYSTYPE** yyv, int yyvz, int* sz)

{
  if (*sz >= YYMAXDEPTH)
    {
      error ("parser stack limit exceeded") ;
      return ;
    }

  int newsize = *sz * 2 ;
  YYSTYPE* nv = new YYSTYPE [newsize] ;
  short* ns = new short [newsize] ;
  
  for (int i = 0; i < *sz; i++)
    {
      nv[i].token = (*yyv)[i].token ;
      nv[i].stmt_tag = (*yyv)[i].stmt_tag ;
      nv[i].expr_tag = (*yyv)[i].expr_tag ;
      nv[i].charv = (*yyv)[i].charv ;
      nv[i].ival = (*yyv)[i].ival ;
      nv[i].stringp = (*yyv)[i].stringp ;
      nv[i].bf_node = (*yyv)[i].bf_node ;
      nv[i].ll_node = (*yyv)[i].ll_node ;
      nv[i].symbol = (*yyv)[i].symbol ;
      nv[i].data_type = (*yyv)[i].data_type ;
      nv[i].hash_entry = (*yyv)[i].hash_entry ;
      nv[i].label = (*yyv)[i].label ;

      ns[i] = (*yys)[i] ;
    }

  if (*sz != YYINITDEPTH)
    {
      delete [] (*yys) ;
      delete [] (*yyv) ;
    }

  *yys = ns ;
  *yyv = nv ;
  *sz  = newsize ;
}
