//
// Class CTime for parsing time file (after running eos_time)
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CTime
{
  //Variables
  private double time0;
  private double time1;
  private double time2;

  private int i1;
  private int i2;

  private String err="";   

  //****************************************************************************
  //"Initialization"
  //
  //Input:  filename = Host name
  //
  //Return:  0 - Ok
  //        -1 - File not found
  //        -2 - Problems while reading file
  //        -3 - Can't close file
  //        -4 - Unknown file format

  public int Init(String filename)
  {
    //Read time    
    BufferedReader ftime;
    String str1;
    String str2;

    try
    {
      ftime=new BufferedReader(new FileReader(filename));
    }
    catch (IOException e1)
    {
      err="Error: Can't find file with time!";
      return -1; 
    }

    try
    {
      str1=ftime.readLine();
      str2=ftime.readLine();
    }
    catch (Exception e2)
    {
      try
      {
        ftime.close();
      }
      catch (Exception e3)
      {}
      err="Error: Problems while reading file with time!";
      return -2;
    }

    try
    {
      ftime.close();
    }
    catch (IOException e5)
    {
      err="Error: Can't close file with time!";
      return -3; 
    }

    time1=0;
    time2=0;

    try
    {
      time1=Double.valueOf(str1).doubleValue();
      time2=Double.valueOf(str2).doubleValue();
    }
    catch (NumberFormatException nfe1)
    {
      err="Error: Format of the file time is invalid!";
      return -4; 
    }

    time0=time1+time2;

    i1=-1;
    i2=-2;

    return 0;
  }

  //****************************************************************************
  public void Copy(CTime ctx)
  {
    time0=ctx.GetTime0();
    time1=ctx.GetTime1();
    time2=ctx.GetTime2();

    i1=ctx.GetAlign1();
    i2=ctx.GetAlign2();
  }

  //****************************************************************************
  public double GetTime0()
  {
    return time0;
  }

  //****************************************************************************
  public double GetTime1()
  {
    return time1;
  }

  //****************************************************************************
  public double GetTime2()
  {
    return time2;
  }

  //****************************************************************************
  public int Align(int i1a, int i2a)
  {
    i1=i1a;
    i2=i2a;

    String t1=CMisc.align_double(time1, i1, i2);
    String t2=CMisc.align_double(time2, i1, i2);

    try
    {
      time1=Double.valueOf(t1).doubleValue();;
      time2=Double.valueOf(t2).doubleValue();
    }
    catch (NumberFormatException nfe1)
    {
      return -1; 
    }

    time0=time1+time2;
    t1=CMisc.align_double(time0, i1, i2);

    try
    {
      time0=Double.valueOf(t1).doubleValue();;
    }
    catch (NumberFormatException nfe1)
    {
      return -1; 
    }

    return 0;
  }
 
  //****************************************************************************
  public int GetAlign1()
  {
    return i1;
  }

  //****************************************************************************
  public int GetAlign2()
  {
    return i2;
  }

  //****************************************************************************
  public String GetTimeStr0()
  {
    String x=String.valueOf(time0);

    if ((i1!=-1) && (i2!=-1))
    {
      x=CMisc.align_double(time0, i1, i2);
    }

    return x;
  }

  //****************************************************************************
  public String GetTimeStr1()
  {
    String x=String.valueOf(time1);

    if ((i1!=-1) && (i2!=-1))
    {
      x=CMisc.align_double(time1, i1, i2);
    }

    return x;
  }

  //****************************************************************************
  public String GetTimeStr2()
  {
    String x=String.valueOf(time2);

    if ((i1!=-1) && (i2!=-1))
    {
      x=CMisc.align_double(time2, i1, i2);
    }

    return x;
  }

  //****************************************************************************
  public String GetLastError()
  {
    String err1=err;
    err="";
    return err1;
  }
}
