//
// Process Instructions
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.net.*;
import java.io.*;
import java.util.*;

public class ProcessInstructions
{
  String f_tmp  =         "_eos_ac_tmp";
  String f_tmp1 =         "_eos_ac_tmp1";
  String f_tmp2 =         "_eos_ac_tmp2";
  String f_tmp_prj =      "_eos_prj_tmp";
  String str_llprof_add = "_llp";

  public ProcessInstructions(Socket sockfd, CGlobal g, CSIParse s)
  {
    String str="";
    String buf="";
    int buf1;
    char buf2;
    String xerr="";
    boolean enter=false;
    int misc;
    
    PrintWriter out;
    BufferedReader in;

    //Parameters
    int initiate=0;
    String str_dir="";
    String str_f_prj="";
    String str_f_prj1="";
    String str_f_diag_out="";
    String str_f_prof_out="";
    String str_f_tree_out="";
    String str_f_info="";
    String str_no_octave="";
    String str_step="";
    String str_opt="";
    String str_f_prj_out="";
    String str_f_tree="";
    String str_f_info_out="";
    String str_option="";
    
    try 
    {
      out=new PrintWriter(sockfd.getOutputStream(), true);
	  in =new BufferedReader(new InputStreamReader(sockfd.getInputStream()));

      for (;;)
      {
        buf="";
        int p=0;
        int esc=0;
        
        while ((buf1=in.read())!='\r')
        {
          if (buf1==-1)
          {
//            System.out.println("Session complete!");
            return;
          }
          
          buf2=(char) buf1;
          
          //Checking for characters
          if ((buf1>31) && (buf1<127))
          {
            if (esc==0)
            {
              p++;
              buf+=buf2;

              //send echo  
              out.print(buf2);
              out.flush();
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1==27)
          {
            //Escape
            if (esc==0) esc=1;
            else if (esc==1) esc=0;
          }
          else if (buf1=='\n')
          {
            //Ignore character
            if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1=='\b')
          {
            //Backspace
            if (esc==0)
            {
              if (p>0) 
              {
                p--;
                buf=buf.substring(0,p);

                //send echo  
                out.print("\b \b");
                out.flush();        
              }
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
        }

        out.print('\r');
        out.print('\n');
        out.flush();
        
        //process instruction ********************************************************
        misc=s.Parse(buf);

        if (misc<0)
        {
          switch (misc)
          {
            case -1:
            {
              xerr="Warning: Line is empty!";
              break;
            }
            case -2:
            {
              xerr="Error: Unknown instruction!";
              break;
            }
            case -3:
            {
              xerr="Error: This instruction needs parameter!";
              break;
            }
            case -4:
            {
              xerr="Error: This instruction doesn't need parameters!";
              break;
            } 
            case -5:
            {
              xerr="Error: Wrong separator!";
              break;
            }
            default:
            {
              xerr="Error: Can't parse!";
              break;
            }
          }

          out.println(xerr);
          out.flush();
        }
        else
        {
          //Process instructions
          if (misc==g.instrInitiate)
          {
            if (s.GetParamStrLwr(g.instrInitiate).equals("ac"))
            {
              initiate=1;

              str_dir="";
              str_f_prj="";
              str_f_prj1="";
              str_f_diag_out="";
              str_f_prof_out="";
              str_f_tree_out="";
              str_f_info="";
              str_no_octave="";
              str_step="";
              str_opt="";
              str_f_prj_out="";
              str_f_tree="";
              str_f_info_out="";
              str_option="";

              out.println("Ok");
              out.flush();
            }
            else
            {
              out.println("Error: Unknown parameter for 'initiate'!");
              out.flush();
            }
          }
          else if (misc==g.instrTerminate)
          {
            out.println("Ok [Session finished]");
            out.flush();
            return; //Finish
          }
          else if (misc==g.instrDirectory)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_dir=s.GetParamStr(g.instrDirectory);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProject)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prj=s.GetParamStr(g.instrFileProject);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProject1)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prj1=s.GetParamStr(g.instrFileProject1);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileDiagOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_diag_out=s.GetParamStr(g.instrFileDiagOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFilePrjOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prj_out=s.GetParamStr(g.instrFilePrjOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileTree)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_tree=s.GetParamStr(g.instrFileTree);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileInfoOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_info_out=s.GetParamStr(g.instrFileInfoOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrOption)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_option=s.GetParamStr(g.instrOption);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProfOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_prof_out=s.GetParamStr(g.instrFileProfOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileInfo)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_info=s.GetParamStr(g.instrFileInfo);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileTreeOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_tree_out=s.GetParamStr(g.instrFileTreeOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrNoOctave)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_no_octave=s.GetParamStr(g.instrNoOctave);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrStep)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_step=s.GetParamStr(g.instrStep);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrOptimization)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_opt=s.GetParamStr(g.instrOptimization);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrGo)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  Go(out, g, fout, str_dir2, str_f_prj,
                                   str_f_prof_out, str_f_tree_out,
                                   str_no_octave, str_step);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrCreateInfo)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else if (str_f_tree.equals(""))
              {
                out.println("Error: Parameter 'file_tree' is empty!");
                out.flush();
              }
              else if (str_f_info_out.equals(""))
              {
                out.println("Error: Parameter 'file_info_out' is empty!");
                out.flush();
              }
              else if (str_option.equals(""))
              {
                out.println("Error: Parameter 'option' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  CreateInfo(out, g, fout, str_dir2, str_f_prj,
                                     str_f_tree, str_f_info_out,
                                     str_option);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunFDC)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  FDC(out, g, fout, str_dir2, str_f_prj, str_opt);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunFDCFAST)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_prj1.equals(""))
              {
                out.println("Error: Parameter 'file_project1' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  FDCFAST(out, g, fout, str_dir2, str_f_prj, str_f_prj1, str_opt);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunUMOL)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  UMOL(out, g, fout, str_dir2, str_f_prj, str_opt);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunFETP)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  FETP(out, g, fout, str_dir2, str_f_prj, str_opt, str_step);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunPROF)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  PROF(out, g, fout, str_dir2, str_f_prj, str_opt);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunLLPROF)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else if (str_f_prof_out.equals(""))
              {
                out.println("Error: Parameter 'file_prof_out' is empty!");
                out.flush();
              }
              else if (str_f_info.equals(""))
              {
                out.println("Error: Parameter 'file_info' is empty!");
                out.flush();
              }
              else if (str_f_prj_out.equals(""))
              {
                out.println("Error: Parameter 'file_project_out' is empty!");
                out.flush();
              }
              else if (str_f_tree.equals(""))
              {
                out.println("Error: Parameter 'file_tree' is empty!");
                out.flush();
              }
              else if (str_f_tree_out.equals(""))
              {
                out.println("Error: Parameter 'file_tree_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  LLPROF(out, g, fout, str_dir2, str_f_prj, str_opt, 
                                       str_f_prof_out, str_f_info,
                                       str_f_prj_out, 
                                       str_f_tree, str_f_tree_out);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else if (misc==g.instrRunSIMPLE)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_prj.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                String str_dir1=CMisc.ProcessDir(g.cfg.GetParamStr(g.cfgPathLocal), 
                                                 g.cfg.GetParamStr(g.cfgSeparator));
 
                String str_dir2=str_dir1+CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));

                BufferedWriter fout=null;

                boolean created=true;

                try
                {
                  fout=new BufferedWriter(new FileWriter(str_dir2+str_f_diag_out));
                }
                catch (IOException e1)
                {
                  out.println("Error: Can't open diagnostic file for writing!");
                  out.flush();
                  created=false;
                }

                if (created)
                {
                  RunSimple(out, g, fout, str_dir2, str_f_prj, str_opt);

                  try
                  {
                    fout.close();
                  }
                  catch (Exception e5)
                  {}
                }
              }
            }
          }
          else
          {
            out.println("Error: Unknown instruction!");
            out.flush();
          }
        }
      }
    }
    catch (IOException e) 
    {
//It may happen when connection is closed
//      System.out.println("Error: Problems during communication!");
      return;
    }
  }      

  //**************************************************************************
  void Go(PrintWriter out,
          CGlobal g,
          BufferedWriter fout,
          String str_dir,
          String str_fp,
          String str_f_prof_out,
          String str_f_tree_out,
          String str_no_octave,
          String str_step)
  {
    int ret=0;
    int i=0;

    int step=0;
    try
    {
      step=Integer.valueOf(str_step).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str_dir+str_f_prof_out);
      fdel.delete();
      fdel=new File(str_dir+str_f_tree_out);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Preliminary diagnostic file is: ");
      fout.write(f_tmp);
      fout.newLine();
      fout.newLine();

      //Compile and run original & save results
      if (step<=01)
      {
        fout.write("01) Compiling (optimization=1), running & coping output files ...");
        fout.newLine();
        fout.newLine();
        fout.write("    Compiling:     ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_diag_out="+f_tmp,   true, true);
        cl.Add("compilation_mode=0",     true, true);
        cl.Add("no_checking=0",          true, true);
        cl.Add("optimizations=1",        true, true);
        cl.Add("compile",                true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
                             
        fout.write("    Running:       ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("runmode=0",              true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();

        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }

        ct.Align(5,1);

        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();
      }

      if (step<=02)
      {
        fout.write("02) Running & checking output files ...");
        fout.newLine();
        fout.newLine();
        fout.write("    Running:       ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("file_out_comp="+f_tmp2,  true, true);
        cl.Add("runmode=1",              true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);
 
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }
 
        if (CMisc.isCorrect(str_dir+f_tmp2)==1)
        {
          fout.write("Ok");
          fout.newLine();
        }
        else
        {
          fout.write("Output is different from original!");
          fout.newLine();
        }
  
        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }
 
        ct.Align(5,1);

        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();
      }        
  
      if (step<=03)
      {
        fout.write("03) Compiling (optimization=2), running & checking output files ...");
        fout.newLine();
        fout.newLine();
        fout.write("    Compiling:     ");
        fout.flush();
  
        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_diag_out="+f_tmp,   true, true);
        cl.Add("compilation_mode=0",     true, true);
        cl.Add("no_checking=0",          true, true);
        cl.Add("optimizations=2",        true, true);
        cl.Add("compile",                true, true);
        cl.Add("terminate",             false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }
  
        fout.write("Ok");
        fout.newLine();
                              
        fout.write("    Running:       ");
        fout.flush();
  
        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("file_out_comp="+f_tmp2,  true, true);
        cl.Add("runmode=1",              true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }
  
        if (CMisc.isCorrect(str_dir+f_tmp2)==1)
        {
          fout.write("Ok");
          fout.newLine();
        }
        else
        {
          fout.write("Output is different from original!");
          fout.newLine();
        }
 
        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }
  
        ct.Align(5,1);

        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();
      } 

      if (step<=04)
      {
        fout.write("04) Compiling (optimization=1, profiling), running & checking output files ...");
        fout.newLine();
        fout.newLine();
  
        if (str_f_prof_out.equals(""))
        {
          fout.write("    TURNED OFF");
          fout.newLine();
          fout.flush();
        }
        else
        {
          fout.write("    Compiling:     ");
          fout.flush();
  
          cl.Add("initiate compilation",   true, true);
          cl.Add("directory="+str_dir,     true, true);
          cl.Add("file_project="+str_fp,   true, true);
          cl.Add("file_diag_out="+f_tmp,   true, true);
          cl.Add("compilation_mode=0",     true, true);
          cl.Add("no_checking=0",          true, true);
          cl.Add("profile=1",              true, true);
          cl.Add("optimizations=1",        true, true);
          cl.Add("compile",                true, true);
          cl.Add("terminate",             false, true);
    
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
  
          fout.write("Ok");
          fout.newLine();
                              
          fout.write("    Running:       ");
          fout.flush();
 
          cl.Add("initiate compilation",          true, true);
          cl.Add("directory="+str_dir,            true, true);
          cl.Add("file_project="+str_fp,          true, true);
          cl.Add("file_out_time="+f_tmp1,         true, true);
          cl.Add("file_out_comp="+f_tmp2,         true, true);
          cl.Add("file_out_prof="+str_f_prof_out, true, true);
          cl.Add("runmode=1",                     true, true);
          cl.Add("run",                           true, true);
          cl.Add("terminate",                    false, true);
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }

          if (CMisc.isCorrect(str_dir+f_tmp2)==1)
          {
            fout.write("Ok");
            fout.newLine();
          }
          else
          {
            fout.write("Output is different from original!");
            fout.newLine();
          }
        }
 
        fout.newLine();
      }

      if (step<=05)
      {
        fout.write("05) Creating tree ...");
        fout.newLine();
        fout.newLine();
  
        if (str_f_tree_out.equals(""))
        {
          fout.write("    TURNED OFF");
          fout.newLine();
          fout.flush();
        }
        else
        {
          fout.write("    Creating tree: ");
          fout.flush();
  
          cl.Add("initiate compilation",      true, true);
          cl.Add("directory="+str_dir,        true, true);
          cl.Add("file_project="+str_fp,      true, true);
          cl.Add("file_diag_out="+f_tmp,      true, true);
          cl.Add("file_out="+str_f_tree_out,  true, true);
          cl.Add("tree",                      true, true);
          cl.Add("terminate",                false, true);
  
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
  
          fout.write("Ok");
          fout.newLine();
        }  
  
        fout.newLine();                             
      }  

      if (step<=06)
      {
        fout.write("06) Parsing (Octave) ...");
        fout.newLine();
        fout.newLine();
  
        if (str_no_octave.equals("1"))
        {
          fout.write("    TURNED OFF");
          fout.newLine();
          fout.flush();
        }
        else
        {
          fout.write("    Parsing:       ");
          fout.flush();
  
          cl.Add("initiate compilation",      true, true);
          cl.Add("directory="+str_dir,        true, true);
          cl.Add("file_project="+str_fp,      true, true);
          cl.Add("file_diag_out="+f_tmp,      true, true);
          cl.Add("file_out="+f_tmp_prj,       true, true);
          cl.Add("filename_add=_tmp",         true, true);
          cl.Add("parse",                     true, true);
          cl.Add("terminate",                false, true);
    
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          fout.write("Ok");
          fout.newLine();
        }

        fout.newLine();                             
      }

      if (step<=07)
      {
        fout.write("07) Compiling (optimization=1, octave), running & checking output files ...");
        fout.newLine();
        fout.newLine();
  
        if (str_no_octave.equals("1"))
        {
          fout.write("    TURNED OFF");
          fout.newLine();
          fout.flush();
        }
        else
        {
          fout.write("    Compiling:     ");
          fout.flush();
  
          cl.Add("initiate compilation",     true, true);
          cl.Add("directory="+str_dir,       true, true);
          cl.Add("file_project="+f_tmp_prj,  true, true);
          cl.Add("file_diag_out="+f_tmp,     true, true);
          cl.Add("compilation_mode=0",       true, true);
          cl.Add("no_checking=0",            true, true);
          cl.Add("optimizations=1",          true, true);
          cl.Add("compile",                  true, true);
          cl.Add("terminate",               false, true);
   
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
   
          fout.write("Ok");
          fout.newLine();
                              
          fout.write("    Running:       ");
          fout.flush();
  
          cl.Add("initiate compilation",     true, true);
          cl.Add("directory="+str_dir,       true, true);
          cl.Add("file_project="+f_tmp_prj,  true, true);
          cl.Add("file_out_time="+f_tmp1,    true, true);
          cl.Add("file_out_comp="+f_tmp2,    true, true);
          cl.Add("runmode=1",                true, true);
          cl.Add("run",                      true, true);
          cl.Add("terminate",               false, true);
    
          ret=cl.Send();
          if (ret==-1)
          {
            out.println("Error: Can't find host with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-2)
          {
            out.println("Error: Can't connect to Compilation Server!");
            out.flush();
            return;
          }
          else if (ret==-4)
          {
            out.println("Error: Can't close connection with Compilation Server!");
            out.flush();
            return;
          }
          else if (ret<0)
          {
            out.println(cl.GetLastError());
            out.flush();
            return;
          }
    
          if (CMisc.isCorrect(str_dir+f_tmp2)==1)
          {
            fout.write("Ok");
            fout.newLine();
          }
          else
          {
            fout.write("Output is different from original!");
            fout.newLine();
          }
   
          //Get time
          if (ct.Init(str_dir+f_tmp1)<0)
          {
            out.println(ct.GetLastError());
            out.flush();
            return;
          }
    
          ct.Align(5,1);
   
          fout.write("                                          [Time: ");
          fout.write(ct.GetTimeStr1());
          fout.write(" + ");      
          fout.write(ct.GetTimeStr2());
          fout.write(" = ");
          fout.write(ct.GetTimeStr0());
          fout.write("]");
          fout.newLine();
        }

        fout.newLine();
      }
  
      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void FDC(PrintWriter out,
           CGlobal g,
           BufferedWriter fout,
           String str_dir,
           String str_fp,
           String str_opt)
  {
    int ret=0;
    int i=0;

    int opt_num=0;
    try
    {
      opt_num=Integer.valueOf(str_opt).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Feedback Directed Compilation");
      fout.newLine();
      fout.newLine();

      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Optimization                  : ");
      fout.write(str_opt);
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling (to get feedback information), running & coping output files ...");
      fout.newLine();
      fout.newLine();
      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations="+str_opt, true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }

      ct.Align(5,1);

      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling and running application using feedback information");
      fout.newLine();
      fout.newLine();

      //Compiling and running using feedback information
      for (int opt=1; opt<=opt_num; opt++)
      {
        String s_opt=String.valueOf(opt);

        fout.write("Compiling (optimization="+s_opt+"), running & coping output files ...");
        fout.newLine();
        fout.newLine();
        fout.write("  Compiling:     ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_diag_out="+f_tmp,   true, true);
        cl.Add("compilation_mode=0",     true, true);
        cl.Add("no_checking=0",          true, true);
        cl.Add("optimizations="+s_opt,   true, true);
        cl.Add("feedback=2",             true, true);
        cl.Add("compile",                true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
                             
        fout.write("  Running:       ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("runmode=0",              true, true);
        cl.Add("feedback=2",             true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
        fout.flush();

        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }
 
        ct.Align(5,1);
 
        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();
      }

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void UMOL(PrintWriter out,
            CGlobal g,
            BufferedWriter fout,
            String str_dir,
            String str_fp,
            String str_opt)
  {
    int ret=0;
    int i=0;

    int opt_num=0;
    try
    {
      opt_num=Integer.valueOf(str_opt).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Investigate main optimization levels");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Number of optimization levels : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();

      //Compiling and running original applications
      for (int opt=1; opt<=opt_num; opt++)
      {
        String s_opt=String.valueOf(opt);

        fout.write("Compiling (optimization="+s_opt+"), running & coping output files ...");
        fout.newLine();
        fout.newLine();
        fout.write("  Compiling:     ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_diag_out="+f_tmp,   true, true);
        cl.Add("compilation_mode=0",     true, true);
        cl.Add("no_checking=0",          true, true);
        cl.Add("optimizations="+s_opt,   true, true);
        cl.Add("compile",                true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
                             
        fout.write("  Running:       ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("runmode=0",              true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
        fout.flush();

        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }

        ct.Align(5,1);

        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();
      }

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void FETP(PrintWriter out,
            CGlobal g,
            BufferedWriter fout,
            String str_dir,
            String str_fp,
            String str_opt,
            String str_step)
  {
    int ret=0;
    int i=0;
    int opt=0;

    int step=0;
    try
    {
      step=Integer.valueOf(str_step).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    if (step<2) step=2;

    double[] tm=new double [step+1];
    double tm_min=1e37;
    double tm_max=0;

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Find Execution Time Precision");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Optimization level            : ");
      fout.write(str_opt);
      fout.newLine();

      //Compiling and running original applications
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling (optimization="+str_opt+"), running & coping output files ...");
      fout.newLine();
      fout.newLine();
      fout.write("Compiling: ");
      fout.flush();

      cl.Add("initiate compilation",    true, true);
      cl.Add("directory="+str_dir,      true, true);
      cl.Add("file_project="+str_fp,    true, true);
      cl.Add("file_diag_out="+f_tmp,    true, true);
      cl.Add("compilation_mode=0",      true, true);
      cl.Add("no_checking=0",           true, true);
      cl.Add("optimizations="+str_opt,  true, true);
      cl.Add("compile",                 true, true);
      cl.Add("terminate",              false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("*****************************************************************************");
      fout.newLine();

      for (opt=1; opt<=step; opt++)
      {
        fout.write("Running: ");
        fout.flush();

        cl.Add("initiate compilation",   true, true);
        cl.Add("directory="+str_dir,     true, true);
        cl.Add("file_project="+str_fp,   true, true);
        cl.Add("file_out_time="+f_tmp1,  true, true);
        cl.Add("runmode=0",              true, true);
        cl.Add("run",                    true, true);
        cl.Add("terminate",             false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Compilation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Compilation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }

        fout.write("Ok");
        fout.newLine();
        fout.flush();

        //Get time
        if (ct.Init(str_dir+f_tmp1)<0)
        {
          out.println(ct.GetLastError());
          out.flush();
          return;
        }

        ct.Align(5,1);

        fout.write("                                          [Time: ");
        fout.write(ct.GetTimeStr1());
        fout.write(" + ");      
        fout.write(ct.GetTimeStr2());
        fout.write(" = ");
        fout.write(ct.GetTimeStr0());
        fout.write("]");
        fout.newLine();
        fout.newLine();

        double time_best=ct.GetTime0();

        tm[opt]=time_best;

        if (time_best>tm_max) tm_max=time_best;
        if (time_best<tm_min) tm_min=time_best;
      }

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Time Min (sec.)                   : ");
      fout.write(String.valueOf(tm_min));
      fout.newLine();

      fout.write("Time Max (sec.)                   : ");
      fout.write(String.valueOf(tm_max));
      fout.newLine();

      fout.newLine();

      double tm_mn=0;
      for (opt=1; opt<=step; opt++)
      {
        tm_mn+=tm[opt];
      }
      tm_mn/=step;
      tm_mn=Math.floor(tm_mn*10)/10;

      fout.write("Time Arithmetic Mean (sec.)       : ");
      fout.write(String.valueOf(tm_mn));
      fout.newLine();

      fout.newLine();

      //Empirical Mean Absolute Deviation
      double tm_ad=0;
      for (opt=1; opt<=step; opt++)
      {
        tm_ad+=Math.abs(tm[opt]-tm_mn);
      }
      tm_ad/=step;
      tm_ad=Math.floor(tm_ad*100+.01)/100;

      fout.write("Empirical Mean Absolute Deviation : ");
      fout.write(String.valueOf(tm_ad));
      fout.newLine();

      //Empirical Mean Square Deviation
      double tm_sd=0;
      for (opt=1; opt<=step; opt++)
      {
        tm_sd+=(tm[opt]-tm_mn)*(tm[opt]-tm_mn);
      }
      tm_sd=Math.sqrt(tm_sd/(step-1));
      tm_sd=Math.floor(tm_sd*100+.01)/100;

      fout.write("Empirical Mean Square Deviation   : ");
      fout.write(String.valueOf(tm_sd));
      fout.newLine();

      fout.newLine();

      //Empirical Range, Range of Variation, Spread
      double tm_s=tm_max-tm_min;
      tm_s=Math.floor(tm_s*10+.1)/10;

      fout.write("EMPIRICAL RANGE                   : ");
      fout.write(String.valueOf(tm_s));
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void FDCFAST(PrintWriter out,
               CGlobal g,
               BufferedWriter fout,
               String str_dir,
               String str_fp,
               String str_fp1,
               String str_opt)
  {
    int ret=0;
    int i=0;

    int opt_num=0;
    try
    {
      opt_num=Integer.valueOf(str_opt).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Dealing with project
    str1=str_dir+str_fp1;

    CProject cp1 = new CProject();

    if (cp1.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject1!");
      out.flush();
      return;
    }

    if (cp1.Parse()<0)
    {
      out.println(cp1.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Feedback Directed Compilation [Fast Feedback Using Small Dataset]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling (to get feedback information), running & coping output files ...");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.newLine();

      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations=0",        true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok  ");

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }

      ct.Align(5,1);

      fout.write("[Time: ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling and running application using feedback information ...");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Optimization                  : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();

      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations="+str_opt, true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);
  
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }
 
      ct.Align(5,1);
 
      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling and running application using feedback information ...");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp1);
      fout.newLine();
      fout.write("Optimization                  : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();

      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations="+str_opt, true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);
  
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }
 
      ct.Align(5,1);
 
      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling (to get feedback information), running & coping output files ...");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp1);
      fout.newLine();
      fout.newLine();

      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations=0",        true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=1",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok  ");

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }

      ct.Align(5,1);

      fout.write("[Time: ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Compiling and running application using feedback information ...");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp1);
      fout.newLine();
      fout.write("Optimization                  : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();

      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("optimizations="+str_opt, true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp1,  true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("feedback=2",             true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);
  
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }
 
      ct.Align(5,1);
 
      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void PROF(PrintWriter out,
           CGlobal g,
           BufferedWriter fout,
           String str_dir,
           String str_fp,
           String str_opt)
  {
    int ret=0;
    int i=0;

    int opt_num=0;
    try
    {
      opt_num=Integer.valueOf(str_opt).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Profiling");
      fout.newLine();
      fout.newLine();

      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Optimization                  : ");
      fout.write(str_opt);
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();
      fout.write("Compiling:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_diag_out="+f_tmp,   true, true);
      cl.Add("compilation_mode=0",     true, true);
      cl.Add("no_checking=0",          true, true);
      cl.Add("profile=1",              true, true);
      cl.Add("optimizations="+str_opt, true, true);
      cl.Add("compile",                true, true);
      cl.Add("terminate",             false, true);
    
      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("Running:         ");
      fout.flush();

      cl.Add("initiate compilation",          true, true);
      cl.Add("directory="+str_dir,            true, true);
      cl.Add("file_project="+str_fp,          true, true);
      cl.Add("file_out_time="+f_tmp1,         true, true);
      cl.Add("file_out_prof="+f_tmp2,         true, true);
      cl.Add("runmode=0",                     true, true);
      cl.Add("run",                           true, true);
      cl.Add("terminate",                    false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Profiling information:");
      fout.newLine();
      fout.newLine();

      fout.write("'procedure name', 'time (sec.)', 'number of calls', 'percent'");
      fout.newLine();
      fout.newLine();

      BufferedReader fin=null;

      //Reading profiling information and adding it to diag file
      try
      {
        fin=new BufferedReader(new FileReader(str_dir+f_tmp2));
      }
      catch (IOException e2)
      {
        fout.write("Error: Can't find file with profiling information!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        while ((str1=fin.readLine())!=null)
        {
          fout.write(str1);
          fout.newLine();
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.write("Error: Problems while reading file with profiling information!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void LLPROF(PrintWriter out,
             CGlobal g,
             BufferedWriter fout,
             String str_dir,
             String str_fp,
             String str_opt,
             String str_prof,
             String str_fi,
             String str_fpo,
             String str_ft,
             String str_fto)
  {
    int ret=0;
    int i=0;

    int opt_num=0;
    try
    {
      opt_num=Integer.valueOf(str_opt).intValue();
    }
    catch (NumberFormatException nfe1)
    {}

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Start output
    try
    {
      fout.write("Reading parameters from file    : ");
      fout.flush();

      str1=str_dir+str_fi;
      String str2="";

      BufferedReader fin=null;
      try
      {
        fin=new BufferedReader(new FileReader(str1));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error: Can't find info file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      int nf=0;  //number of files
      int nt=0;  //number of timers

      try
      {
        while ((str2=fin.readLine())!=null)
        {
          if (str2.length()>0)
          {
            if (str2.charAt(0)=='*')
            {
              nf++;
            }
            else if (str2.charAt(0)=='#')
            {
              nt++;
            }
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.newLine();
        fout.write("Error: Problems reading info file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }
 
      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      fout.write("Ok");
      fout.newLine();
  
      fout.newLine();
      fout.write("Number of files                 : ");
      fout.write(String.valueOf(nf));
      fout.newLine();
      fout.write("Number of timers                : ");
      fout.write(String.valueOf(nt));
      fout.newLine();
      fout.flush();

      if (nt<1)
      {
        fout.newLine();
        fout.write("Nothing to do!");
        fout.newLine();
        fout.flush();

        out.println("Ok");
        out.flush();

        return;
      }

      String[] Fil=new String [nf];

      str1=str_dir+str_fi;

      try
      {
        fin=new BufferedReader(new FileReader(str1));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      nf=0;

      try
      {
        while ((str2=fin.readLine())!=null)
        {
          if (str2.length()>0)
          {
            if (str2.charAt(0)=='*')
            {
              Fil[nf]=str2.substring(1,str2.length());
              nf++;
            }
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      //Creating new project
      fout.newLine();
      fout.write("Creating new project file       : ");
      fout.flush();

      String fprj1=str_dir+str_fpo;
         
      CProject cp_new = new CProject();
      cp_new.Init(cp, fprj1);
  
      for (i=1; i<cp.GetNumFiles(); i++)
      {
        boolean found=false;

        for (int j=0; j<nf; j++)
        {
          if (Fil[j].equals(cp.GetFileName(i)+cp.GetFileExt(i)))
          {
            found=true;
            break;
          }
        }
      
        if (!found)
        {
          if (cp_new.AddLine1(cp.GetLngNameOrig(i), cp.GetFileName(i), 
                              cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
          {
            out.println("Error: Too many files in new project file!");
            out.flush();
            return;
          }
        }
        else
        {
          if (cp_new.AddLine1(cp.GetLngNameOrig(i), 
                              cp.GetFileName(i)+str_llprof_add,
                              cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
          {
            out.println("Error: Too many files in new project file!");
            out.flush();
            return;
          }
        }
      }
   
      fout.write("Ok");
      fout.newLine();

      fout.write("Adding timer calls to sources   : ");
      fout.flush();

      str1=str_dir+str_fi;

      try
      {
        fin=new BufferedReader(new FileReader(str1));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      nf=0;

      BufferedReader fin1=null;
      BufferedWriter fout1=null;
      int lin=0;

      String call0="";
      String call1="";
      String call2="";
      if      (cp.GetLngNameOrig(0).equals("f77"))
      {
        call1="      "+g.cfg.GetParamStr(g.cfgEOSTimCL_F77);
      }
      else if (cp.GetLngNameOrig(0).equals("f90"))
      {
        call1="      "+g.cfg.GetParamStr(g.cfgEOSTimCL_F90);
      }
      else if (cp.GetLngNameOrig(0).equals("c"))
      {
        call1=g.cfg.GetParamStr(g.cfgEOSTimCL_C);
        call2=";";
      }
      else if (cp.GetLngNameOrig(0).equals("cpp"))
      {
        call0=g.cfg.GetParamStr(g.cfgEOSTimCL_CPP1);
        call1=g.cfg.GetParamStr(g.cfgEOSTimCL_CPP);
        call2=";";
      }

      try
      {
        while ((str2=fin.readLine())!=null)
        {
          if (str2.length()>0)
          {
            if (str2.charAt(0)=='*')
            {
              str1=str2.substring(1,str2.length());

              String f2a=str1;
              String f2b="";

              int i1=str1.lastIndexOf(".");
              if (i1!=-1)
              {
                f2a=str1.substring(0, i1);
                f2b=str1.substring(i1, str1.length());
              }
  
              //opening file
              if (fin1!=null)
              {
                //Finishing source filesSearching for the line
                try
                {
                  while ((str2=fin1.readLine())!=null)
                  {
                    fout1.write(str2);
                    fout1.newLine();
                  }
                }
                catch (Exception e1)
                {
                  try
                  {
                    fin.close();
                    fin1.close();
                    fout1.close();
                  }
                  catch (Exception e2)
                  {}
                  fout.write("Error writing transformed file!");
                  fout.newLine();
                  fout.flush();
                  out.println("Ok");
                  out.flush();
                  return;
                }

                try
                {
                  fin1.close();
                  fout1.close();
                }
                catch (Exception e2)
                {}
              } 

              try
              {
                fin1=new BufferedReader(new FileReader(str_dir+str1));
                fout1=new BufferedWriter(new FileWriter(str_dir+f2a+str_llprof_add+f2b));
              }
              catch (IOException e2)
              {
                fout.newLine();
                fout.write("Error reading source file"+str1);
                fout.newLine();
                fout.flush();
                out.println("Ok");
                out.flush();
                return;
              }

              lin=0;
            }
            else if (str2.charAt(0)=='#')
            {
              int i1=str2.indexOf(" ");
              int i2=str2.indexOf(" ", i1+1);

              String Ind=str2.substring(1,i1); 

              String s1=str2.substring(i1+1, i2);
              String s2=str2.substring(i2+1, str2.length());

              int t1=0;
              try
              {
                t1=Integer.valueOf(Ind).intValue();
              }
              catch (NumberFormatException nfe1)
              {}

              int l1=0;
              try
              {
                l1=Integer.valueOf(s1).intValue();
              }
              catch (NumberFormatException nfe1)
              {}

              int l2=0;
              try
              {
                l2=Integer.valueOf(s2).intValue();
              }
              catch (NumberFormatException nfe1)
              {}

              //Searching for the line
              boolean found=false;
              try
              {
                while (!found && ((str2=fin1.readLine())!=null))
                {
                  lin++;
                  if (lin==l1 || lin==l2)
                  {
                    if (lin==l1)
                    {
                      if (!call0.equals(""))
                      {
  		        fout1.write(call0);
                        fout1.newLine();
                      }
                      fout1.write(call1+"1("+Ind+")"+call2);
                      fout1.newLine();
                      fout1.write(str2);
                      fout1.newLine();
                    }
                    if (lin==l2)
                    {
                      if (lin!=l1)
                      {
                        fout1.write(str2);
                        fout1.newLine();
                      }
                      fout1.write(call1+"2("+Ind+")"+call2);
                      fout1.newLine();
                      found=true;
                    }
                  }
                  else
                  {
                    fout1.write(str2);
                    fout1.newLine();
                  }
                }
              }
              catch (Exception e1)
              {
                try
                {
                  fin.close();
                  if (fin1!=null) fin1.close();
                  if (fout1!=null) fout1.close();
                }
                catch (Exception e2)
                {}
                fout.write("Error reading original file!");
                fout.newLine();
                fout.flush();
                out.println("Ok");
                out.flush();
                return;
              }
            }
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      if (fin1!=null)
      {
        //Finishing source filesSearching for the line
        try
        {
          while ((str2=fin1.readLine())!=null)
          {
            fout1.write(str2);
            fout1.newLine();
          }
        }
        catch (Exception e1)
        {
          try
          {
            fin.close();
            fin1.close();
            fout1.close();
          }
          catch (Exception e2)
          {}
          fout.write("Error writing transformed file!");
          fout.newLine();
          fout.flush();
          out.println("Ok");
          out.flush();
          return;
        }

        try
        {
          fin1.close();
          fout1.close();
        } 
        catch (Exception e2)
        {}
      } 

      fout.write("Ok");
      fout.newLine();

      fout.write("Creating new tree file          : ");

      fin=null;
      fout1=null;

      String str="";
      String str1a="";
      String str1ax="";
      String str1b="";
      String str1c="";
      String str1d="";
      String str1e="";
      String str1x="";
      String str1y="";
      int ind1=0;
      int ind2=0;
      int istr1x=0;
      int istr1y=0;

      int cur=0;

      try
      {
        fin=new BufferedReader(new FileReader(str_dir+str_ft));
        fout1=new BufferedWriter(new FileWriter(str_dir+str_fto));
      }
      catch (IOException e2)
      {
        fout.write("Error while opening tree and info files!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        while ((str=fin.readLine())!=null)
        {
          boolean incorrect=true;

          ind1=str.indexOf("'");
          ind2=str.indexOf("'", ind1+1);

          if ((ind1!=-1) && (ind2!=-1))
          {
            str1a=str.substring(ind1+1, ind2);

            if (!str1a.equals(str1ax))
            {
              str1ax=str1a;
              cur=0;
            }


            boolean found=false;

            for (i=0; i<nf; i++)
            {
              if (str1a.equals(Fil[i]))
              {
                found=true;
                break;
              }
            }

            String f2a=str1;
            String f2b="";
  
            int i1=str1.lastIndexOf(".");
            if (i1!=-1)
            {
              f2a=str1.substring(0, i1);
              f2b=str1.substring(i1, str1.length());
            }
              
            str1a=f2a+str_llprof_add+f2b;

            ind1=str.indexOf("'", ind2+1);
            ind2=str.indexOf("'", ind1+1);

            if ((ind1!=-1) && (ind2!=-1))
            {
              str1b=str.substring(ind1+1, ind2);

              ind1=str.indexOf("'", ind2+1);
              ind2=str.indexOf("'", ind1+1);

              if ((ind1!=-1) && (ind2!=-1))
              {
                str1c=str.substring(ind1+1, ind2);

                if (str1c.toLowerCase().equals("loop"))
                {
                  ind1=str.indexOf("'", ind2+1);
                  ind2=str.indexOf("'", ind1+1);
                  if ((ind1!=-1) && (ind2!=-1))
                  {
                    str1d=str.substring(ind1+1, ind2);
                    ind1=str.indexOf("'", ind2+1);
                    ind2=str.indexOf("'", ind1+1);
                    if ((ind1!=-1) && (ind2!=-1))
                    {
                      str1e=str.substring(ind1+1, ind2);
                      ind1=str.indexOf("'", ind2+1);
                      ind2=str.indexOf("'", ind1+1);
                      if ((ind1!=-1) && (ind2!=-1))
                      {
                        str1x=str.substring(ind1+1, ind2);
                        ind1=str.indexOf("'", ind2+1);
                        ind2=str.indexOf("'", ind1+1);
                        if ((ind1!=-1) && (ind2!=-1))
                        {
                          str1y=str.substring(ind1+1, ind2);
                          incorrect=false;

                          istr1x=0;
                          istr1y=0;
                          try
                          {
                            istr1x=Integer.valueOf(str1x).intValue();
                            istr1y=Integer.valueOf(str1y).intValue();
                          }
                          catch (NumberFormatException nfe1)
                          {}
                        }
                      }
                    }
                  }

                  istr1x+=cur;
                  istr1y+=cur;

                  if (!incorrect && str.charAt(0)=='+')
                  {
                    istr1x+=1;
                    istr1y+=1;
                    cur+=2;
                  }

                  String str9="'"+str1a+"', '"+str1b+"', '"+str1c+"', '"+str1d+"', '"+str1e+
                      "', '"+String.valueOf(istr1x)+
                      "', '"+String.valueOf(istr1y)+"'";

                  if (!incorrect && str.charAt(0)=='+') str9="+"+str9;

                  fout1.write(str9);
                  fout1.newLine();
                }
                else
                {
                  incorrect=false;

                  if (str.charAt(0)=='+') fout1.write("+");

                  fout1.write("'"+str1a+"', '"+str1b+"', '"+str.substring(ind1+1, str.length()));
                  fout1.newLine();
                }
              }
            }
          }

          if (incorrect)
          {
            fout.write("Error: Can't understand format of the tree file!");
            fout.newLine();
            fout.flush();
            out.println("Ok");
            out.flush();
            return;
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
          fout1.close();
        }
        catch (Exception e2)
        {}

        fout.write("Error: Problems while working with tree and info files!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
        fout1.close();
      }
      catch (Exception e3)
      {}

      fout.write("Ok");
      fout.newLine();

      //Copying timer file from the directory and change max number of timers
      fout.write("Copying timer file to the dir   : ");
      fout.flush();

      fin1=null;
      fout1=null;

      str1=g.cfg.GetParamStr(g.cfgEOSTimD);
      str2=g.cfg.GetParamStr(g.cfgEOSTimF);
      String str3=g.cfg.GetParamStr(g.cfgEOSTimM);

      try
      {
        fin1=new BufferedReader(new FileReader(str1+str2));
        fout1=new BufferedWriter(new FileWriter(str_dir+str2));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error opening EOS Timers file");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        while ((str2=fin1.readLine())!=null)
        {
          String str4=str2;
          i=str2.indexOf(str3);
          if (i>=0)
          {
            str4=str2.substring(0, i)+String.valueOf(nt)+
                 str2.substring(i+str3.length(), str2.length());
          }

          fout1.write(str4);
          fout1.newLine();
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin1.close();
          fout1.close();
        }
        catch (Exception e2)
        {}
        fout.write("Error while working with EOS Timers file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin1.close();
        fout1.close();
      }
      catch (Exception e2)
      {}

      fout.write("Ok");
      fout.newLine();

      //Adding file to the project
      str1=g.cfg.GetParamStr(g.cfgEOSTimF);
      String f2a=str1;
      String f2b="";
  
      int i1=str1.lastIndexOf(".");
      if (i1!=-1)
      {
        f2a=str1.substring(0, i1);
        f2b=str1.substring(i1, str1.length());
      }

      String lng_llprof="";
      String opt_llprof="";
      if      (cp.GetLngNameOrig(0).equals("f77"))
      {
        lng_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_F77X);
        opt_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_F77O);
      }
      else if (cp.GetLngNameOrig(0).equals("f90"))
      {
        lng_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_F90X);
        opt_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_F90O);
      }
      else if (cp.GetLngNameOrig(0).equals("c"))
      {
        lng_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_CX);
        opt_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_CO);
      }
      else if (cp.GetLngNameOrig(0).equals("cpp"))
      {
        lng_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_CPPX);
        opt_llprof=g.cfg.GetParamStr(g.cfgEOSTimCL_CPPO);
      }

      if (cp_new.AddLine1(lng_llprof, f2a, f2b, opt_llprof)<0)
      {
        out.println("Error: Too many files in new project file!");
        out.flush();
        return;
      }

      if (cp_new.Save()<0)
      {
        fout.newLine();
        fout.write("Error: Can't save new project!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      //Delete output files if exist
      File fdel=null;
      try
      {
        fdel=new File(str_dir+g.cfg.GetParamStr(g.cfgEOSTimP));
        fdel.delete();
      }
      catch (NullPointerException e1)
      {}

      fout.write("*****************************************************************************");
      fout.newLine();

      //Create CClient for Compilation Server
      CClient cl=new CClient();
      if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                  g.cfg.GetParamInt(g.cfgCSerPort),
                  16)!=0)
      {
        out.println("Error (internal): Can't initialize object CClient!");
        out.flush();
        return;
      }

      CTime ct=new CTime();

      fout.write("Run application");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename                : ");
      fout.write(str_fpo);
      fout.newLine();
      fout.write("Optimization Level              : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();
      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",     true, true);
      cl.Add("directory="+str_dir,       true, true);
      cl.Add("file_project="+str_fpo,    true, true);
      cl.Add("file_diag_out="+f_tmp,     true, true);
      cl.Add("compilation_mode=0",       true, true);
      cl.Add("no_checking=0",            true, true);
      cl.Add("optimizations="+str_opt,   true, true);
      cl.Add("compile",                  true, true);
      cl.Add("terminate",               false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",     true, true);
      cl.Add("directory="+str_dir,       true, true);
      cl.Add("file_project="+str_fpo,    true, true);
      cl.Add("file_out_time="+f_tmp1,    true, true);
      cl.Add("runmode=0",                true, true);
      cl.Add("run",                      true, true);
      cl.Add("terminate",               false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }

      ct.Align(5,1);

      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Profiling information (loop level):");
      fout.newLine();
      fout.newLine();

      //reading profiling information
      String[] Tim=new String[nt];
      String[] Cal=new String[nt];
      double   TA=0;

      for (i=0; i<nt; i++)
      {
        Tim[i]="";
        Cal[i]="";
      }

      fin1=null;

      try
      {
        fin1=new BufferedReader(new FileReader(str_dir+g.cfg.GetParamStr(g.cfgEOSTimP)));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error opening EOS Timers profile file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        while ((str2=fin1.readLine())!=null)
        {
          i1=str2.indexOf("timer={");
          int i2=str2.indexOf("}", i1+7);
          if ((i1>=0) && (i2>=0))
          {
            String t1=str2.substring(i1+7, i2);

            i1=str2.indexOf("time={",i2+1);
            i2=str2.indexOf("}", i1+6);
            if ((i1>=0) && (i2>=0))
            {
              String t2=str2.substring(i1+6, i2);

              i1=str2.indexOf("calls={",i2+1);
              i2=str2.indexOf("}", i1+7);
              if ((i1>=0) && (i2>=0))
              {
                String t3=str2.substring(i1+7, i2);

                int    ti=0;
                double tt=0;
                try
                {
                  ti=Integer.valueOf(t1.trim()).intValue();
                  tt=Double.valueOf(t2.trim()).doubleValue();
                }
                catch (NumberFormatException nfe1)
                {}

                ti--;
//                TA+=tt;

                Tim[ti]=t2;
                Cal[ti]=t3;
              }
            }
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin1.close();
        }
        catch (Exception e2)
        {}
        fout.write("Error while working with EOS Timers profile file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin1.close();
      }
      catch (Exception e2)
      {}

      TA=ct.GetTime0();

      fout.write("'procedure name', 'timer', 'time (sec.)', 'number of calls', 'percent'");
      fout.newLine();
      fout.newLine();

      str1=str_dir+str_fi;

      try
      {
        fin=new BufferedReader(new FileReader(str1));
      }
      catch (IOException e2)
      {
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        while ((str2=fin.readLine())!=null)
        {
          if (str2.length()>0)
          {
            if (str2.charAt(0)=='*')
            {
              str1=str2.substring(1,str2.length());
 
              fout.write(str1+":");
              fout.newLine();
            }
            else if (str2.charAt(0)=='#')
            {
              i1=str2.indexOf(" ");
              String Ind=str2.substring(1,i1); 

              int t1=0;
              try
              {
                t1=Integer.valueOf(Ind).intValue();
              }
              catch (NumberFormatException nfe1)
              {}
              t1--;

              fout.write("                    "+Ind+",       "+Tim[t1]+
                         ",          "+Cal[t1]);

              double tt=0;
              try
              {
                tt=Double.valueOf(Tim[t1]).doubleValue();
              }
              catch (NumberFormatException nfe1)
              {}

              if (TA!=0)
              {
                double p1=(tt*100)/TA;
                int p=(int) p1;
                str1=String.valueOf(p);
                if (p<10) str1="  "+p;
                else if (p<100) str1=" "+p;
                fout.write(",      "+str1+"%");
              }
              fout.newLine();
            }
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}
        fout.newLine();
        fout.write("Error reading file with parameters!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void RunSimple(PrintWriter out,
                 CGlobal g,
                 BufferedWriter fout,
                 String str_dir,
                 String str_fp,
                 String str_opt)
  {
    int ret=0;
    int i=0;

    if (str_opt.equals("")) str_opt="1";

    String str1=str_dir+str_fp;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Create CClient for Compilation Server
    CClient cl=new CClient();
    if (cl.Init(g.cfg.GetParamStr(g.cfgCSerHost),
                g.cfg.GetParamInt(g.cfgCSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    CTime ct=new CTime();

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Run application");
      fout.newLine();
      fout.newLine();
      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Optimization Level            : ");
      fout.write(str_opt);
      fout.newLine();
      fout.newLine();


      fout.write("*****************************************************************************");
      fout.newLine();
      fout.write("  Compiling:     ");
      fout.flush();

      cl.Add("initiate compilation",     true, true);
      cl.Add("directory="+str_dir,       true, true);
      cl.Add("file_project="+str_fp,     true, true);
      cl.Add("file_diag_out="+f_tmp,     true, true);
      cl.Add("compilation_mode=0",       true, true);
      cl.Add("no_checking=0",            true, true);
      cl.Add("optimizations="+str_opt,   true, true);
      cl.Add("compile",                  true, true);
      cl.Add("terminate",               false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
                             
      fout.write("  Running:       ");
      fout.flush();

      cl.Add("initiate compilation",   true, true);
      cl.Add("directory="+str_dir,     true, true);
      cl.Add("file_project="+str_fp,   true, true);
      cl.Add("file_out_time="+f_tmp1,  true, true);
      cl.Add("runmode=0",              true, true);
      cl.Add("run",                    true, true);
      cl.Add("terminate",             false, true);

      ret=cl.Send();
      if (ret==-1)
      {
        out.println("Error: Can't find host with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-2)
      {
        out.println("Error: Can't connect to Compilation Server!");
        out.flush();
        return;
      }
      else if (ret==-4)
      {
        out.println("Error: Can't close connection with Compilation Server!");
        out.flush();
        return;
      }
      else if (ret<0)
      {
        out.println(cl.GetLastError());
        out.flush();
        return;
      }

      fout.write("Ok");
      fout.newLine();
      fout.flush();

      //Get time
      if (ct.Init(str_dir+f_tmp1)<0)
      {
        out.println(ct.GetLastError());
        out.flush();
        return;
      }

      ct.Align(5,1);

      fout.write("                                          [Time: ");
      fout.write(ct.GetTimeStr1());
      fout.write(" + ");      
      fout.write(ct.GetTimeStr2());
      fout.write(" = ");
      fout.write(ct.GetTimeStr0());
      fout.write("]");
      fout.newLine();
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void CreateInfo(PrintWriter out,
                  CGlobal g,
                  BufferedWriter fout,
                  String str_dir,
                  String str_fp,
                  String str_ft,
                  String str_fi_out,
                  String str_o)
  {
    String str="";
    String str1="";
    String str2="";
    String str1a="";
    String str1ax="";
    String str1b="";
    String str1x="";
    String str1y="";
    int i=0;
    int j=0;
    int ind1=0;
    int ind2=0;

    //Start output
    try
    {
      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Project filename              : ");
      fout.write(str_fp);
      fout.newLine();
      fout.write("Tree file                     : ");
      fout.write(str_ft);
      fout.newLine();
      fout.write("Info file (new)               : ");
      fout.write(str_fi_out);
      fout.newLine();
      fout.write("Mode (mpc or llprof)          : ");
      fout.write(str_o);
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();

      fout.write("Creating info file from tree file : ");

      BufferedReader fin=null;
      BufferedWriter fout1=null;

      try
      {
        fin=new BufferedReader(new FileReader(str_dir+str_ft));
        fout1=new BufferedWriter(new FileWriter(str_dir+str_fi_out));
      }
      catch (IOException e2)
      {
        fout.write("Error while opening tree and info files!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      int nf=0;
      int cur=0;

      try
      {
        while ((str=fin.readLine())!=null)
        {
          boolean incorrect=true;

          ind1=str.indexOf("'");
          ind2=str.indexOf("'", ind1+1);

          if ((ind1!=-1) && (ind2!=-1))
          {
            str1a=str.substring(ind1+1, ind2);

            ind1=str.indexOf("'", ind2+1);
            ind2=str.indexOf("'", ind1+1);

            if ((ind1!=-1) && (ind2!=-1))
            {
              str1b=str.substring(ind1+1, ind2);

              ind1=str.indexOf("'", ind2+1);
              ind2=str.indexOf("'", ind1+1);

              if ((ind1!=-1) && (ind2!=-1))
              {
                String str1c=str.substring(ind1+1, ind2);

                str1c=str1c.toLowerCase();

                if (str1c.equals("loop"))
                {
                  ind1=str.indexOf("'", ind2+1);
                  ind2=str.indexOf("'", ind1+1);
                  if ((ind1!=-1) && (ind2!=-1))
                  {
                    ind1=str.indexOf("'", ind2+1);
                    ind2=str.indexOf("'", ind1+1);
                    if ((ind1!=-1) && (ind2!=-1))
                    {
                      ind1=str.indexOf("'", ind2+1);
                      ind2=str.indexOf("'", ind1+1);
                      if ((ind1!=-1) && (ind2!=-1))
                      {
                        str1x=str.substring(ind1+1, ind2);
                        ind1=str.indexOf("'", ind2+1);
                        ind2=str.indexOf("'", ind1+1);
                        if ((ind1!=-1) && (ind2!=-1))
                        {
                          str1y=str.substring(ind1+1, ind2);
                          incorrect=false;
                        }
                      }
                    }
                  }

                  if (!incorrect && str.charAt(0)=='+')
                  {
                    if (!str1a.equals(str1ax))
                    {
                      fout1.write("*"+str1a);
                      fout1.newLine();
                      fout1.newLine();

                      str1ax=str1a;

                      nf++;

//                      cur=0;
                    }

                    cur++;
                    str1=String.valueOf(cur);
                    i=str1.length();
                    str2="";
                    for (j=0; j<5-i; j++) str2+="0";
                    str2+=str1;
      
                    if (str_o.equals("2"))
                       fout1.write("#"+str2+" "+str1x+" "+str1y);
                    else
                       fout1.write("#"+str1b+"_mpc_"+str2+" "+str1x+" "+str1y);
             
                    fout1.newLine();
                    fout1.newLine();

                    fout1.flush();
                  }
                }
                else incorrect=false;
              }
            }
          }

          if (incorrect)
          {
            fout.write("Error: Can't understand format of the tree file!");
            fout.newLine();
            fout.flush();
            out.println("Ok");
            out.flush();
            return;
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
          fout1.close();
        }
        catch (Exception e2)
        {}

        fout.write("Error: Problems while working with tree and info files!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
        fout1.close();
      }
      catch (Exception e3)
      {}

      fout.write("Ok");
      fout.newLine();

      String[] Fil=new String[nf];

      fout.write("Reading filenames from tree file  : ");

      fin=null;

      try
      {
        fin=new BufferedReader(new FileReader(str_dir+str_ft));
      }
      catch (IOException e2)
      {
        fout.write("Error while opening tree file!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      nf=0;

      try
      {
        while ((str=fin.readLine())!=null)
        {
          boolean incorrect=true;

          ind1=str.indexOf("'");
          ind2=str.indexOf("'", ind1+1);

          if ((ind1!=-1) && (ind2!=-1))
          {
            str1a=str.substring(ind1+1, ind2);

            ind1=str.indexOf("'", ind2+1);
            ind2=str.indexOf("'", ind1+1);

            if ((ind1!=-1) && (ind2!=-1))
            {
              str1b=str.substring(ind1+1, ind2);

              ind1=str.indexOf("'", ind2+1);
              ind2=str.indexOf("'", ind1+1);

              if ((ind1!=-1) && (ind2!=-1))
              {
                String str1c=str.substring(ind1+1, ind2);

                str1c=str1c.toLowerCase();

                if (str1c.equals("loop"))
                {
                  ind1=str.indexOf("'", ind2+1);
                  ind2=str.indexOf("'", ind1+1);
                  if ((ind1!=-1) && (ind2!=-1))
                  {
                    ind1=str.indexOf("'", ind2+1);
                    ind2=str.indexOf("'", ind1+1);
                    if ((ind1!=-1) && (ind2!=-1))
                    {
                      ind1=str.indexOf("'", ind2+1);
                      ind2=str.indexOf("'", ind1+1);
                      if ((ind1!=-1) && (ind2!=-1))
                      {
                        str1x=str.substring(ind1+1, ind2);
                        ind1=str.indexOf("'", ind2+1);
                        ind2=str.indexOf("'", ind1+1);
                        if ((ind1!=-1) && (ind2!=-1))
                        {
                          str1y=str.substring(ind1+1, ind2);
                          incorrect=false;
                        }
                      }
                    }
                  }

                  if (!incorrect && str.charAt(0)=='+')
                  {
                    if (!str1a.equals(str1ax))
                    {
                      Fil[nf]=str1a;
                      nf++;
                      str1ax=str1a;
                    }
                  }
                }
                else incorrect=false;
              }
            }
          }

          if (incorrect)
          {
            fout.write("Error: Can't understand format of the tree file!");
            fout.newLine();
            fout.flush();
            out.println("Ok");
            out.flush();
            return;
          }
        }
      }
      catch (Exception e1)
      {
        try
        {
          fin.close();
        }
        catch (Exception e2)
        {}

        fout.write("Error: Problems while working with tree and info files!");
        fout.newLine();
        fout.flush();
        out.println("Ok");
        out.flush();
        return;
      }

      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}

      fout.write("Ok");
      fout.newLine();

      fout.write("*****************************************************************************");
      fout.newLine();
    }
    catch (Exception e2)
    {
      out.println("Error: Problems while writing diagnostic files!");
      out.flush();
      return;
    }
 
    out.println("Ok");
    out.flush();
  }
}

