//
// Process Instructions
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.net.*;
import java.io.*;

public class ProcessInstructions
{
  String f_tmp ="_eos_tmp";
  String f_tmp1="_eos_tmp1";
  String f_tmp2="_eos_tmp2";

  public ProcessInstructions(Socket sockfd, CGlobal g, CSIParse s)
  {
    String str="";
    String buf="";
    int buf1;
    char buf2;
    String xerr="";
    boolean enter=false;
    int misc;
    
    PrintWriter out;
    BufferedReader in;

    //Parameters
    int initiate=0;
    String str_dir="";
    String str_f_diag_out="";
    String str_f_project="";
    String str_f_out="";
    String str_fn_add="";
    String str_profile="";
    String str_comp_mode="";
    String str_no_check="";
    String str_optim="";
    String str_fo_time="";
    String str_fo_comp="";
    String str_fo_prof="";
    String str_prof_type="";
    String str_run_mode="";
    String str_tool="";
    String str_feedback="";
    String str_prm="";
    String str_prm1="";

    try 
    {
      out=new PrintWriter(sockfd.getOutputStream(), true);
      in =new BufferedReader(new InputStreamReader(sockfd.getInputStream()));

      for (;;)
      {
        buf="";
        int p=0;
        int esc=0;
        
        while ((buf1=in.read())!='\r')
        {
          if (buf1==-1)
          {
//            System.out.println("Session complete!");
            return;
          }
          
          buf2=(char) buf1;
          
          //Checking for characters
          if ((buf1>31) && (buf1<127))
          {
            if (esc==0)
            {
              p++;
              buf+=buf2;

              //send echo  
              out.print(buf2);
              out.flush();
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1==27)
          {
            //Escape
            if (esc==0) esc=1;
            else if (esc==1) esc=0;
          }
          else if (buf1=='\n')
          {
            //Ignore character
            if (esc==1) esc=2;
            else esc=0;
          }
          else if (buf1=='\b')
          {
            //Backspace
            if (esc==0)
            {
              if (p>0) 
              {
                p--;
                buf=buf.substring(0,p);

                //send echo  
                out.print("\b \b");
                out.flush();        
              }
            }
            else if (esc==1) esc=2;
            else esc=0;
          }
        }

        out.print('\r');
        out.print('\n');
        out.flush();
        
        //process instruction ********************************************************
        misc=s.Parse(buf);

        if (misc<0)
        {
          switch (misc)
          {
            case -1:
            {
              xerr="Warning: Line is empty!";
              break;
            }
            case -2:
            {
              xerr="Error: Unknown instruction!";
              break;
            }
            case -3:
            {
              xerr="Error: This instruction needs parameter!";
              break;
            }
            case -4:
            {
              xerr="Error: This instruction doesn't need parameters!";
              break;
            } 
            case -5:
            {
              xerr="Error: Wrong separator!";
              break;
            }
            default:
            {
              xerr="Error: Can't parse!";
              break;
            }
          }

          out.println(xerr);
          out.flush();
        }
        else
        {
          //Process instructions
          if (misc==g.instrInitiate)
          {
            if (s.GetParamStrLwr(g.instrInitiate).equals("compilation"))
            {
              initiate=1;

              str_dir="";
              str_f_diag_out="";
              str_f_project="";
              str_f_out="";
              str_fn_add="";
              str_profile="";
              str_comp_mode="";
              str_no_check="";
              str_optim="";
              str_fo_time="";
              str_fo_comp="";
              str_fo_prof="";
              str_prof_type="";
              str_run_mode="";
              str_tool="";
              str_feedback="";
              str_prm="";
              str_prm1="";

              out.println("Ok");
              out.flush();
            }
            else
            {
              out.println("Error: Unknown parameter for 'initiate'!");
              out.flush();
            }
          }
          else if (misc==g.instrTerminate)
          {
            out.println("Ok [Session finished]");
            out.flush();
            return; //Finish
          }
          else if (misc==g.instrDirectory)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_dir=s.GetParamStr(g.instrDirectory);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileDiagOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_diag_out=s.GetParamStr(g.instrFileDiagOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileProject)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_project=s.GetParamStr(g.instrFileProject);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileOut)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_f_out=s.GetParamStr(g.instrFileOut);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFilenameAdd)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_fn_add=s.GetParamStr(g.instrFilenameAdd);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrParse)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_out.equals(""))
              {
                out.println("Error: Parameter 'file_out' is empty!");
                out.flush();
              }
              else if (str_fn_add.equals(""))
              {
                out.println("Error: Parameter 'filename_add' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                Parse(out, g, str_dir, str_f_project, str_f_out, str_fn_add, str_f_diag_out);
              }
            }
          }
          else if (misc==g.instrTree)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_out.equals(""))
              {
                out.println("Error: Parameter 'file_out' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                Tree(out, g, str_dir, str_f_project, str_f_out, str_f_diag_out);
              }
            }
          }
          else if (misc==g.instrProfile)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_profile=s.GetParamStr(g.instrProfile);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrCompMode)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_comp_mode=s.GetParamStr(g.instrCompMode);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrNoCheck)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_no_check=s.GetParamStr(g.instrNoCheck);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrOptimizations)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_optim=s.GetParamStr(g.instrOptimizations);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrCompile)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_f_diag_out.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_profile.equals("")) str_profile="0";
                if (str_comp_mode.equals("")) str_comp_mode="0";
                if (str_no_check.equals("")) str_no_check="0";
                if (str_optim.equals("")) str_optim="0";
                Compile(out, g, str_dir, str_f_project, str_f_diag_out, 
                                str_profile, str_comp_mode, str_no_check, 
                                str_optim, str_feedback);
              }
            }
          }
          else if (misc==g.instrFileOutTime)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_fo_time=s.GetParamStr(g.instrFileOutTime);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileOutComp)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_fo_comp=s.GetParamStr(g.instrFileOutComp);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFileOutProf)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_fo_prof=s.GetParamStr(g.instrFileOutProf);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrProfType)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_prof_type=s.GetParamStr(g.instrProfType);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrRunMode)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_run_mode=s.GetParamStr(g.instrRunMode);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrTool)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_tool=s.GetParamStr(g.instrTool);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrFeedback)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_feedback=s.GetParamStr(g.instrFeedback);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrParam)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_prm=s.GetParamStr(g.instrParam);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrParam1)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              str_prm1=s.GetParamStr(g.instrParam1);

              out.println("Ok");
              out.flush();
            }
          }
          else if (misc==g.instrRun)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_fo_time.equals(""))
              {
                out.println("Error: Parameter 'file_diag_out' is empty!");
                out.flush();
              }
              else
              {
                if (str_run_mode.equals("")) str_run_mode="0";
                Run(out, g, str_dir, str_f_project, 
                            str_fo_time, str_fo_comp, str_fo_prof,
                            str_prof_type, str_run_mode, str_tool,
                            str_feedback);
              }
            }
          }
          else if (misc==g.instrRemoveObj)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else
              {
                RemoveObj(out, g, str_dir, str_f_project);
              }
            }
          }
          else if (misc==g.instrRunTool)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_f_project.equals(""))
              {
                out.println("Error: Parameter 'file_project' is empty!");
                out.flush();
              }
              else if (str_tool.equals(""))
              {
                out.println("Error: Parameter 'tool' is empty!");
                out.flush();
              }
              else
              {
                RunTool(out, g, str_dir, str_f_project, str_tool, str_prm, str_prm1);
              }
            }
          }
          else if (misc==g.instrCDCF)
          {
            if (initiate==0)
            {
              out.println("Error: Use 'initiate' first!");
              out.flush();
            }
            else
            {
              if (str_dir.equals(""))
              {
                out.println("Error: Parameter 'directory' is empty!");
                out.flush();
              }
              else if (str_prm.equals(""))
              {
                out.println("Error: Parameter 'param' is empty!");
                out.flush();
              }
              else if (str_prm1.equals(""))
              {
                out.println("Error: Parameter 'param1' is empty!");
                out.flush();
              }
              else
              {
                CDCF(out, g, str_dir, str_prm, str_prm1);
              }
            }
          }
          else
          {
            out.println("Error: Unknown instruction!");
            out.flush();
          }
        }
      }
    }
    catch (IOException e) 
    {
//It may happen when connection is closed
//      System.out.println("Error: Problems during communication!");
      return;
    }
  }      

  //**************************************************************************
  void Parse(PrintWriter out, 
             CGlobal g,
             String str_dir, 
             String str_f_project, 
             String str_f_out,
             String str_fn_add,
             String str_f_diag_out)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;
    String str2=str_dir+str_f_out;
    String str3=str_dir+str_f_diag_out;

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str2);
      fdel.delete();
      fdel=new File(str3);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(g.lng, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }
    if (cp.ParseLng()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Copy project
    CProject cp_new = new CProject();
    cp_new.Init(cp, str2);

    int mainLng=cp.GetLng(0);

    //Create CClient for Transformation Server
    if (g.lng.GetConfig(mainLng).GetParamStr(g.lngTSerHost).equals(""))
    {
      out.println("Error: Main language in project file is not supported by Transformation Server!");
      out.flush();
      return;
    }

    CClient cl=new CClient();
    if (cl.Init(g.lng.GetConfig(mainLng).GetParamStr(g.lngTSerHost),
                g.lng.GetConfig(mainLng).GetParamInt(g.lngTSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    //List of files
//    cp_new.AddLine(mainLng, "", "", cp.GetExtraOpt(0));
    for (int i=1; i<cp.GetNumFiles(); i++)
    {
      String fnew=cp.GetFileName(i)+str_fn_add;

      String f1=str_dir+cp.GetFileName(i)+cp.GetFileExt(i);
      String f2=str_dir+fnew+cp.GetFileExt(i);

      if (cp_new.AddLine(cp.GetLng(i), fnew, cp.GetFileExt(i), cp.GetExtraOpt(i))<0)
      {
        out.println("Error: Too many files in new project file!");
        out.flush();
        return;
      }

      if (cp.GetLng(i)==mainLng)
      {
        //Parse
        cl.Add("initiate transformations",  true, true);
        cl.Add("filein="+f1,                true, true);
        cl.Add("fileout="+f2,               true, true);
        cl.Add("fileouterror="+str3,        true, true);
        cl.Add("parse",                     true, true);
        cl.Add("terminate",                false, true);

        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Transformation Server!");
          out.flush();
          return;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Transformation Server!");
          out.flush();
          return;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Transformation Server!");
          out.flush();
          return;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          return;
        }
      }
    }

    ret=cp_new.Save();
    if (ret==-1)
    {
      out.println(cp_new.GetLastError());
      out.flush();
      return;
    }

    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void Tree(PrintWriter out, 
            CGlobal g,
            String str_dir, 
            String str_f_project, 
            String str_f_out,
            String str_f_diag_out)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;
    String str2=str_dir+str_f_out;
    String str3=str_dir+str_f_diag_out;
    String str4=str_dir+f_tmp;

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str2+"1");
      fdel.delete();
      fdel=new File(str2+"2");
      fdel.delete();
      fdel=new File(str3);
      fdel.delete();
      fdel=new File(str4);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(g.lng, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }
    if (cp.ParseLng()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    int mainLng=cp.GetLng(0);

    //Create CClient for Transformation Server
    if (g.lng.GetConfig(mainLng).GetParamStr(g.lngTSerHost).equals(""))
    {
      out.println("Error: Main language in project file is not supported by Transformation Server!");
      out.flush();
      return;
    }

    CClient cl=new CClient();
    if (cl.Init(g.lng.GetConfig(mainLng).GetParamStr(g.lngTSerHost),
                g.lng.GetConfig(mainLng).GetParamInt(g.lngTSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    BufferedWriter fout1=null;
    BufferedWriter fout2=null;

    try
    {
      fout1=new BufferedWriter(new FileWriter(str2+"1"));
      fout2=new BufferedWriter(new FileWriter(str2+"2"));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open tree file for writing!");
      out.flush();
      return;
    }

    //List of files
    boolean er=false;

    for (int i=1; (i<cp.GetNumFiles()) && !er; i++)
    {
      String f1=str_dir+cp.GetFileName(i)+cp.GetFileExt(i);

      if (cp.GetLng(i)==mainLng)
      {
        //Parse
        cl.Add("initiate transformations",  true, true);
        cl.Add("filein="+f1,                true, true);
        cl.Add("fileout="+str4,             true, true);
        cl.Add("fileouterror="+str3,        true, true);
        cl.Add("tree" ,                     true, true);
        cl.Add("terminate",                false, true);

        //Next line (delay) was added on 2002.10.07
        //to work properly with transformation server
        //if it restarts
        try
        {
          Thread.currentThread().sleep(100);
        }
        catch (InterruptedException e)
        {}
  
        ret=cl.Send();
        if (ret==-1)
        {
          out.println("Error: Can't find host with Transformation Server!");
          out.flush();
          er=true;
          break;
        }
        else if (ret==-2)
        {
          out.println("Error: Can't connect to Transformation Server!");
          out.flush();
          er=true;
          break;
        }
        else if (ret==-4)
        {
          out.println("Error: Can't close connection with Transformation Server!");
          out.flush();
          er=true;
          break;
        }
        else if (ret<0)
        {
          out.println(cl.GetLastError());
          out.flush();
          er=true;
          break;
        }

        try
        {
          fout1.write("file '"+cp.GetFileName(i)+cp.GetFileExt(i)+"'");
          fout1.newLine();
        }
        catch (Exception e2)
        {
          out.println("Error: Problems while writing tree file!");
          out.flush();
          er=true;
          break;
        }

        //Parsing tree file
        BufferedReader fin=null;

        try
        {
          fin=new BufferedReader(new FileReader(str4));
        }
        catch (IOException e1)
        {
          out.println("Error: Can't find temporal tree file!");
          out.flush();
          er=true;
          break;
        }

        String x="";
        String proc="";
        String x1="";
        int i1;
        try
        {
          while (((x=fin.readLine())!=null) && !er)
          {
            fout1.write(" "+x);
            fout1.newLine();

            boolean wr=false;

            x=x.trim();

            if ((x.length()>9) && (x.substring(0,9).toLowerCase().equals("procedure")))
            {
              proc=x.substring(9, x.length()).trim();
            }                        
            else if ((x.length()>3) && (!x.substring(0,3).toLowerCase().equals("end")))
            {
              i1=x.indexOf("'");
              x1="'"+x.substring(0, i1).trim()+"', "+x.substring(i1,x.length());
              wr=true;
            }

            if (wr)
            {
              fout2.write("'"+cp.GetFileName(i)+cp.GetFileExt(i)+"', ");
              fout2.write(proc);
              fout2.write(", ");
              fout2.write(x1);
              fout2.newLine();
            }
          }
        }
        catch (Exception e2)
        {
          try
          {
            fin.close();
          }
          catch (Exception e3)
          {}
          out.println("Error: Problems while reading/writing tree file!");
          out.flush();
          er=true;
          break;
        }

        try
        {
          fout1.write("end");
          fout1.newLine();
        }
        catch (Exception e2)
        {
          out.println("Error: Problems while writing tree file!");
          out.flush();
          er=true;
          break;
        }

        try
        {
          fin.close();
        }
        catch (Exception e5)
        {
          out.println("Error: Can't close temporal tree file!");
          out.flush();
          er=true;
          break;
        }
      }
    }

    try
    {
      fout1.close();
      fout2.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close tree file!");
      out.flush();
      return;
    }

    //deleting tmp file
    try
    {
      fdel=new File(str4);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    if (er) return;
   
    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void Compile(PrintWriter out, 
               CGlobal g,
               String str_dir, 
               String str_f_project, 
               String str_f_diag_out,
               String str_profile,
               String str_comp_mode,
               String str_no_check,
               String str_optim,
               String str_feedback)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(g.lng, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }
    if (cp.ParseLng()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    //Preparing names of files
    String str2=str_dir+cp.GetExe();
    String str3=str_dir+str_f_diag_out;
    String str4=str_dir+f_tmp;

    if (cp.GetLngNameOrig(0).equals("java")) str2+=".class"; 

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str2);
      fdel.delete();
      fdel=new File(str3);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    int mainLng=cp.GetLng(0);

    //Preparing big bat file
    BufferedWriter fbatout=null;

    try
    {
      fbatout=new BufferedWriter(new FileWriter(str4+".bat"));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open bat file!");
      out.flush();
      return; 
    }

    try
    {
//      if ((g.cfg.GetParamStr(g.cfgOS).toLowerCase().equals("windows")) && 
//          (str_dir.length()>1) && (str_dir.charAt(1)==':'))
//      {
//        fbatout.write(str_dir.substring(0,2));
//        fbatout.newLine();
//      }

      fbatout.write(g.cfg.GetParamStr(g.cfgCD)+" "+str_dir);
      fbatout.newLine();

      if (str_comp_mode.equals("0"))
      {
        //Deleting all object files
        fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" *"+g.cfg.GetParamStr(g.cfgObj));
        fbatout.newLine();
      }

      //adding files
      String x="";

      int Optim=0; 

      try
      {
        Optim=Integer.valueOf(str_optim).intValue();
      }
      catch (NumberFormatException nfe1)
      {
      }

      for (int i=1; i<cp.GetNumFiles(); i++)
      {
        x=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngCL)+" "+
          g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngCompOnly)+" ";

        if (str_feedback.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngFeedback1)+" ";
        }
        else if (str_feedback.equals("2"))
        {
          x+=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngFeedback2)+" ";
          if (!g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngFeedback3).equals(""))
          {
            x+=cp.GetExe()+
               g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngFeedback3)+" ";
          }
        }

        if (str_profile.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngProfComp)+" ";
        }

        if (str_comp_mode.equals("3"))
        {
          x+=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngCompAsm)+" ";
        }

        if (Optim!=0 && !str_feedback.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(i)).GetParamStr(g.lngOptim, Optim-1)+" ";
        }

        x+=cp.GetExtraOpt(i)+" "+cp.GetFileName(i)+cp.GetFileExt(i)+" "+
           g.cfg.GetParamStr(g.cfgApp2)+" "+str_f_diag_out;
        
        fbatout.write(x);
        fbatout.newLine();
      }

      if ((!str_comp_mode.equals("2")) && (!str_comp_mode.equals("3")))
      {
        //Linking
        x=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngCL)+" "+
          cp.GetExtraOpt(0)+" ";

        if (str_feedback.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngFeedback1)+" ";
        }
        else if (str_feedback.equals("2"))
        {
          x+=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngFeedback2)+" ";
          if (!g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngFeedback3).equals(""))
          {
            x+=cp.GetExe()+
               g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngFeedback3)+" ";
          }
        }

        if (Optim!=0 && !str_feedback.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngOptim, Optim-1)+" ";
        }

        String xx=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngEXE);
        if (xx.equals("")) 
        {
          xx=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngEXE1)+" ";
        }
 
        x+=xx+cp.GetExe()+" "+"*"+g.cfg.GetParamStr(g.cfgObj)+" ";

        if (str_profile.equals("1"))
        {
          x+=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngProfLink)+" ";
        }

        xx=g.lng.GetConfig(cp.GetLng(0)).GetParamStr(g.lngEXE2);
        if (!xx.equals("")) x+=xx+" ";
 
        x+=g.cfg.GetParamStr(g.cfgApp2)+" "+str_f_diag_out;

        fbatout.write(x);
        fbatout.newLine();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fbatout.close();
      }
      catch (Exception e3)
      {}
      out.println("Error: Problems while writing bat file!");
      out.flush();
      return;
    }

    try
    {
      fbatout.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close bat file!");
      out.flush();
      return; 
    }

    //Send request to run bat file
    CClient cl=new CClient();

    if (cl.Init(g.cfg.GetParamStr(g.cfgRSerHost),
                g.cfg.GetParamInt(g.cfgRSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    cl.Add("initiate run",                                     true, true);
    cl.Add("security_id="+g.cfg.GetParamStr(g.cfgSecurityID),  true, true);
    cl.Add("directory="+str_dir,                               true, true);
    cl.Add("file_bat="+f_tmp+".bat",                           true, true);
    cl.Add("go",                                               true, true);
    cl.Add("terminate",                                       false, true);

    ret=cl.Send();
    if (ret==-1)
    {
      out.println("Error: Can't find host with Run Server!");
      out.flush();
      return;
    }
    else if (ret==-2)
    {
      out.println("Error: Can't connect to Run Server!");
      out.flush();
      return;
    }
    else if (ret==-4)
    {
      out.println("Error: Can't close connection with Run Server!");
      out.flush();
      return;
    }
    else if (ret<0)
    {
      out.println(cl.GetLastError());
      out.flush();
      return;
    }

    if (str_no_check.equals("0"))
    {
      //Check that executable file was created
      FileInputStream fin1=null;

      try
      {
        fin1=new FileInputStream(str2);
      }
      catch (FileNotFoundException e1)
      {
        out.println("Error: Executable file was not created [Check log files]!");
        out.flush();
        return;
      }

      try
      {
        fin1.close();
      }
      catch (Exception e4)
      {}
    }

    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void Run(PrintWriter out, 
           CGlobal g,
           String str_dir, 
           String str_f_project, 
           String str_fo_time,
           String str_fo_comp,
           String str_fo_prof,
           String str_prof_type,
           String str_run_mode,
           String str_tool,
           String str_feedback)
  {
    int ret=0;
    int i=0;

    if (str_prof_type.equals("")) str_prof_type="0";

    //checking compatibility
    //if windows -> instead of type='1' (dcpi) do type='0' (normal profile)
    if (g.cfg.GetParamStr(g.cfgProfileID).equals("0"))
    {
      str_prof_type="0";
    }

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(g.lng, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }
    if (cp.ParseLng()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    if (str_tool.equals("1"))
    {
      cp.SetExe(cp.GetExe()+g.cfg.GetParamStr(g.cfgTool1b));
    }
    else if (str_tool.equals("2"))
    {
      cp.SetExe(cp.GetExe()+g.cfg.GetParamStr(g.cfgTool2b));
    }

    //Checking that executable file exist
    FileInputStream fin1=null;

    try
    {
      String strz=str_dir+cp.GetExe();
      if (cp.GetLngNameOrig(0).equals("java")) strz+=".class"; 
      fin1=new FileInputStream(strz);
    }
    catch (FileNotFoundException e1)
    {
      out.println("Error: Can't find executable file!");
      out.flush();
      return;
    }

    try
    {
      fin1.close();
    }
    catch (Exception e4)
    {}

    //Preparing names of files
    String str2=str_dir+str_fo_time;
    String str3=str_dir+str_fo_comp;
    String str4=str_dir+str_fo_prof;
    String str5=str_dir+f_tmp;

    //Delete output files if exist
    File fdel=null;
    try
    {
      fdel=new File(str2);
      fdel.delete();
      if (!str_fo_comp.equals(""))
      {
        fdel=new File(str3);
        fdel.delete();
      }
      if (!str_fo_prof.equals(""))
      {
        fdel=new File(str4);
        fdel.delete();
      }
    }
    catch (NullPointerException e1)
    {}

    //Preparing big bat file
    BufferedWriter fbatout=null;

    try
    {
      fbatout=new BufferedWriter(new FileWriter(str5+".bat"));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open bat file!");
      out.flush();
      return; 
    }

    try
    {
      if (str_run_mode.equals("0") || str_run_mode.equals("2"))
      {
        //copy preparation file
        for (i=0; i<cp.GetPrepMax(); i++)
        {
          fbatout.write(cp.GetPrep(i));
          fbatout.newLine();
        }
      }

      //delete application output files
      for (i=0; i<cp.GetCorMax(); i++)
      {
        fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetCor(i));
        fbatout.newLine();
      }

      if (str_run_mode.equals("0"))
      {
        //delete application std output files
        for (i=0; i<cp.GetCorMax(); i++)
        {
          fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetCor(i)+".std");
          fbatout.newLine();
        }
      }

      //exe file
      String x="";

      if (str_feedback.equals("1") && !g.cfg.GetParamStr(g.cfgFB1).equals(""))
      {
        x=g.cfg.GetParamStr(g.cfgFB1)+" "+cp.GetExe();
        if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
        {
          x+=g.cfg.GetParamStr(g.cfgAddExt);
        }

        fbatout.write(x);
        fbatout.newLine();
      }

      if (str_fo_prof.equals(""))
      {
        if (str_feedback.equals("1") && !g.cfg.GetParamStr(g.cfgFB8).equals(""))
        {
          x=g.cfg.GetParamStr(g.cfgFB8);

          fbatout.write(x);
          fbatout.newLine();
        }

        x="eos_time -fe ";
        if (cp.GetLngNameOrig(0).equals("java"))
        {
          x+="\"java "+cp.GetExe()+"\"";
        }
        else
        {
          x+="."+g.cfg.GetParamStr(g.cfgSeparator)+cp.GetExe();
        }

        if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
        {
          x+=g.cfg.GetParamStr(g.cfgAddExt);
        }

        if (str_feedback.equals("1") && !g.cfg.GetParamStr(g.cfgFB2).equals(""))
        {
          x+=g.cfg.GetParamStr(g.cfgFB2);
        }

        if (!cp.GetParam().equals("")) x+=" -fp "+cp.GetParamFile();
        x+=" -ft "+str_fo_time;

        fbatout.write(x);
        fbatout.newLine();

        if (str_feedback.equals("1") && !g.cfg.GetParamStr(g.cfgFB3).equals(""))
        {
          x=g.cfg.GetParamStr(g.cfgFB3)+" "+cp.GetExe();
          if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
          {
            x+=g.cfg.GetParamStr(g.cfgAddExt);
          }
          x+=" "+g.cfg.GetParamStr(g.cfgFB4)+" "+cp.GetExe();
          if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
          {
            x+=g.cfg.GetParamStr(g.cfgAddExt);
          }
          x+=g.cfg.GetParamStr(g.cfgFB5)+" "+cp.GetExe();
          if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
          {
            x+=g.cfg.GetParamStr(g.cfgAddExt);
          }
          x+=g.cfg.GetParamStr(g.cfgFB6)+" "+cp.GetExe();
          if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
          {
            x+=g.cfg.GetParamStr(g.cfgAddExt);
          }
          x+=g.cfg.GetParamStr(g.cfgFB7);

          fbatout.write(x);
          fbatout.newLine();
        }
      }
      else
      {
        if (str_prof_type.equals("0") || str_prof_type.equals(""))
        {
          String y=cp.GetExe();
          int j=y.lastIndexOf(".");
          if (j>=0) y=y.substring(0, j);

          //prepare without ext
          if (!g.cfg.GetParamStr(g.cfgPB1).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPB1)+" "+y);
            fbatout.newLine();
          }

          //prepare with ext
          if (!g.cfg.GetParamStr(g.cfgPB2).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPB2)+" "+cp.GetExe());
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgPE1).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPE1)+" "+cp.GetExe()
                          +" "+cp.GetParam());
            fbatout.newLine();
          }
          else
          {
            x="eos_time -fe ";
            if (cp.GetLngNameOrig(0).equals("java"))
            {
              x+="\"java "+cp.GetExe()+"\"";
            }
            else
            {
              x+="."+g.cfg.GetParamStr(g.cfgSeparator)+cp.GetExe();
              if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
              {
                x+=g.cfg.GetParamStr(g.cfgAddExt);
              }
            }
            if (!cp.GetParam().equals("")) x+=" -fp "+cp.GetParamFile();
            x+=" -ft "+str_fo_time;

            fbatout.write(x);
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgPE2).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPE2)+" "+y);
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgPE3).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPE3)+" "+cp.GetExe());
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgPA1).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPA1)+" "+y+" > "+f_tmp1);
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgPA2).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgPA2)+" > "+f_tmp1);
            fbatout.newLine();
          }
        }
        else if (str_prof_type.equals("1"))
        {
          if (!g.cfg.GetParamStr(g.cfgProf1a).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf1a));
            fbatout.newLine();
          }
          if (!g.cfg.GetParamStr(g.cfgProf1b).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf1b));
            fbatout.newLine();
          }

          x="eos_time -fe ";
          if (cp.GetLngNameOrig(0).equals("java"))
          {
            x+="\"java "+cp.GetExe()+"\"";
          }
          else
          {
            x+="."+g.cfg.GetParamStr(g.cfgSeparator)+cp.GetExe();
            if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
            {
              x+=g.cfg.GetParamStr(g.cfgAddExt);
            }
          }
          if (!cp.GetParam().equals("")) x+=" -fp "+cp.GetParamFile();
          x+=" -ft "+str_fo_time;

          fbatout.write(x);
          fbatout.newLine();

          if (!g.cfg.GetParamStr(g.cfgProf1c).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf1c));
            fbatout.newLine();
          }

          if (!g.cfg.GetParamStr(g.cfgProf1d).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf1d)+" "+cp.GetExe()+" > "+f_tmp1);
            fbatout.newLine();
          }
        }
        else if (str_prof_type.equals("2"))
        {
          fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetExe()+
                        g.cfg.GetParamStr(g.cfgProf2b));
          fbatout.newLine();

          if (!g.cfg.GetParamStr(g.cfgProf2a).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf2a)+" "+cp.GetExe());
            fbatout.newLine();
          }

          x="eos_time -fe "+"."+g.cfg.GetParamStr(g.cfgSeparator)+cp.GetExe()+
            g.cfg.GetParamStr(g.cfgProf2b);
          if (!g.cfg.GetParamStr(g.cfgAddExt).equals(""))
          {
            x+=g.cfg.GetParamStr(g.cfgAddExt);
          }
          if (!cp.GetParam().equals("")) x+=" -fp "+cp.GetParamFile();
          x+=" -ft "+str_fo_time;

          fbatout.write(x);
          fbatout.newLine();

          if (!g.cfg.GetParamStr(g.cfgProf2c).equals(""))
          {
            fbatout.write(g.cfg.GetParamStr(g.cfgProf2c)+" "+cp.GetExe()+" > "+f_tmp1);
            fbatout.newLine();
          }
        }
      }

      //Correctness files
      if (str_run_mode.equals("0"))
      {
        //copy preparation file
        for (i=0; i<cp.GetCorMax(); i++)
        {
          fbatout.write(g.cfg.GetParamStr(g.cfgCopy)+" "+
                        cp.GetCor(i)+" "+cp.GetCor(i)+".std");
          fbatout.newLine();
        }
      }
      else if (str_run_mode.equals("1"))
      {
        //check correctness
        for (i=0; i<cp.GetCorMax(); i++)
        {
          fbatout.write(g.cfg.GetParamStr(g.cfgDiff)+" "+
                        cp.GetCor(i)+" "+cp.GetCor(i)+".std"+" "+
                        g.cfg.GetParamStr(g.cfgApp1)+" "+f_tmp2);
          fbatout.newLine();
        }
      }
    }
    catch (Exception e2)
    {
      try
      {
        fbatout.close();
      }
      catch (Exception e3)
      {}
      out.println("Error: Problems while writing bat file!");
      out.flush();
      return;
    }

    try
    {
      fbatout.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close bat file!");
      out.flush();
      return; 
    }

    //Send request to run bat file
    CClient cl=new CClient();

    if (cl.Init(g.cfg.GetParamStr(g.cfgRSerHost),
                g.cfg.GetParamInt(g.cfgRSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    cl.Add("initiate run",                                     true, true);
    cl.Add("security_id="+g.cfg.GetParamStr(g.cfgSecurityID),  true, true);
    cl.Add("directory="+str_dir,                               true, true);
    cl.Add("file_bat="+f_tmp+".bat",                           true, true);
    cl.Add("go",                                               true, true);
    cl.Add("terminate",                                       false, true);

    ret=cl.Send();
    if (ret==-1)
    {
      out.println("Error: Can't find host with Run Server!");
      out.flush();
      return;
    }
    else if (ret==-2)
    {
      out.println("Error: Can't connect to Run Server!");
      out.flush();
      return;
    }
    else if (ret==-4)
    {
      out.println("Error: Can't close connection with Run Server!");
      out.flush();
      return;
    }
    else if (ret<0)
    {
      out.println(cl.GetLastError());
      out.flush();
      return;
    }

    //Check correctness
    if (str_run_mode.equals("1"))
    {
      //Check diff file
      int ch=0;

      try
      {
        fin1=new FileInputStream(str_dir+f_tmp2);
      }
      catch (FileNotFoundException e1)
      {
        out.println("Error: File with comparison results was not created!");
        out.flush();
        return;
      }

      try
      {
        ch=fin1.read();
      }
      catch (Exception e3)
      {
        try
        {
          fin1.close();
        }
        catch (Exception e4)
        {}
        out.println("Error: Problems while reading file with comparison results!");
        out.flush();
        return;
      }

      try
      {
        fin1.close();
      }
      catch (Exception e4)
      {
        out.println("Error: Can't close file with comparison results!");
        out.flush();
        return;
      }

      //Additional check if files exist
      for (i=0; i<cp.GetCorMax(); i++)
      {
        if (!CMisc.isFileExist(str_dir+cp.GetCor(i)))
        {
          ch=1;
          break;
        }
        if (!CMisc.isFileExist(str_dir+cp.GetCor(i)+".std"))
        {
          ch=1;
          break;
        }
      }

      //Saving results of comparison
      BufferedWriter fout=null;

      try
      {
        fout=new BufferedWriter(new FileWriter(str_dir+str_fo_comp));
      }
      catch (IOException e2)
      {
        out.println("Error: Can't open file with final comparison result!");
        out.flush();
        return;
      }

      try
      {
        if (ch==-1) fout.write("1");
        else fout.write("0");
        fout.newLine();
      }
      catch (IOException e2)
      {
        try
        {
          fout.close();
        }
        catch (Exception e4)
        {}
        
        out.println("Error: Problems while writing file with final comparison result!");
        out.flush();
        return;
      }

      try
      {
        fout.close();
      }
      catch (Exception e4)
      {
        out.println("Error: Can't close file with final comparison result!");
        out.flush();
        return;
      }
    }

    //Checking profiling information
    if (!str_fo_prof.equals(""))
    {
      //Parsing profiling information
      BufferedReader fprof1=null;
      BufferedWriter fprof2=null;
  
      try
      {
        fprof1=new BufferedReader(new FileReader(str_dir+f_tmp1));
        fprof2=new BufferedWriter(new FileWriter(str_dir+str_fo_prof));         
      }
      catch (IOException e1)
      {
        out.println("Error: Can't open files for profiling!");
        out.flush();
        return; 
      }

      int found=0;
      String x="";

      long period=-1;

      long clk=-1;

      try
      {
        clk=Long.valueOf(g.cfg.GetParamStr(g.cfgProf1e)).longValue();
      }
      catch (NumberFormatException nfe1)
      {}

      try
      {
        while (((x=fprof1.readLine())!=null) && found!=2)
        {
          x=CMisc.tab2space(x);
          String ProfParams[]=CMisc.parse_prof(8, x);

          if ((period==-1) && (ProfParams[0].equals("cycles")))
          {
            try
            {
              period=Long.valueOf(ProfParams[2]).longValue();
            }
            catch (NumberFormatException nfe1)
            {
            }
          }

          boolean found1=true;
          boolean ignore=false;

          double dd0=0;
          double dd1=0;
          double dd2=0;
          double dd3=0;
          double dd4=0;
          double dd5=0;

          if (str_prof_type.equals("0") || str_prof_type.equals(""))
          {
            if (g.cfg.GetParamStr(g.cfgProfileID).equals("0"))
            {
              try
              {
                dd0=Double.valueOf(ProfParams[0]).doubleValue();
                dd1=Double.valueOf(ProfParams[1]).doubleValue();
                dd2=Double.valueOf(ProfParams[2]).doubleValue();
                dd3=Double.valueOf(ProfParams[3]).doubleValue();
                dd4=Double.valueOf(ProfParams[4]).doubleValue();
              }
              catch (NumberFormatException nfe1)
              {
                found1=false;
              }
            }
            else if (g.cfg.GetParamStr(g.cfgProfileID).equals("1"))
            {
              try
              {
                dd0=Double.valueOf(ProfParams[0]).doubleValue();
                dd1=Double.valueOf(ProfParams[1]).doubleValue();
                dd2=Double.valueOf(ProfParams[2]).doubleValue();
              }
              catch (NumberFormatException nfe1)
              {
                found1=false;
              }
   
              try
              {
                dd3=Double.valueOf(ProfParams[3]).doubleValue();
                dd4=Double.valueOf(ProfParams[4]).doubleValue();
                dd5=Double.valueOf(ProfParams[5]).doubleValue();
              }
              catch (NumberFormatException nfe1)
              {
                ignore=true;
              }
            }
            else if (g.cfg.GetParamStr(g.cfgProfileID).equals("2"))
            {
              try
              {
                dd0=Double.valueOf(ProfParams[0]).doubleValue();
                dd1=Double.valueOf(ProfParams[1]).doubleValue();
                dd2=Double.valueOf(ProfParams[2]).doubleValue();
              }
              catch (NumberFormatException nfe1)
              {
               found1=false;
               }
            }          
          }
          else if (str_prof_type.equals("1"))
          {
            try
            {
              dd0=Double.valueOf(ProfParams[0]).doubleValue();
            }
            catch (NumberFormatException nfe1)
            {
              found1=false;
            }
          }
          else if (str_prof_type.equals("2"))
          {
            try
            {
              dd0=Double.valueOf(ProfParams[0]).doubleValue();
              dd1=Double.valueOf(ProfParams[3]).doubleValue();
              dd2=Double.valueOf(ProfParams[4]).doubleValue();
              dd3=Double.valueOf(ProfParams[5]).doubleValue();
              dd4=Double.valueOf(ProfParams[6]).doubleValue();
            }
            catch (NumberFormatException nfe1)
            {
              found1=false;
            }
          }

          if (!found1 && found==1)
          {
            found=2;
          }

          if (found1 && found==0)
          {
            found=1;
          }

          if (found==1 && !ignore)
          {
            try
            {
              if (str_prof_type.equals("0"))
              {
                if (g.cfg.GetParamStr(g.cfgProfileID).equals("0"))
                {
                  String y=CMisc.GetProcName0(ProfParams[5].trim());

                  boolean prnt=true;

                  if (((cp.GetLngNameOrig(0).equals("f77")) || 
                       (cp.GetLngNameOrig(0).equals("f90"))) && 
                      ((y.charAt(0)=='_') ||
                      (!CMisc.isAllUpperCase(y)))) prnt=false;
 
                  if (prnt)
                  {
                    fprof2.write("'");
                    fprof2.write(y);
                    fprof2.write("', '");
                    fprof2.write(String.valueOf(dd0/1000));
                    fprof2.write("', '");
                    fprof2.write(ProfParams[4].trim());
                    fprof2.write("', '");
                    fprof2.write(ProfParams[1].trim());
                    fprof2.write("'");
                    fprof2.newLine();
                  }
                }
                else if (g.cfg.GetParamStr(g.cfgProfileID).equals("1"))
                {
                  fprof2.write("'");
                  fprof2.write(CMisc.GetProcName1(ProfParams[6].trim()));
                  fprof2.write("', '");
                  fprof2.write(ProfParams[2].trim());
                  fprof2.write("', '");
                  fprof2.write(ProfParams[3].trim());
                  fprof2.write("', '");
                  fprof2.write(ProfParams[0].trim());
                  fprof2.write("'");
                  fprof2.newLine();
                }
                else if (g.cfg.GetParamStr(g.cfgProfileID).equals("2"))
                {
                  String y=ProfParams[4].trim();
                  
                  if ((cp.GetLngNameOrig(0).equals("f77")) || 
                      (cp.GetLngNameOrig(0).equals("f90")))
                  {
                    y=CMisc.GetProcName1(y);
                  }

                  fprof2.write("'");
                  fprof2.write(y);
                  fprof2.write("', '");
                  fprof2.write(ProfParams[1].trim());
                  fprof2.write("', '0', '");
                  fprof2.write(ProfParams[0].trim());
                  fprof2.write("'");
                  fprof2.newLine();
                }
                else
                {
                  String y=ProfParams[6].trim();
                
                  if ((cp.GetLngNameOrig(0).equals("f77")) || 
                      (cp.GetLngNameOrig(0).equals("f90")))
                  {
                    y=CMisc.GetProcName1(y);
                  }

                  fprof2.write("'");
                  fprof2.write(y);
                  fprof2.write("', '");
                  fprof2.write(ProfParams[3].trim());
                  fprof2.write("', '0', '");
                  fprof2.write("0");
                  fprof2.write("'");
                  fprof2.newLine();
                }
              }
              else if (str_prof_type.equals("1"))
              {
                String y=ProfParams[3].trim();
                  
                if ((cp.GetLngNameOrig(0).equals("f77")) || 
                    (cp.GetLngNameOrig(0).equals("f90")))
                {
                  y=CMisc.GetProcName1(y);
                }

                long l1=0;
                try
                {
                  l1=Long.valueOf(ProfParams[0]).longValue();
                }
                catch (NumberFormatException nfe1)
                {}

                double tm=(l1*period);
                double tm1=clk*1000000;
                tm=tm/tm1;

                fprof2.write("'");
                fprof2.write(y);
                fprof2.write("', '");
                fprof2.write(String.valueOf(tm));
                fprof2.write("', '");
                fprof2.write(String.valueOf(l1*period));
                fprof2.write("', '");
                fprof2.write(ProfParams[1].trim());
                fprof2.write("', '");
                fprof2.write("0");
                fprof2.write("', '");
                fprof2.write("0");
                fprof2.write("'");
                fprof2.newLine();
              }
              else if (str_prof_type.equals("2"))
              {
                String y=ProfParams[7].trim();
                  
                if ((cp.GetLngNameOrig(0).equals("f77")) || 
                    (cp.GetLngNameOrig(0).equals("f90")))
                {
                  y=CMisc.GetProcName1(y);
                }

                long l1=0;
                try
                {
                  l1=Long.valueOf(ProfParams[0]).longValue();
                }
                catch (NumberFormatException nfe1)
                {}

                double tm=l1;
                double tm1=clk*1000000;
                tm=tm/tm1;

                fprof2.write("'");
                fprof2.write(y);
                fprof2.write("', '");
                fprof2.write(String.valueOf(tm));
                fprof2.write("', '");
                fprof2.write(String.valueOf(l1));
                fprof2.write("', '");
                fprof2.write(ProfParams[1].trim());
                fprof2.write("', '");
                fprof2.write(ProfParams[3].trim());
                fprof2.write("', '");
                fprof2.write(ProfParams[5].trim());
                fprof2.write("'");
                fprof2.newLine();
              }
            }
            catch (Exception e2)
            {
              try
              {
                fprof1.close();
                fprof2.close();
              }
              catch (Exception e3)
              {}
              out.println("Error: Problems while writing file with independent profiling info!");
              out.flush();
              return;
            }
          }
        }
      }
      catch (Exception e2)
      {
        try
        {
          fprof1.close();
          fprof2.close();
        }
        catch (Exception e3)
        {}
        out.println("Error: Problems while reading/writing files for profiling!");
        out.flush();
        return;
      }

      try
      {
        fprof1.close();
        fprof2.close();
      }
      catch (Exception e5)
      {
        out.println("Error: Can't close files for profiling!");
        out.flush();
        return; 
      }

      if (found==0)
      {
        out.println("Error: Can't parse file with profiling info!");
        out.flush();
        return; 
      }   
    }

    //Deleting tmp files
    try
    {
//      fdel=new File(str_dir+f_tmp1);
//      fdel.delete();
      fdel=new File(str_dir+f_tmp2);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}

    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void RemoveObj(PrintWriter out, 
                 CGlobal g,
                 String str_dir, 
                 String str_f_project)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    String str4=str_dir+f_tmp;

    //Preparing bat file
    BufferedWriter fbatout=null;

    try
    {
      fbatout=new BufferedWriter(new FileWriter(str4+".bat"));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open bat file!");
      out.flush();
      return; 
    }

    try
    {
      fbatout.write(g.cfg.GetParamStr(g.cfgCD)+" "+str_dir);
      fbatout.newLine();

      for (int i=1; i<cp.GetNumFiles(); i++)
      {
        fbatout.write(g.cfg.GetParamStr(g.cfgDel));
        fbatout.write(" ");
        fbatout.write(cp.GetFileName(i));
        fbatout.write(g.cfg.GetParamStr(g.cfgObj));
        fbatout.newLine();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fbatout.close();
      }
      catch (Exception e3)
      {}
      out.println("Error: Problems while writing bat file!");
      out.flush();
      return;
    }

    try
    {
      fbatout.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close bat file!");
      out.flush();
      return; 
    }

    //Send request to run bat file
    CClient cl=new CClient();

    if (cl.Init(g.cfg.GetParamStr(g.cfgRSerHost),
                g.cfg.GetParamInt(g.cfgRSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    cl.Add("initiate run",                                     true, true);
    cl.Add("security_id="+g.cfg.GetParamStr(g.cfgSecurityID),  true, true);
    cl.Add("directory="+str_dir,                               true, true);
    cl.Add("file_bat="+f_tmp+".bat",                           true, true);
    cl.Add("go",                                               true, true);
    cl.Add("terminate",                                       false, true);

    ret=cl.Send();
    if (ret==-1)
    {
      out.println("Error: Can't find host with Run Server!");
      out.flush();
      return;
    }
    else if (ret==-2)
    {
      out.println("Error: Can't connect to Run Server!");
      out.flush();
      return;
    }
    else if (ret==-4)
    {
      out.println("Error: Can't close connection with Run Server!");
      out.flush();
      return;
    }
    else if (ret<0)
    {
      out.println(cl.GetLastError());
      out.flush();
      return;
    }

    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void RunTool(PrintWriter out, 
               CGlobal g,
               String str_dir, 
               String str_f_project,
               String str_tool,
               String str_prm,
               String str_prm1)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
    
    String str1=str_dir+str_f_project;

    //Dealing with project
    CProject cp = new CProject();

    if (cp.Init(null, str1, 4096)!=0)
    {
      out.println("Error (internal): Can't initialize object CProject!");
      out.flush();
      return;
    }

    if (cp.Parse()<0)
    {
      out.println(cp.GetLastError());
      out.flush();
      return;
    }

    String str4=str_dir+f_tmp;

    //Preparing bat file
    BufferedWriter fbatout=null;

    try
    {
      fbatout=new BufferedWriter(new FileWriter(str4+".bat"));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open bat file!");
      out.flush();
      return; 
    }

    try
    {
      fbatout.write(g.cfg.GetParamStr(g.cfgCD)+" "+str_dir);
      fbatout.newLine();

      if (str_tool.equals("1"))
      {
        fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetExe()+g.cfg.GetParamStr(g.cfgTool1b));
        fbatout.newLine();

        fbatout.write(g.cfg.GetParamStr(g.cfgTool1a)+" "+cp.GetExe()+" "+str_prm1);
        fbatout.newLine();
      }
      else if (str_tool.equals("2"))
      {
        fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetExe()+g.cfg.GetParamStr(g.cfgTool2b));
        fbatout.newLine();

        fbatout.write(g.cfg.GetParamStr(g.cfgTool2a)+" "+cp.GetExe());
        fbatout.newLine();
      }
      else if (str_tool.equals("4"))
      {
        fbatout.write(g.cfg.GetParamStr(g.cfgDel)+" "+cp.GetExe()+g.cfg.GetParamStr(g.cfgTool4b));
        fbatout.newLine();

        fbatout.write(g.cfg.GetParamStr(g.cfgTool4a)+" "+cp.GetExe()+" "+str_prm+" "+str_prm1);
        fbatout.newLine();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fbatout.close();
      }
      catch (Exception e3)
      {}
      out.println("Error: Problems while writing bat file!");
      out.flush();
      return;
    }

    try
    {
      fbatout.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close bat file!");
      out.flush();
      return; 
    }

    //Send request to run bat file
    CClient cl=new CClient();

    if (cl.Init(g.cfg.GetParamStr(g.cfgRSerHost),
                g.cfg.GetParamInt(g.cfgRSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    cl.Add("initiate run",                                     true, true);
    cl.Add("security_id="+g.cfg.GetParamStr(g.cfgSecurityID),  true, true);
    cl.Add("directory="+str_dir,                               true, true);
    cl.Add("file_bat="+f_tmp+".bat",                           true, true);
    cl.Add("go",                                               true, true);
    cl.Add("terminate",                                       false, true);

    ret=cl.Send();
    if (ret==-1)
    {
      out.println("Error: Can't find host with Run Server!");
      out.flush();
      return;
    }
    else if (ret==-2)
    {
      out.println("Error: Can't connect to Run Server!");
      out.flush();
      return;
    }
    else if (ret==-4)
    {
      out.println("Error: Can't close connection with Run Server!");
      out.flush();
      return;
    }
    else if (ret<0)
    {
      out.println(cl.GetLastError());
      out.flush();
      return;
    }

    out.println("Ok");
    out.flush();
  }

  //**************************************************************************
  void CDCF(PrintWriter out, 
            CGlobal g,
            String str_dir, 
            String str_prm,
            String str_prm1)
  {
    int ret=0;

    str_dir=CMisc.ProcessDir(str_dir, g.cfg.GetParamStr(g.cfgSeparator));
   
    //Preparing big bat file
    String str4=str_dir+f_tmp;

    BufferedWriter fbatout=null;
    BufferedReader fbatin=null;

    try
    {
      fbatout=new BufferedWriter(new FileWriter(str4+".bat"));
      fbatin=new BufferedReader(new FileReader(g.cfg.GetParamStr(g.cfgFileList)));
    }
    catch (IOException e1)
    {
      out.println("Error: Can't open bat files!");
      out.flush();
      return; 
    }

    try
    {
      fbatout.write(g.cfg.GetParamStr(g.cfgCD)+" "+str_dir);
      fbatout.newLine();
      fbatout.write(g.cfg.GetParamStr(g.cfgMD)+" "+str_prm);
      fbatout.newLine();
      fbatout.write(g.cfg.GetParamStr(g.cfgMD)+" "+str_prm+str_prm1);
      fbatout.newLine();

      String x="";
      try
      {
        while ((x=fbatin.readLine())!=null)
        {
          fbatout.write(g.cfg.GetParamStr(g.cfgCopy)+" "+x+" "+str_prm+str_prm1);
          fbatout.newLine();
        }
      }
      catch (Exception e2)
      {
        try
        {
          fbatout.close();
          fbatin.close();
        }
        catch (Exception e3)
        {}
        out.println("Error: Problems while reading/writing bat files!");
        out.flush();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fbatout.close();
        fbatin.close();
      }
      catch (Exception e3)
      {}
      out.println("Error: Problems while reading/writing bat files!");
      out.flush();
      return;
    }

    try
    {
      fbatout.close();
      fbatin.close();
    }
    catch (Exception e5)
    {
      out.println("Error: Can't close bat file!");
      out.flush();
      return; 
    }

    //Send request to run bat file
    CClient cl=new CClient();

    if (cl.Init(g.cfg.GetParamStr(g.cfgRSerHost),
                g.cfg.GetParamInt(g.cfgRSerPort),
                16)!=0)
    {
      out.println("Error (internal): Can't initialize object CClient!");
      out.flush();
      return;
    }

    cl.Add("initiate run",                                     true, true);
    cl.Add("security_id="+g.cfg.GetParamStr(g.cfgSecurityID),  true, true);
    cl.Add("directory="+str_dir,                               true, true);
    cl.Add("file_bat="+f_tmp+".bat",                           true, true);
    cl.Add("go",                                               true, true);
    cl.Add("terminate",                                       false, true);

    ret=cl.Send();
    if (ret==-1)
    {
      out.println("Error: Can't find host with Run Server!");
      out.flush();
      return;
    }
    else if (ret==-2)
    {
      out.println("Error: Can't connect to Run Server!");
      out.flush();
      return;
    }
    else if (ret==-4)
    {
      out.println("Error: Can't close connection with Run Server!");
      out.flush();
      return;
    }
    else if (ret<0)
    {
      out.println(cl.GetLastError());
      out.flush();
      return;
    }

    out.println("Ok");
    out.flush();
  }
}
