//
// Process Instructions
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

#include "param.h"
#include "siparse.h"
#include "global.h"
#include "misc.h"

//Sockets
#ifdef W32
#  include <winsock2.h>
#  include <wsipx.h>
#else
#  include <sys/socket.h>
#  include <sys/types.h>
#  include <memory.h>
#  include <netinet/in.h>
#endif

#include <string.h>

#include <iostream.h>
#include <fstream.h>

#include <stdio.h>
#include <stdlib.h> //for system

void ProcessInstructions(int sockfd, CGlobal* g, CSIParse* s)
{
  #define maxstrlen 1024
  
  char str[maxstrlen]="";
  char buf[maxstrlen]="";
  char buf1[maxstrlen]="";  
  char* xerr=new char[1024];
  int misc;

  //Names
  char fileBat[]="run.bat";

  //Parameters
  int initiate=0;
  char str_dir[maxstrlen]="";
  char str_bat[maxstrlen]="";
#ifdef W32
  char str_sep[2]="\\";
#else
  char str_sep[2]="/";
#endif

  for (;;)
  {
    buf[0]=0;
    buf1[0]=0;
    int p=0;
    int esc=0;
    int nread=0;

    while (((nread=recv(sockfd, buf1, 1, 0))>0) && (buf1[0]!='\r'))
    {
      //Checking for characters
      if ((buf1[0]>31) && (buf1[0]<127))
      {
        if (esc==0)
        {
          buf[p]=buf1[0];
          p++;

          //send echo  
          buf1[1]=0;
          int misc=send(sockfd, buf1, 1, 0);
  
          if (misc<0)
          {
            cerr << "Error: Can't send data to client!\n";
            return;
          }
        }
        else if (esc==1) esc=2;
        else esc=0;
      }
      else if (buf1[0]==27)
      {
        //Escape
        if (esc==0) esc=1;
        else if (esc==1) esc=0;
      }
      else if (buf1[0]=='\n')
      {
        //Ignore character
        if (esc==1) esc=2;
        else esc=0;
      }
      else if (buf1[0]=='\b')
      {
        //Backspace
        if (esc==0)
        {
          if (p>0) 
          {
            p--;
            buf[p]=0;

            //send echo  
            buf1[0]='\b';
            buf1[1]=' ';
            buf1[2]='\b';
            buf1[3]=0;
            int misc=send(sockfd, buf1, 3, 0);
  
            if (misc<0)
            {
              cerr << "Error: Can't send data to client!\n";
              return;
            }
          }
        }
        else if (esc==1) esc=2;
        else esc=0;
      }
    }

    buf[p]=0;

    if (nread<0)
    {
//      cerr << "Error: Can't receive data from client!\n";
//      Probably connection was closed
      return;
    }

    if (nread==0)
    {
//      cout << "Session complete!\n";
      return;
    }

    if (SendMes(sockfd, "", 0, 1024)<0) 
    {
      cerr << "Error: Can't send data to client!\n";
      return;
    }

    strcat(str, buf);

    //process str ********************************************************
    misc=s->Parse(str);

    if (misc<0)
    {
      switch (misc)
      {
        case -1:
        {
          strcpy(xerr, "Warning: Line is empty!");
          break;
        }
        case -2:
        {
          strcpy(xerr, "Error: Unknown instruction!");
          break;
        }
        case -3:
        {
          strcpy(xerr, "Error: This instruction needs parameter!");
          break;
        }
        case -4:
        {
          strcpy(xerr, "Error: This instruction doesn't need parameters!");
          break;
        }
        case -5:
        {
          strcpy(xerr, "Error: Wrong separator!");
          break;
        }
        default:
        {
          strcpy(xerr, "Error: Can't parse!");
          break;
        }
      }

      if (SendMes(sockfd, xerr, 0, 1024)<0) return;
    }
    else
    {
      //Process instructions
      switch(misc)
      {
        case instrInitiate:
        {
          if (strcmp(s->GetParamStrLwr(instrInitiate), "run")==0) 
          {
            initiate=1;
            strcpy(str_dir, "");
            strcpy(str_bat, "");
            if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          }
          else
          {
            if (SendMes(sockfd, "Error: Unknown parameter for 'initiate'!", 0, 1024)<0) return;
          }
          break;
        }
        case instrTerminate:
        {
          SendMes(sockfd, "Ok [Session finished]", 0, 1024);
          return; //Finish
          break;
        }
        case instrGo:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }

          //Check security ID
          if (strcmp(g->clp->GetParamStr(paramSecurity_ID), 
             s->GetParamStr(instrSecurity_ID))!=0)
          {
            if (SendMes(sockfd, "Error: Security ID is wrong!", 0, 1024)<0) return; 
            return;
          }

          //Run ...
          if ((strcmp(str_dir, "")==0) ||
	      (strcmp(str_bat, "")==0))
	  {
            if (SendMes(sockfd, "Error: One of parameters is empty!", 0, 1024)<0) return; 
            break;
	  }

          //check for the main bat file
          strcpy(str, str_dir);
	  if (str[strlen(str)-1]!=str_sep[0]) strcat(str, str_sep);
	  strcat(str, str_bat);
	  
#ifdef W32
          ifstream fin(str, ios::nocreate);
#else
          ifstream fin(str);
#endif
	  
	  if (!fin)
	  {
            if (SendMes(sockfd, "Error: Can't find bat file!", 0, 1024)<0) return; 
	  }
          else
	  {
            fin.close();

            //preparing temproral bat file to run
            ofstream fout(fileBat);
	  
            if (!fout)
  	    {
              if (SendMes(sockfd, "Error: Internal - Can't open bat file!", 0, 1024)<0) return; 
              return;
  	    }

#ifdef W32
            //checking if there is a disk name
            bool noDiskName=true;
            char DiskName=0;
            for (int q=0; q<strlen(str_dir); q++)
            {
              if (str_dir[q]!=' ')
              {
                if (((q+1)<strlen(str_dir)) && str_dir[q+1]==':')
                {
                  noDiskName=false;
                  DiskName=str_dir[q];
                }
                break;
              }
            }

            if (!noDiskName)
            {
              fout << str_dir[q] << ":\n";
            }    

            fout << "cd " << str_dir << "\n";
            fout << str_bat << "\n";
	  
            fout.close();

            strcpy(str, fileBat);

            system(str);

            strcpy(str, "del ");
            strcat(str, fileBat);

            system(str);
#else
            fout << "cd " << str_dir << "\n";
	    fout << "chmod -f 755 " << str_bat << "\n";
 	    fout << "./" << str_bat << "\n";
	  
	    fout.close();

            strcpy(str, "chmod -f 755 ");
	    strcat(str, fileBat);
	  
 	    system(str);

            strcpy(str, "./");
  	    strcat(str, fileBat);

            system(str);

            strcpy(str, "rm ");
            strcat(str, fileBat);

            system(str);
#endif

            if (SendMes(sockfd, "Ok", 0, 1024)<0) return; 
          }
	  
          //clear strings, but don't clear initiate
          strcpy(str_dir, "");
          strcpy(str_bat, "");

          break;
        }
        case instrDirectory:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(str_dir, s->GetParamStr(instrDirectory));
          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrSecurity_ID:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          if (SendMes(sockfd, "Ok", 0, 1024)<0) return;
          break;
        }
        case instrFileBat:
        {
          if (initiate==0)
          {
            if (SendMes(sockfd, "Error: Use 'initiate' first!", 0, 1024)<0) return; 
            break;
          }
          strcpy(str_bat, s->GetParamStr(instrFileBat));
          if (SendMes(sockfd, "Ok", 0, 1024)<0) return; 
          break;
        }
        default:
        {
          break;
        }
      }
    }

    strcpy(str,"");
  }
}
