//
// Process Messages
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

#include "param.h"
#include "siparse.h"
#include "pi.h"
#include "global.h"
#include "misc.h"

#ifdef W32
#  include <winsock2.h>
#  include <wsipx.h>
#else
#  include <sys/types.h>
#  include <sys/socket.h>
#endif

#include <signal.h>
#include <setjmp.h>

#include <string.h>

#include <iostream.h>

jmp_buf error_recovery_context;
char global_error_buffer[1024];

#ifndef W32
void sig_bus(int);
#endif
void sig_segv(int);

void ProcessMessages (int sockfd, CGlobal* g)
{
  int misc;

  //Set Signals
  if (g->clp->GetParamLong(paramCatch)==1)
  {
#ifndef W32
    signal(SIGBUS, sig_bus);
#endif
    signal(SIGSEGV, sig_segv);
  
    if (setjmp(error_recovery_context))
    {
      SendMes(sockfd, global_error_buffer, 0, 1024);
      return;
    }
  }

  //Initialize instructions (need to do it in each process!)
  CSIParse* sip = new CSIParse();
     
  if (sip->Init(10, 32, 256, 1)!=0)
  //             |   |    |  |
  //             |   |    |  |------- Case Sensitive (0) or not (1)
  //             |   |    |
  //             |   |    |---------- Max Length of Parameter
  //             |   |               
  //             |   |--------------- Max Length of Option
  //             |
  //             |------------------- Max Number of Options
  {
    cout << "\nError: Can't initialize object CSIParse!\n";
    return;
  }

  misc=0;
  
  misc =sip->Add("initiate",    1, ' ', "");   //Param 0
  misc+=sip->Add("security_id", 1, '=', "");   //Param 1 
  misc+=sip->Add("terminate",   0, 0,   "");   //Param 2
  misc+=sip->Add("go",          0, 0,   "");   //Param 3
  misc+=sip->Add("directory",   1, '=', "");   //Param 4 
  misc+=sip->Add("file_bat",    1, '=', "");   //Param 5

  if (misc!=0)
  {
    cout << "\nError: Internal, while initializing CSIParse!\n";
    return;
  }
  
  //Process Instructions
  ProcessInstructions(sockfd, g, sip);
}

#ifndef W32
void sig_bus(int signo)
{
  strcpy(global_error_buffer, "Error: Process died!\n");
  cerr << "Error: Child process died!\n";
  longjmp(error_recovery_context, signo);
}
#endif

void sig_segv(int signo)
{
  strcpy(global_error_buffer, "Error: Process died!\n");
  cerr << "Error: Child process died!\n";
  longjmp(error_recovery_context, signo);
}
